#include <swift/atomic.h>

int swift_atomic_get(swift_atomic_t *atomic)
{
	return atomic->value;
}

void swift_atomic_set(swift_atomic_t *atomic, int value)
{
	atomic->value = value;
}

void swift_atomic_add(swift_atomic_t *atomic, int value)
{
	__sync_add_and_fetch(&atomic->value, value);
}

void swift_atomic_sub(swift_atomic_t *atomic, int value)
{
	__sync_sub_and_fetch(&atomic->value, value);
}

void swift_atomic_inc(swift_atomic_t *atomic)
{
	__sync_add_and_fetch(&atomic->value, 1);
}

void swift_atomic_dec(swift_atomic_t *atomic)
{
	__sync_sub_and_fetch(&atomic->value, 1);
}

int swift_atomic_add_ret(swift_atomic_t *atomic, int value)
{
	return __sync_add_and_fetch(&atomic->value, value);
}

int swift_atomic_sub_ret(swift_atomic_t *atomic, int subtrahend)
{
	return __sync_sub_and_fetch(&atomic->value, subtrahend);
}

int swift_atomic_inc_ret(swift_atomic_t *atomic)
{
	return __sync_add_and_fetch(&atomic->value, 1);
}

int swift_atomic_dec_ret(swift_atomic_t *atomic)
{
	return __sync_sub_and_fetch(&atomic->value, 1);
}

bool swift_atomic_inc_test(swift_atomic_t *atomic)
{
	return (__sync_add_and_fetch(&atomic->value, 1) == 0);
}

bool swift_atomic_dec_test(swift_atomic_t *atomic)
{
	return (__sync_sub_and_fetch(&atomic->value, 1) == 0);
}

bool swift_atomic_add_unless(swift_atomic_t *atomic, int addend, int comparand)
{
	int value = swift_atomic_get(atomic);

	if (value == comparand)
		return false;

	do {
		value = swift_atomic_get(atomic);

		if (value == comparand)
			return false;
	} while (!swift_atomic_cmp_swap(atomic, value, value + addend));

	return true;
}

int swift_atomic_swap(swift_atomic_t *atomic, int value)
{
	int old_value;

	do {
		old_value = atomic->value;
	} while (!__sync_bool_compare_and_swap(&atomic->value, old_value,
		value));

	return old_value;
}

bool swift_atomic_cmp_swap(swift_atomic_t *atomic, int old_value, int value)
{
	return __sync_bool_compare_and_swap(&atomic->value, old_value, value);
}
