#include <swift/data.h>

bool swift_init_list(swift_list_t *list)
{
	if (!list)
		return false;

	list->prev = list->next = list;

	return true;
}

void swift_term_list(swift_list_t *list)
{
	if (!list)
		return;

	list->prev = list->next = list;
}

swift_list_t *swift_new_list(void)
{
	swift_list_t *list;

	list = calloc(1, sizeof(swift_list_t));

	if (!list)
		return NULL;

	if (!swift_init_list(list))
		goto err_free_list;

	return list;

err_free_list:
	SWIFT_FREE(list);
	return NULL;
}

void swift_del_list(swift_list_t *list)
{
	if (!list)
		return;

	swift_term_list(list);
	SWIFT_FREE(list);
}

bool swift_list_is_empty(swift_list_t *list)
{
	if (!list || list->next == list || list->prev == list || !list->prev ||
		!list->next)
		return true;

	return false;
}

swift_list_iter_t swift_list_first(swift_list_t *list)
{
	if (!list || swift_list_is_empty(list))
		return SWIFT_LIST_ITER(NULL, NULL, NULL);

	return SWIFT_LIST_ITER(list, list->next,
		list->next && (list->next->next != list) ? list->next->next : NULL);
}

swift_list_iter_t swift_list_last(swift_list_t *list)
{
	if (!list || swift_list_is_empty(list))
		return SWIFT_LIST_ITER(NULL, NULL, NULL);

	return  SWIFT_LIST_ITER(list, list->prev,
		list->prev && (list->prev->prev != list) ? list->prev->prev : NULL);
}

bool swift_list_next(swift_list_iter_t *iter)
{
	if (!iter || !iter->node)
		return false;

	iter->node = iter->next;
	iter->next =
		iter->next && (iter->next->next != iter->list) ? iter->next->next :
			NULL;

	return !!iter->node;
}

bool swift_list_prev(swift_list_iter_t *iter)
{
	if (!iter || !iter->node)
		return false;

	iter->node = iter->next;
	iter->next =
		iter->next && (iter->next->prev != iter->list) ? iter->next->prev :
			NULL;

	return !!iter->node;
}

bool swift_list_insert_before(swift_list_t *list,
	swift_list_t *entry)
{
	if (!list || !entry)
		return false;

	entry->next = list;
	entry->prev = list->prev;
	list->prev->next = entry;
	list->prev = entry;

	return true;
}

bool swift_list_insert_after(swift_list_t *list,
	swift_list_t *entry)
{
	if (!list || !entry)
		return false;

	entry->next = list->next;
	entry->prev = list;
	list->next->prev = entry;
	list->next = entry;

	return true;
}

bool swift_list_remove(swift_list_t *list, swift_list_t *entry)
{
	if (!list || !entry)
		return false;

	entry->prev->next = entry->next;
	entry->next->prev = entry->prev;
	entry->next = entry;
	entry->prev = entry;

	return true;
}

bool swift_list_push(swift_list_t *list, swift_list_t *entry)
{
	return swift_list_insert_before(list, entry);
}

bool swift_list_unshift(swift_list_t *list, swift_list_t *entry)
{
	return swift_list_insert_after(list, entry);
}

swift_list_t *swift_list_pop(swift_list_t *list)
{
	swift_list_t *entry;

	if (!list || swift_list_is_empty(list))
		return NULL;

	entry = list->prev;
	swift_list_remove(list, entry);

	return entry;
}

swift_list_t *swift_list_shift(swift_list_t *list)
{
	swift_list_t *entry;

	if (!list || swift_list_is_empty(list))
		return NULL;

	entry = list->next;
	swift_list_remove(list, entry);

	return entry;
}
