#include <swift/dylib.h>
#include <swift/unicode.h>

#define WIN32_MEAN_AND_LEAN
#include <windows.h>

struct swift_lib_t {
	HMODULE handle;
};

swift_lib_t *swift_open_lib(const char *path_)
{
	swift_lib_t *lib;
	uint16_t *path;

	/* Allocate a lib object. */
	lib = malloc(sizeof(swift_lib_t));

	if (!lib)
		return NULL;

	if (!path_) {
		/* Return this process as lib. */
		lib->handle = GetModuleHandle(NULL);

		return lib;
	}

	/* Convert the path to native format. */
	path = swift_new_conv_str("utf-16", "utf-8", path_, SIZE_MAX);

	if (!path)
		goto err_free_lib;

	/* Open the library. */
	lib->handle = LoadLibrary(path);
	SWIFT_FREE(path);

	if (!lib->handle)
		goto err_free_lib;

	return lib;

err_free_lib:
	SWIFT_FREE(lib);
	return NULL;
}

void swift_close_lib(swift_lib_t *lib)
{
	if (!lib)
		return;

	/* Close the lib. */
	FreeLibrary(lib->handle);

	/* Free the lib object. */
	SWIFT_FREE(lib);
}

swift_fn_t swift_get_code_sym(swift_lib_t *lib, const char *sym_)
{
	swift_fn_t code_addr;
	char *sym;

	if (!lib)
		return NULL;

	/* Convert the symbol to native format. */
	sym = swift_new_conv_str("ascii", "utf-8", sym_, SIZE_MAX);

	if (!sym)
		return NULL;

	/* Search for the sym within the lib. */
	code_addr = (swift_fn_t)GetProcAddress(lib->handle, sym);
	SWIFT_FREE(sym);

	if (!code_addr)
		return NULL;

	return code_addr;
}

void *swift_get_data_sym(swift_lib_t *lib, const char *sym_)
{
	void *data_addr;
	char *sym;

	if (!lib)
		return NULL;

	/* Convert the symbol to native format. */
	sym = swift_new_conv_str("ascii", "utf-8", sym_, SIZE_MAX);

	if (!sym)
		return NULL;

	/* Search for the sym within the lib. */
	data_addr = GetProcAddress(lib->handle, sym);
	SWIFT_FREE(sym);

	if (!data_addr)
		return NULL;

	return data_addr;
}
