#include <swift/dylib.h>

#include <dlfcn.h>

struct swift_lib_t {
	void *handle;
};

swift_lib_t *swift_open_lib(const char *path)
{
	swift_lib_t *lib;

	/* Allocate a library object. */
	lib = malloc(sizeof(swift_lib_t));

	if (!lib)
		return NULL;

	/* Open the library. */
	lib->handle = dlopen(path, RTLD_LAZY);

	if (!lib->handle)
		goto err_free_lib;

	return lib;

err_free_lib:
	SWIFT_FREE(lib);
	return NULL;
}

void swift_close_lib(swift_lib_t *lib)
{
	if (!lib)
		return;

	/* Close the library. */
	dlclose(lib->handle);

	/* Free the library object. */
	SWIFT_FREE(lib);
}

swift_fn_t swift_get_code_sym(swift_lib_t *lib, const char *sym)
{
	if (!lib)
		return NULL;

	/* Search for the sym within the library. */
	return (swift_fn_t)dlsym(lib->handle, sym);
}

void *swift_get_data_sym(swift_lib_t *lib, const char *sym)
{
	if (!lib)
		return NULL;

	/* Search for the sym within the library. */
	return dlsym(lib->handle, sym);
}
