#include <swift/atomic.h>
#include <swift/data.h>
#include <swift/event.h>
#include <swift/thread.h>

#include "event.h"
#include "loop.h"
#include "msg.h"
#include "source.h"
#include "timer.h"

swift_ev_ctx_t *ev_ctx;

static swift_ref_t ref = SWIFT_REF(0);

void swift_do_term_events(swift_ref_t *ref)
{
	swift_htable_iter_t iter;
	swift_ev_loop_t *loop;
	swift_ev_src_t *src;

	if (!ev_ctx)
		return;

	swift_mutex_lock(ev_ctx->lock);

	swift_term_timers();
	swift_term_ev_msgs();

	for (iter = swift_htable_first(ev_ctx->loops); iter.data;
		swift_htable_next(&iter)) {
		loop = *(swift_ev_loop_t **)iter.data;
		swift_del_ev_loop(loop);
	}

	for (iter = swift_htable_first(ev_ctx->srcs); iter.data;
		swift_htable_next(&iter)) {
		src = *(swift_ev_src_t **)iter.data;
		swift_del_ev_src(src);
	}

	swift_del_htable(ev_ctx->loops);
	swift_del_htable(ev_ctx->srcs);
	swift_del_mutex(ev_ctx->lock);
	SWIFT_FREE(ev_ctx);
	swift_term_threads();
}

bool swift_init_events(void)
{
	swift_htable_iter_t iter;
	swift_ev_loop_t *loop;

	if (swift_acquire_ref_unless_zero(&ref))
		return true;

	if (!swift_init_threads())
		return false;

	ev_ctx = malloc(sizeof(swift_ev_ctx_t));

	if (!ev_ctx)
		goto err_term_threads;

	ev_ctx->lock = swift_new_mutex(SWIFT_MUTEX_RECURSIVE);

	if (!ev_ctx->lock)
		goto err_free_ev_ctx;

	ev_ctx->srcs = swift_new_htable(swift_hash_ev_src,
		swift_eq_ev_srcs, sizeof(swift_ev_src_t *));

	if (!ev_ctx->srcs)
		goto err_del_lock;

	ev_ctx->loops = swift_new_htable(swift_hash_ev_loop,
		swift_eq_ev_loops, sizeof(swift_ev_loop_t *));

	if (!ev_ctx->loops)
		goto err_del_srcs;

	if (!swift_init_main_ev_loop())
		goto err_del_loops;

	if (!swift_init_ev_msgs())
		goto err_term_ev_loops;

	if (!swift_init_timers())
		goto err_term_ev_msgs;

	swift_acquire_ref(&ref);
	ev_ctx->is_running = false;

	return true;

err_term_ev_msgs:
	swift_term_ev_msgs();
err_term_ev_loops:
	for (iter = swift_htable_first(ev_ctx->loops); iter.data;
		swift_htable_next(&iter)) {
		loop = *(swift_ev_loop_t **)iter.data;
		swift_del_ev_loop(loop);
	}
err_del_loops:
	swift_del_htable(ev_ctx->loops);
err_del_srcs:
	swift_del_htable(ev_ctx->srcs);
err_del_lock:
	swift_del_mutex(ev_ctx->lock);
err_free_ev_ctx:
	SWIFT_FREE(ev_ctx);
err_term_threads:
	swift_term_threads();
	return false;
}

void swift_term_events(void)
{
	swift_release_ref(&ref, swift_do_term_events);
}
