#include <swift/data.h>
#include <swift/event.h>
#include <swift/thread.h>

#include "event.h"
#include "loop.h"
#include "msg.h"

extern swift_ev_ctx_t *ev_ctx;

static swift_ev_msg_t *swift_new_ev_msg(swift_run_ev_msg_fn_t run_msg,
	void *user_data)
{
	swift_ev_msg_t *msg;

	if (!run_msg)
		return NULL;

	msg = calloc(1, sizeof(swift_ev_msg_t));

	if (!msg)
		return NULL;

	msg->run_msg = run_msg;
	msg->user_data = user_data;
	swift_init_list(&msg->msgs);

	return msg;
}

static void swift_del_ev_msgs(swift_ev_loop_t *loop)
{
	swift_list_t *node = NULL;
	swift_ev_msg_t *msg;

	if (!loop)
		return;

	while ((node = swift_list_pop(&loop->msgs))) {
		msg = SWIFT_CONTAINER_OF(node, swift_ev_msg_t, msgs);
		SWIFT_FREE(msg);
	}
}

static void swift_poll_ev_msgs(void *user_data)
{
	swift_list_t *node = NULL;
	swift_ev_loop_t *loop = swift_get_cur_ev_loop();
	swift_ev_msg_t *msg;

	if (!loop)
		return;

	while ((node = swift_list_pop(&loop->msgs))) {
		msg = SWIFT_CONTAINER_OF(node, swift_ev_msg_t, msgs);
		msg->run_msg(msg->user_data);
		SWIFT_FREE(msg);
	}
}

bool swift_init_ev_msgs(void)
{
	return swift_add_ev_src(swift_poll_ev_msgs, NULL, SWIFT_MAIN_THREAD |
		SWIFT_WORKER_THREAD);
}

void swift_term_ev_msgs(void)
{
	swift_htable_iter_t iter;
	swift_ev_loop_t *loop;

	swift_mutex_lock(ev_ctx->lock);

	for (iter = swift_htable_first(ev_ctx->loops); iter.data;
		swift_htable_next(&iter)) {
		loop = *(swift_ev_loop_t **)iter.data;

		swift_del_ev_msgs(loop);
	}

	swift_remove_ev_src(swift_poll_ev_msgs);
	swift_mutex_unlock(ev_ctx->lock);
}

bool swift_post_ev_msg(swift_thread_id_t thread_id,
	swift_run_ev_msg_fn_t run_msg, void *user_data)
{
	swift_ev_loop_t *loop;
	swift_ev_msg_t *msg;

	if (!run_msg)
		return false;

	if (!(loop = swift_get_ev_loop(thread_id)))
		return false;

	if (!(msg = swift_new_ev_msg(run_msg, user_data)))
		return false;

	swift_list_push(&loop->msgs, &msg->msgs);

	return true;
}
