#include <swift/atomic.h>
#include <swift/data.h>
#include <swift/fs.h>
#include <swift/string.h>

static swift_ref_t ref = SWIFT_REF(0);

static void swift_do_term_fs(swift_ref_t *ref)
{
	swift_term_unicode();
}

bool swift_init_fs(void)
{
	if (swift_acquire_ref_unless_zero(&ref))
		return true;

	if (!swift_init_unicode())
		return false;

	swift_acquire_ref(&ref);

	return true;
}

void swift_term_fs(void)
{
	swift_release_ref(&ref, swift_do_term_fs);
}

bool swift_new_dir_r(const char *cpath)
{
	swift_array_iter_t iter;
	swift_array_t *nodes;
	swift_str_t *path, *node;

	if (!cpath)
		return false;

	path = swift_new_cstr(cpath);

	if (!path)
		return false;

	nodes = swift_split_cstr(path, "/");

	if (!nodes)
		goto err_del_path;

	swift_clear_str(path);

	for (iter = swift_array_first(nodes); iter.data;
		swift_array_next(&iter)) {
		node = *(swift_str_t **)iter.data;

		swift_append_str(path, node);
		swift_append_uchar(path, '/');

		if (swift_check_path(path->data))
			continue;

		if (!swift_new_dir(path->data))
			goto err_del_nodes;
	}

	swift_del_str_array(nodes);
	swift_del_str(path);

	return true;

err_del_nodes:
	swift_del_str_array(nodes);
err_del_path:
	swift_del_str(path);
	return false;
}

static bool swift_do_del_dir_r(swift_str_t *path)
{
	swift_dir_t *dir;
	const char *node;
	size_t pos;

	if (!swift_is_dir(path->data)) {
		swift_del_file(path->data);
		return true;
	}

	dir = swift_open_dir(path->data);

	if (!dir)
		return false;

	while ((node = swift_read_dir(dir))) {
		if (strcmp(node, ".") == 0 || strcmp(node, "..") == 0)
			continue;

		swift_append_uchar(path, '/');
		swift_append_cstr(path, node);
		swift_do_del_dir_r(path);
		pos = swift_rfind_cstr(path, SIZE_MAX, "/");
		swift_remove_str(path, pos, SIZE_MAX);
	}

	swift_close_dir(dir);
	swift_del_dir(path->data);

	return true;
}

bool swift_del_dir_r(const char *cpath)
{
	swift_str_t *path;
	size_t count;

	cpath = cpath ? cpath : ".";

	path = swift_new_cstr(cpath);

	count = path->count;

	if (path->data[count - 1] == '/' || path->data[count - 1] == '\\')
		swift_remove_str(path, count - 1, SIZE_MAX);

	swift_do_del_dir_r(path);

	return true;
}
