#include <swift/fs.h>

#include <dirent.h>
#include <unistd.h>

#include <sys/stat.h>
#include <sys/types.h>

struct swift_dir_t {
	DIR *handle;
};

bool swift_check_path(const char *path)
{
	struct stat file_info;

	if (lstat(path, &file_info) < 0)
		return false;

	return true;
}

bool swift_is_file(const char *path)
{
	struct stat file_info;

	if (lstat(path, &file_info) < 0)
		return false;

	return S_ISREG(file_info.st_mode);
}

bool swift_is_dir(const char *path)
{
	struct stat file_info;

	if (lstat(path, &file_info) < 0)
		return false;

	return S_ISDIR(file_info.st_mode);
}

bool swift_is_symlink(const char *path)
{
	struct stat file_info;

	if (lstat(path, &file_info) < 0)
		return false;

	return S_ISLNK(file_info.st_mode);
}

bool swift_get_atime(const char *path, swift_time_t *time)
{
	struct stat file_info;

	if (!time)
		return false;

	if (lstat(path, &file_info) < 0)
		return false;

	*time = SWIFT_TIME(file_info.st_atime);

	return true;
}

bool swift_get_ctime(const char *path, swift_time_t *time)
{
	struct stat file_info;

	if (!time)
		return false;

	if (lstat(path, &file_info) < 0)
		return false;

	*time = SWIFT_TIME(file_info.st_ctime);

	return true;
}

bool swift_get_mtime(const char *path, swift_time_t *time)
{
	struct stat file_info;

	if (!time)
		return false;

	if (lstat(path, &file_info) < 0)
		return false;

	*time = SWIFT_TIME(file_info.st_mtime);

	return true;
}

uint64_t swift_get_file_size(const char *path)
{
	struct stat file_info;

	if (!time)
		return false;

	if (lstat(path, &file_info) < 0)
		return false;

	return file_info.st_size;
}

bool swift_new_dir(const char *path)
{
	/* TODO: permissions should be available as an argument. */
	return (mkdir(path, S_IRWXU | S_IRWXG | S_IRWXO) == 0);
}

bool swift_del_dir(const char *path)
{
	return (rmdir(path) == 0);
}

swift_dir_t *swift_open_dir(const char *path)
{
	swift_dir_t *dir;

	dir = malloc(sizeof(swift_dir_t));

	if (!dir)
		return NULL;

	dir->handle = opendir(path);

	if (!dir->handle)
		goto err_free_dir;

	return dir;

err_free_dir:
	SWIFT_FREE(dir);
	return NULL;
}

void swift_close_dir(swift_dir_t *dir)
{
	if (!dir)
		return;

	closedir(dir->handle);
	SWIFT_FREE(dir);
}

const char *swift_read_dir(swift_dir_t *dir)
{
	struct dirent *entry;

	if (!dir)
		return NULL;

	entry = readdir(dir->handle);

	if (!entry)
		return NULL;

	return entry->d_name;
}

bool swift_move_file(const char *dst_path, const char *src_path)
{
	return (rename(src_path, dst_path) == 0);
}

bool swift_del_file(const char *path)
{
	return (unlink(path) == 0);
}
