#include <swift/gui.h>
#include <swift/io.h>
#include <swift/window.h>

static void swift_handle_button_mouse_over(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	if (control->action.mouse_over)
		control->sprite.tex_pos.xywh.x = 2 * control->sprite.size.wh.w;

	control->mouse_over(wgts, control);
}

static void swift_handle_button_release(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	if (swift_widget_has_mouse_over(wgts, control))
		swift_handle_button_mouse_over(wgts, control);

	control->mouse_up(wgts, control, button, pressed);
}

static void swift_handle_button_click(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	if (swift_widget_has_mouse_over(wgts, control) && control->action.clicked)
			control->sprite.tex_pos.xywh.x = control->sprite.size.wh.w;

	control->mouse_down(wgts, control, button, pressed);
}

static void swift_handle_button_mouse_exit(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	control->sprite.tex_pos.xywh.x = 0;
	control->mouse_exit(wgts, control);
}

void swift_create_button(swift_widget_t *control, swift_widget_t *parent,
	swift_vec3f_t pos, swift_vec2f_t hw, swift_str_t *img_path, bool visible)
{
	swift_init_widget(control);
	control->pos.xy.x = pos.xyz.x;
	control->pos.xy.y = pos.xyz.y;
	control->sprite.size = hw;
	control->sprite.tex_pos.xywh.h = hw.wh.h;
	control->sprite.tex_pos.xywh.w = hw.wh.w;
	control->type = SWIFT_CONTROL_BUTTON;
	control->control_draw = &swift_draw_base;
	control->control_mouse_down = &swift_handle_button_click;
	control->control_mouse_up = &swift_handle_button_release;
	control->control_update_pos = &swift_handle_control_move;
	control->control_mouse_over = &swift_handle_button_mouse_over;
	control->control_mouse_exit = &swift_handle_button_mouse_exit;
	control->control_unload = &swift_unload_base;
	control->sprite.tex = swift_add_texture(img_path->data);
	control->sprite.shader = swift_get_shader("swift_basic_shader");

	if (visible)
		swift_widget_add(parent, control);
	else
		swift_widget_add_hidden(parent, control);

	control->sprite.pos.xyz.x = control->pos.xy.x + parent->sprite.pos.xyz.x;
	control->sprite.pos.xyz.y = control->pos.xy.y + parent->sprite.pos.xyz.y;
	control->sprite.pos.xyz.z = pos.xyz.z;

	swift_del_str(img_path);
}

void swift_create_button_lbl(swift_widget_t *control, swift_widget_t *parent,
	swift_vec3f_t pos, swift_vec2f_t hw, swift_str_t *img_path,
		swift_str_t *text, swift_vec4f_t color, swift_font_t *font,
			size_t font_size, bool visible)
{
	size_t text_width;
	swift_widget_t *lbl;

	if (!text || !font)
		return;

	swift_create_button(control, parent, pos, hw, img_path, visible);

	text_width = swift_get_string_width(text, font_size, font);

	if (!(lbl = calloc(1, sizeof(swift_widget_t)))) {
		eprintf("lbl in swift_create_button could not be made.\n");
		return;
	}

	swift_create_label(lbl, control, SWIFT_VEC3F((hw.wh.w - text_width) / 2,
		((hw.wh.h - font_size) / 2) - 2, pos.xyz.z), hw, color, font, 64,
			text, font_size, true);

	lbl->action.can_click_behind = true;
	control->control = lbl;
}
