#include <swift/gui.h>
#include <swift/io.h>
#include <swift/window.h>

static void swift_update_chatbox_offset(swift_chatbox_t *chat)
{
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&chat->list,
			swift_array_get(&chat->list, chat->voffset), chat->voffset);
	swift_widget_t *item;
	swift_text_t *str;
	bool isonlist = false;

	while (iter.index < chat->voffset + chat->max && iter.data) {
		item = *(swift_widget_t **)iter.data;
		str = item->control;

		if (chat->control && chat->control == item)
			isonlist = true;

		if (iter.index == chat->voffset) {
			item->pos.xy.x = chat->offset.xy.x;
			item->pos.xy.y = chat->offset.xy.y + 4;
		}
		else {
			item->pos.xy.x = chat->offset.xy.x;
			item->pos.xy.y = chat->offset.xy.y + ((iter.index - chat->voffset) *
				str->font_size + 4);
		}

		item->sprite.pos.xyz.x = item->pos.xy.x +
			item->parent->sprite.pos.xyz.x;
		item->sprite.pos.xyz.y = item->pos.xy.y +
			item->parent->sprite.pos.xyz.y;

		swift_lbl_pos_update(str, item);
		swift_array_next(&iter);
	}

	chat->selected = isonlist ? true : false;

	if (chat->selected) {
		chat->select.pos.xyz.x = chat->control->sprite.pos.xyz.x;
		chat->select.pos.xyz.y = chat->control->sprite.pos.xyz.y;
	}
}

static void swift_draw_chatbox(swift_widgets_t *wgts, swift_widget_t *control)
{
	swift_chatbox_t *chat = control->control;
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&chat->list,
			swift_array_get(&chat->list, chat->voffset), chat->voffset);
	swift_widget_t *item;

	swift_render(wgts->window, &control->sprite);

	if (chat->selected)
		swift_render(wgts->window, &chat->select);

	if (chat->isover)
		swift_render(wgts->window, &chat->over);

	while (iter.index < chat->voffset + chat->max && iter.data) {
		item = *(swift_widget_t **)iter.data;
		item->control_draw(wgts, item);
		swift_array_next(&iter);
	}
}

static void swift_draw_label(swift_widgets_t *wgts, swift_widget_t *control)
{
	swift_text_t *str = control->control;

	if (str) {
		swift_array_iter_t iter;

		for (iter = swift_array_first(str->buf); iter.data;
			swift_array_next(&iter))
			swift_render(wgts->window, iter.data);
	}
}

static void swift_handle_slabel_move(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_text_t *str = control->control;
	swift_lbl_pos_update(str, control);
}

static void swift_handle_chatbox_release(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	swift_chatbox_t *chat = control->control;
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&chat->list,
			swift_array_get(&chat->list, chat->voffset), chat->voffset);
	swift_widget_t *item;

	while (iter.index < chat->voffset + chat->max && iter.data) {
		item = *(swift_widget_t **)iter.data;

		if (swift_widget_rect_contains(wgts, item)) {
			if (item->action.clicked) {
				item->action.clicked = false;
				item->control_mouse_down(wgts, item, button, pressed);
			}
		}

		swift_array_next(&iter);
	}
}

static void swift_handle_chatbox_click(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	swift_chatbox_t *chat = control->control;

	if (chat->control && swift_widget_rect_contains(wgts, chat->control)) {
		chat->selected = false;
		chat->control = NULL;
	}
	else {
		swift_array_iter_t iter = SWIFT_ARRAY_ITER(&chat->list,
			swift_array_get(&chat->list, chat->voffset), chat->voffset);
		swift_widget_t *item;

		while (iter.index < chat->voffset + chat->max && iter.data) {
			item = *(swift_widget_t **)iter.data;

			if (swift_widget_rect_contains(wgts, item)) {
				item->action.clicked = true;
				item->control_mouse_down(wgts, item, button, pressed);
			}

			swift_array_next(&iter);
		}
	}
}

static void swift_handle_chatbox_mouse_over(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_chatbox_t *chat = control->control;
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&chat->list,
			swift_array_get(&chat->list, chat->voffset), chat->voffset);
	swift_widget_t *item;

	while (iter.index < chat->voffset + chat->max && iter.data) {
		item = *(swift_widget_t **)iter.data;

		if (swift_widget_rect_contains(wgts, item)) {
			item->action.mouse_over = true;
			item->control_mouse_over(wgts, item);
		}
		else {
			if (item->action.mouse_over) {
				item->action.mouse_over = false;
				item->control_mouse_exit(wgts, item);
			}
		}

		swift_array_next(&iter);
	}
}

static void swift_handle_chatbox_mouse_exit(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_chatbox_t *chat = control->control;
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&chat->list,
			swift_array_get(&chat->list, chat->voffset), chat->voffset);
	swift_widget_t *item;

	while (iter.index < chat->voffset + chat->max && iter.data) {
		item = *(swift_widget_t **)iter.data;

		if (item->action.mouse_over) {
			item->action.mouse_over = false;
			item->control_mouse_exit(wgts, item);
		}

		swift_array_next(&iter);
	}
}

static void swift_handle_chatbox_label_click(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	swift_chatbox_t *chat = (swift_chatbox_t *)control->parent->control;

	if (swift_widget_usable(control)) {
		if (control->action.clicked) {
			chat->control = control;
			chat->selected = true;
			chat->select.pos.xyz.x = control->sprite.pos.xyz.x;
			chat->select.pos.xyz.y = control->sprite.pos.xyz.y;
		}
	}

	control->mouse_down(wgts, control, button, pressed);
}

static void swift_handle_chatbox_label_mouse_over(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_chatbox_t *chat = control->parent->control;

	if (control->action.mouse_over) {
		chat->isover = true;
		chat->over.pos.xyz.x = control->sprite.pos.xyz.x;
		chat->over.pos.xyz.y = control->sprite.pos.xyz.y;
	}

	control->mouse_over(wgts, control);
}

static void swift_handle_chatbox_label_mouse_exit(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_chatbox_t *chat = control->parent->control;
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&chat->list,
			swift_array_get(&chat->list, chat->voffset), chat->voffset);
	swift_widget_t *item;
	bool isover = false;

	while (iter.index < chat->voffset + chat->max && iter.data) {
		item = *(swift_widget_t **)iter.data;

		if (item->action.mouse_over)
			isover = true;

		swift_array_next(&iter);
	}

	if (!isover)
		chat->isover = false;

	control->mouse_exit(wgts, control);
}

static void handle_chatbox_scroll(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	swift_chatbox_t *chat = control->parent->control;

	if (chat->list.count >= chat->max)
		chat->voffset = control->value;

	swift_update_chatbox_offset(chat);
}

static void swift_handle_chatbox_move(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_chatbox_t *chat = control->control;
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&chat->list,
			swift_array_get(&chat->list, chat->voffset), chat->voffset);
	swift_widget_t *item;

	swift_handle_control_move(wgts, control);

	while (iter.index < chat->voffset + chat->max && iter.data) {
		item = *(swift_widget_t **)iter.data;

		item->control_update_pos(wgts, item);
		swift_array_next(&iter);
	}
}

static void swift_set_chatbox_indexs(swift_widget_t *control, uint32_t index)
{
	swift_chatbox_t *chat = control->control;
	swift_array_iter_t iter;
	swift_widget_t *item;

	if (!chat || !control)
		return;

	if (index <= 0)
		iter = swift_array_first(&chat->list);
	else
		iter = SWIFT_ARRAY_ITER(&chat->list,
			swift_array_get(&chat->list, index), index);

	while (iter.data) {
		item = *(swift_widget_t **)iter.data;
		item->value = iter.index;
		swift_array_next(&iter);
	}
}

static void swift_remove_chatbox_msg_max(swift_widget_t *control)
{
	swift_chatbox_t *chat = control->control;
	swift_array_iter_t iter = swift_array_last(&chat->list);
	swift_widget_t *item = *(swift_widget_t **)iter.data;

	if (!item)
		return;

	swift_unload_label(NULL, item);

	swift_array_remove_range(&chat->list, iter.index, 1);
}

void swift_create_chatbox(swift_widget_t *control, swift_widget_t *parent,
	swift_vec3f_t pos, swift_vec2f_t hw, float barwidth, swift_vec2f_t btn_hw,
		swift_vec2f_t offset, swift_font_t *font, swift_str_t *imgchatbg,
			swift_str_t *imgbg, swift_str_t *imgup, swift_str_t *imgdown,
				swift_str_t *bar, swift_vec4f_t select_color,
					swift_vec4f_t over_color, size_t font_size, size_t max_msgs,
					 bool visible)
{
	swift_chatbox_t *chat = calloc(1, sizeof(swift_chatbox_t));

	if (!chat) {
		fprintf(stderr, "POINTER_NULL, chat in swift_create_chatbox.\n");
		return;
	}

	swift_init_array(&chat->list, NULL, sizeof(swift_widget_t *));

	swift_init_widget(control);
	control->sprite.tex = swift_add_texture(imgchatbg->data);
	control->sprite.shader = swift_get_shader("swift_basic_shader");
	control->pos.xy.x = pos.xyz.x;
	control->pos.xy.y = pos.xyz.y;
	control->sprite.size = hw;
	control->sprite.tex_pos.xywh.h = control->sprite.tex->size.wh.h;
	control->sprite.tex_pos.xywh.w = control->sprite.tex->size.wh.w;
	control->control_mouse_down = &swift_handle_chatbox_click;
	control->control_mouse_up = &swift_handle_chatbox_release;
	control->control_mouse_over = &swift_handle_chatbox_mouse_over;
	control->control_mouse_exit = &swift_handle_chatbox_mouse_exit;
	control->control_update_pos = &swift_handle_chatbox_move;
	control->control_unload = &swift_unload_chatbox;
	control->type = SWIFT_CONTROL_LISTBOX;
	control->control_draw = &swift_draw_chatbox;

	chat->select = swift_init_sprite_notex(select_color);
	chat->select.size.wh.h = font_size + 2;
	chat->select.size.wh.w = control->sprite.size.wh.w  - barwidth;

	chat->over = swift_init_sprite_notex(over_color);
	chat->over.size.wh.h = font_size + 2;
	chat->over.size.wh.w = control->sprite.size.wh.w  - barwidth;

	chat->offset = offset;
	chat->voffset = 0;
	chat->max_msgs = max_msgs;

	if (visible)
		swift_widget_add(parent, control);
	else
		swift_widget_add_hidden(parent, control);

	control->sprite.pos.xyz.x = control->pos.xy.x + parent->sprite.pos.xyz.x;
	control->sprite.pos.xyz.y = control->pos.xy.y + parent->sprite.pos.xyz.y;
	control->sprite.pos.xyz.z = pos.xyz.z;

	chat->over.pos.xyz.x = offset.xy.x + control->sprite.pos.xyz.x;
	chat->over.pos.xyz.y = offset.xy.y + control->sprite.pos.xyz.y;
	chat->over.pos.xyz.z = pos.xyz.z;

	chat->select.pos.xyz.x = offset.xy.x + control->sprite.pos.xyz.x;
	chat->select.pos.xyz.y = offset.xy.y + control->sprite.pos.xyz.y;
	chat->select.pos.xyz.z = pos.xyz.z;

	chat->max = hw.wh.h / (font_size + 4 + offset.xy.y);
	chat->font = font;
	chat->font_size = font_size;

	control->control = chat;
	swift_create_vscrollbar(&chat->vbar, control, SWIFT_VEC3F(hw.wh.w -
		barwidth, 0, pos.xyz.z), SWIFT_VEC2F(barwidth, hw.wh.h), btn_hw, 0, 0,
			imgbg, imgup, imgdown, bar, true);

	swift_set_scrollbar_buttons_mouse_down(&chat->vbar, &handle_chatbox_scroll);
	chat->vbar.action.always_useable = true;

	swift_del_str(imgchatbg);
}

void swift_chatbox_add(swift_widget_t *control, swift_str_t *username,
	swift_vec4f_t user_color, swift_str_t *msg, swift_vec4f_t msg_color)
{
	swift_widget_t *item = calloc(1, sizeof(swift_widget_t));
	swift_text_t *str = calloc(1, sizeof(swift_text_t));
	swift_chatbox_t *chat = control->control;

	if (!item || !str || !chat) {
		fprintf(stderr, "POINTER_NULL init_text or chat \
			in swift_create_chatbox.\n");
		return;
	}

	swift_init_widget(item);
	item->type = SWIFT_CONTROL_LABEL;
	item->pos.xy.x = chat->offset.xy.x;
	item->pos.xy.y = chat->offset.xy.y;
	item->control_draw = &swift_draw_label;
	item->control_mouse_down = &swift_handle_chatbox_label_click;
	item->control_mouse_over = &swift_handle_chatbox_label_mouse_over;
	item->control_mouse_exit = &swift_handle_chatbox_label_mouse_exit;
	item->control_update_pos = &swift_handle_slabel_move;
	item->parent = control;

	str->color = user_color;
	str->str = swift_new_strf("%s%s", username->data, msg->data);
	str->maxchars = SWIFT_VEC2F(100, 0);
	str->offset = SWIFT_VEC2F(0, 0);
	str->font = chat->font;
	str->font_size = chat->font_size;

	if (!str->buf) {
		if (!(str->buf = swift_new_array_size(NULL, sizeof(swift_sprite_t), 2)))
			eprintf("Data Buf in Text Set Equaled Nothing /n");
	}

	item->sprite.size.wh.h = chat->font_size;
	item->sprite.size.wh.w = chat->over.size.wh.w;
	item->control = str;

	item->sprite.pos.xyz.x = item->pos.xy.x + control->sprite.pos.xyz.x;
	item->sprite.pos.xyz.y = item->pos.xy.y + control->sprite.pos.xyz.y;
	item->sprite.pos.xyz.z = control->sprite.pos.xyz.z;

	swift_create_chatbox_text_vertex(str, item, user_color, username);
	swift_create_chatbox_text_vertex(str, item, msg_color, msg);
	item->value = 0;

	if (chat->list.count >= chat->max_msgs)
		swift_remove_chatbox_msg_max(control);

	swift_array_insert(&chat->list, 0, &item);
	swift_set_chatbox_indexs(control, 0);
	swift_update_chatbox_offset(chat);

	if (chat->list.count >= chat->max)
		swift_vscrollbar_value_reset(&chat->vbar, chat->vbar.value,
			(chat->list.count - 1) - chat->max);
}

void swift_unload_chatbox(swift_widgets_t *wgts, swift_widget_t *control)
{
	swift_chatbox_t *chat = control->control;
	swift_array_iter_t iter;
	swift_widget_t *item;

	chat->control = NULL;

	for (iter = swift_array_first(&chat->list); iter.data;
		swift_array_next(&iter)) {
		item = *(swift_widget_t **)iter.data;
		swift_unload_label(wgts, item);
		SWIFT_FREE(item);
	}

	swift_term_array(&chat->list);
	swift_unload_base(wgts, control);
}

void swift_set_chatbox_selected(swift_widget_t *control, swift_str_t *username,
	swift_vec4f_t user_color, swift_str_t *msg, swift_vec4f_t msg_color)
{
	swift_chatbox_t *chat = control->control;
	swift_widget_t *item;
	swift_text_t *str;

	if (!chat)
		return;

	item = chat->control;
	str = item->control;

	if (!item || !str)
		return;

	if (username) {
		swift_del_str(str->str);
		str->str = swift_new_strf("%s%s", username->data, msg->data);
		swift_array_remove_range(str->buf, 0, str->buf->count);
		swift_create_chatbox_text_vertex(str, item, user_color, username);
		swift_create_chatbox_text_vertex(str, item, msg_color, msg);
	}
}

void swift_set_chatbox_msg(swift_widget_t *control, swift_str_t *username,
	swift_vec4f_t user_color, swift_str_t *msg, swift_vec4f_t msg_color,
		size_t index)
{
	swift_chatbox_t *chat = control->control;
	swift_widget_t *item;
	swift_text_t *str;

	if (!chat)
		return;

	item = *(swift_widget_t **)swift_array_get(&chat->list, index);
	str = item->control;

	if (!item || !str)
		return;

	if (username) {
		swift_del_str(str->str);
		str->str = swift_new_strf("%s%s", username->data, msg->data);
		swift_array_remove_range(str->buf, 0, str->buf->count);
		swift_create_chatbox_text_vertex(str, item, user_color, username);
		swift_create_chatbox_text_vertex(str, item, msg_color, msg);
	}
}

void swift_remove_chatbox_msg(swift_widget_t *control, size_t index)
{
	swift_chatbox_t *chat = control->control;
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&chat->list,
			swift_array_get(&chat->list, index), index);
	swift_widget_t *item = *(swift_widget_t **)iter.data;

	if (!item)
		return;

	swift_unload_label(NULL, item);

	swift_array_remove_range(&chat->list, index, 1);
	swift_set_chatbox_indexs(control, index);
	swift_update_chatbox_offset(chat);
	swift_vscrollbar_value_reset(&chat->vbar, chat->vbar.value,
		(chat->list.count - 1) - chat->max);
}

void swift_remove_chatbox_msg_range(swift_widget_t *control, size_t index,
	size_t count)
{
	swift_chatbox_t *chat = control->control;
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&chat->list,
			swift_array_get(&chat->list, index), index);
	swift_widget_t *item;

	while (iter.index < index + count && iter.data) {
		item = *(swift_widget_t **)iter.data;

		swift_unload_label(NULL, item);
		swift_array_next(&iter);
	}

	swift_array_remove_range(&chat->list, index, count);
	swift_set_chatbox_indexs(control, index);
	swift_update_chatbox_offset(chat);
	swift_vscrollbar_value_reset(&chat->vbar, chat->vbar.value,
		(chat->list.count - 1) - chat->max);
}

size_t swift_get_selected_msg_index(swift_widget_t *control)
{
	swift_chatbox_t *chat = control->control;

	if (!chat || !control)
		return 0;

	return chat->control->value;
}
