#include <swift/gui.h>
#include <swift/io.h>
#include <swift/window.h>

static void swift_handle_check_click(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	if (button == SWIFT_MOUSE_BUTTON_LEFT && pressed == true)
		control->action.checked ^= true;

	if (control->action.checked)
		control->sprite.tex_pos.xywh.x = 0;
	else
		control->sprite.tex_pos.xywh.x = control->sprite.size.wh.w;

	control->mouse_down(wgts, control, button, pressed);
}

void swift_create_checkbox(swift_widget_t *control, swift_widget_t *parent,
	swift_vec3f_t pos, swift_vec2f_t hw, swift_str_t *img_path, bool visible)
{
	swift_init_widget(control);
	control->pos.xy.x = pos.xyz.x;
	control->pos.xy.y = pos.xyz.y;
	control->sprite.size = hw;
	control->sprite.tex_pos.xywh.h = hw.wh.h;
	control->sprite.tex_pos.xywh.w = hw.wh.w;
	control->type = SWIFT_CONTROL_CHECKBOX;
	control->control_draw = &swift_draw_base;
	control->control_mouse_down = &swift_handle_check_click;
	control->control_update_pos = &swift_handle_control_move;
	control->control_unload = &swift_unload_base;
	control->sprite.tex = swift_add_texture(img_path->data);
	control->sprite.shader = swift_get_shader("swift_basic_shader");

	if (visible)
		swift_widget_add(parent, control);
	else
		swift_widget_add_hidden(parent, control);

	control->sprite.pos.xyz.x = control->pos.xy.x + parent->sprite.pos.xyz.x;
	control->sprite.pos.xyz.y = control->pos.xy.y + parent->sprite.pos.xyz.y;
	control->sprite.pos.xyz.z = pos.xyz.z;

	swift_del_str(img_path);
}

void swift_create_checkbox_lbl(swift_widget_t *control, swift_widget_t *parent,
	swift_vec3f_t pos, swift_vec2f_t hw, swift_str_t *img_path,
		swift_str_t *text, swift_font_t *font, swift_vec4f_t color,
			size_t font_size, bool visible)
{
	swift_widget_t *lbl;

	swift_create_checkbox(control, parent, pos, hw, img_path, visible);

	if (!text || !font)
		return;

	if (!(lbl = calloc(1, sizeof(swift_widget_t)))) {
		fprintf(stderr, "POINTER_NULL, lbl in swift_create_button.\n");
		return;
	}

	swift_create_label(lbl, control, SWIFT_VEC3F(hw.wh.w + 2,
		((hw.wh.h - font_size) / 2) - 2, pos.xyz.z), hw, color, font, 64, text,
			font_size, true);

	lbl->action.can_click_behind = true;
	control->control = lbl;
}
