#include <swift/gui.h>
#include <swift/io.h>
#include "input.h"

bool swift_create_gui_callbacks(swift_widgets_t *wgts)
{
	swift_intern_gui_cb_t *cbs = calloc(1, sizeof(swift_intern_gui_cb_t));

	if (!wgts || !cbs)
		return false;

	wgts->callbacks = cbs;

	return true;
}

void swift_destroy_gui_callbacks(swift_widgets_t *wgts)
{
	if (wgts && wgts->callbacks)
		SWIFT_FREE(wgts->callbacks);
}

void swift_gui_char_callback(swift_window_t *window, unsigned int key)
{
	swift_widgets_t *wgts = window->user_data;

	if (wgts->focused && wgts->focused->type == SWIFT_CONTROL_TEXTBOX)
		wgts->focused->control_key_down(wgts, wgts->focused, key, 1);

	if (wgts && wgts->callbacks) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		if (cbs->chars)
			cbs->chars(wgts, key);
	}
}

void swift_gui_key_callback(swift_window_t *window, int key, int scancode,
	int action, int mods)
{
	swift_widgets_t *wgts = window->user_data;

	if (wgts->focused) {
		if (wgts->focused->type == SWIFT_CONTROL_TEXTBOX && (key ==
			SWIFT_KEY_BACKSPACE || key == SWIFT_KEY_ENTER)) {
			if (action == SWIFT_BUTTON_PRESS || action == SWIFT_BUTTON_REPEAT)
				wgts->focused->control_key_down(wgts, wgts->focused, key,
					action);
		}
		else {
			if (wgts->focused->type != SWIFT_CONTROL_TEXTBOX)
				wgts->focused->control_key_down(wgts, wgts->focused, key,
					action);
		}
	}

	if (wgts && wgts->callbacks) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		if (cbs->keys)
			cbs->keys(wgts, key, scancode, action, mods);
	}
}

void swift_gui_mouse_pos_callback(swift_window_t *window, double x, double y)
{
	swift_widgets_t *wgts = window->user_data;

	swift_widget_move(wgts, (float)x,(float)y);
	wgts->mouse_pos.xy.x = (float)x;
	wgts->mouse_pos.xy.y = (float)y;

	if (wgts && wgts->callbacks) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		if (cbs->cursor_pos)
			cbs->cursor_pos(wgts, x, y);
	}
}

void swift_gui_mouse_press_callback(swift_window_t *window, int button,
	int pressed, int mods)
{
	swift_widgets_t *wgts = window->user_data;
	swift_widget_t *control = NULL;

	wgts->clicked = pressed;
	wgts->mouse_clicked.xy.x = wgts->mouse_pos.xy.x;
	wgts->mouse_clicked.xy.y = wgts->mouse_pos.xy.y;
	wgts->button = button;

	if (swift_widget_check_focus(wgts))
		control = wgts->focused;
	else
		control = &wgts->root;

	if (pressed == 1)
		swift_widget_mouse_press(wgts, control);
	else
		swift_widget_mouse_release(wgts, control);

	if (wgts && wgts->callbacks) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		if (cbs->mouse_button)
			cbs->mouse_button(wgts, button, pressed, mods);
	}
}

void swift_gui_mouse_wheel_callback(swift_window_t *window, double x, double y)
{
	swift_widgets_t *wgts = window->user_data;

	if (wgts && wgts->callbacks) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		if (cbs->scroll)
			cbs->scroll(wgts, x, y);
	}
}

void swift_gui_window_close(swift_window_t *window)
{
	swift_widgets_t *wgts = window->user_data;

	if (wgts && wgts->callbacks) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		if (cbs->win_close)
			cbs->win_close(wgts);
	}
}

void swift_gui_mods_callback(swift_window_t *window, unsigned int key, int mods)
{
	swift_widgets_t *wgts = window->user_data;

	if (wgts && wgts->callbacks) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		if (cbs->mods)
			cbs->mods(wgts, key, mods);
	}
}

void swift_gui_window_pos_callback(swift_window_t *window, int x, int y)
{
	swift_widgets_t *wgts = window->user_data;

	if (wgts && wgts->callbacks) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		if (cbs->win_pos)
			cbs->win_pos(wgts, x, y);
	}
}

void swift_gui_window_resize_callback(swift_window_t *window, int width,
	int height)
{
	swift_widgets_t *wgts = window->user_data;

	if (wgts && wgts->callbacks) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		if (cbs->win_resize)
			cbs->win_resize(wgts, width, height);
		else
			swift_handle_window_resize(window, width, height);
	}
}

void swift_gui_window_refresh_callback(swift_window_t *window)
{
	swift_widgets_t *wgts = window->user_data;

	if (wgts && wgts->callbacks) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		if (cbs->win_refresh)
			cbs->win_refresh(wgts);
	}
}

void swift_gui_window_focus_callback(swift_window_t *window, int focused)
{
	swift_widgets_t *wgts = window->user_data;

	if (wgts->moving)
		wgts->moving = false;

	if (wgts && wgts->callbacks) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		if (cbs->win_focus)
			cbs->win_focus(wgts, focused);
	}
}

void swift_gui_iconify_callback(swift_window_t *window, int iconified)
{
	swift_widgets_t *wgts = window->user_data;

	if (wgts && wgts->callbacks) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		if (cbs->iconify)
			cbs->iconify(wgts, iconified);
	}
}

void swift_gui_buffer_resize_callback(swift_window_t *window, int width,
	int height)
{
	swift_widgets_t *wgts = window->user_data;

	if (wgts && wgts->callbacks) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		if (cbs->buffer_resize)
			cbs->buffer_resize(wgts, width, height);
	}
}

void swift_gui_cursor_entered_callback(swift_window_t *window, int entered)
{
	swift_widgets_t *wgts = window->user_data;

	if (wgts && wgts->callbacks) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		if (cbs->cursor_enter)
			cbs->cursor_enter(wgts, entered);
	}
}

void swift_gui_drops_callback(swift_window_t *window, int count,
	const char **paths)
{
	swift_widgets_t *wgts = window->user_data;

	if (wgts && wgts->callbacks) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		if (cbs->drops)
			cbs->drops(wgts, count, paths);
	}
}

void swift_gui_set_char_callback(swift_widgets_t *wgts,
	swift_gui_char_fn_t callback)
{
	if (wgts && wgts->callbacks && callback) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		cbs->chars = callback;
	}
}

void swift_gui_set_mods_callback(swift_widgets_t *wgts,
	swift_gui_mods_fn_t callback)
{
	if (wgts && wgts->callbacks && callback) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		cbs->mods = callback;
	}
}

void swift_gui_set_key_callback(swift_widgets_t *wgts,
	swift_gui_key_fn_t callback)
{
	if (wgts && wgts->callbacks && callback) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		cbs->keys = callback;
	}
}

void swift_gui_set_cursor_pos_callback(swift_widgets_t *wgts,
	swift_gui_cur_pos_fn_t callback)
{
	if (wgts && wgts->callbacks && callback) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		cbs->cursor_pos = callback;
	}
}

void swift_gui_set_mouse_press_callback(swift_widgets_t *wgts,
	swift_gui_mouse_btn_fn_t callback)
{
	if (wgts && wgts->callbacks && callback) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		cbs->mouse_button = callback;
	}
}

void swift_gui_set_scroll_callback(swift_widgets_t *wgts,
	swift_gui_scroll_fn_t callback)
{
	if (wgts && wgts->callbacks && callback) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		cbs->scroll = callback;
	}
}

void swift_gui_set_wgts_close_callback(swift_widgets_t *wgts,
	swift_gui_win_close_fn_t callback)
{
	if (wgts && wgts->callbacks && callback) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		cbs->win_close = callback;
	}
}

void swift_gui_set_wgts_pos_callback(swift_widgets_t *wgts,
	swift_gui_win_pos_fn_t callback)
{
	if (wgts && wgts->callbacks && callback) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		cbs->win_pos = callback;
	}
}

void swift_gui_set_wgts_resize_callback(swift_widgets_t *wgts,
	swift_gui_win_resize_fn_t callback)
{
	if (wgts && wgts->callbacks && callback) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		cbs->win_resize = callback;
	}
}

void swift_gui_set_wgts_refresh_callback(swift_widgets_t *wgts,
	swift_gui_win_refresh_fn_t callback)
{
	if (wgts && wgts->callbacks && callback) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		cbs->win_refresh = callback;
	}
}

void swift_gui_set_wgts_focus_callback(swift_widgets_t *wgts,
	swift_gui_win_focus_fn_t callback)
{
	if (wgts && wgts->callbacks && callback) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		cbs->win_focus = callback;
	}
}

void swift_gui_set_iconify_callback(swift_widgets_t *wgts,
	swift_gui_iconify_fn_t callback)
{
	if (wgts && wgts->callbacks && callback) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		cbs->iconify = callback;
	}
}

void swift_gui_set_buffer_resize_callback(swift_widgets_t *wgts,
	swift_gui_buf_resize_fn_t callback)
{
	if (wgts && wgts->callbacks && callback) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		cbs->buffer_resize = callback;
	}
}

void swift_gui_set_cursor_entered_callback(swift_widgets_t *wgts,
	swift_gui_cur_enter_fn_t callback)
{
	if (wgts && wgts->callbacks && callback) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		cbs->cursor_enter = callback;
	}
}

void swift_gui_set_drops_callback(swift_widgets_t *wgts,
	swift_gui_drop_fn_t callback)
{
	if (wgts && wgts->callbacks && callback) {
		swift_intern_gui_cb_t *cbs = wgts->callbacks;

		cbs->drops = callback;
	}
}
