#include <swift/gui.h>
#include <swift/io.h>
#include <swift/window.h>

static void swift_draw_label(swift_widgets_t *wgts, swift_widget_t *control)
{
	swift_text_t *str = control->control;

	if (str) {
		swift_array_iter_t iter;

		for (iter = swift_array_first(str->buf); iter.data;
			swift_array_next(&iter))
			swift_render(wgts->window, iter.data);
	}
}

static void swift_handle_label_click(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	if(control->action.can_use_event)
		control->mouse_down(wgts, control, button, pressed);
}

static void swift_handle_slabel_move(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_handle_control_move(wgts, control);
	swift_lbl_pos_update(control->control, control);
}

void swift_create_label(swift_widget_t *control, swift_widget_t *parent,
	swift_vec3f_t pos, swift_vec2f_t hw, swift_vec4f_t color,
		swift_font_t *font, size_t maxcharspl, swift_str_t *string,
			size_t font_size, bool visible)
{
	swift_text_t *str = calloc(1, sizeof(swift_text_t));

	if (!str) {
		eprintf("POINTER_NULL, str in swift_create_label.\n");
		return;
	}

	swift_init_widget(control);
	control->pos.xy.x = pos.xyz.x;
	control->pos.xy.y = pos.xyz.y;
	control->sprite.size = hw;
	control->type = SWIFT_CONTROL_LABEL;
	control->control_draw = &swift_draw_label;
	control->control_mouse_down = &swift_handle_label_click;
	control->control_unload = &swift_unload_label;
	control->control_update_pos = &swift_handle_slabel_move;

	swift_text_set(str, SWIFT_VEC2F(0, 0), SWIFT_VEC2F(0, maxcharspl),
		font, color, string, font_size);

	control->control = str;

	if (visible)
		swift_widget_add(parent, control);
	else
		swift_widget_add_hidden(parent, control);

	control->sprite.pos.xyz.x = control->pos.xy.x + parent->sprite.pos.xyz.x;
	control->sprite.pos.xyz.y = control->pos.xy.y + parent->sprite.pos.xyz.y;
	control->sprite.pos.xyz.z = pos.xyz.z;

	swift_create_text_vertex(str, control);
}

void swift_unload_label(swift_widgets_t *wgts, swift_widget_t *control)
{
	swift_text_t *str = control->control;

	swift_del_array(str->buf);
	swift_del_str(str->str);

	swift_unload_base2(wgts, control);
}

void swift_update_label_string(swift_widget_t *control, swift_str_t *string)
{
	swift_text_t *str = control->control;

	if (string) {
		swift_del_str(str->str);
		str->str = string;

		swift_create_text_vertex(str, control);
	}
}

void swift_update_label_string_col(swift_widget_t *control, swift_str_t *string,
	swift_vec4f_t color)
{
	swift_text_t *str = control->control;

	str->color = color;

	if (string) {
		swift_del_str(str->str);
		str->str = string;
		swift_create_text_vertex(str, control);
	}
	else
		swift_lbl_col_update(str, control);
}

swift_str_t *swift_get_label_string(swift_widget_t *control)
{
	swift_text_t *str = control->control;

	return str ? str->str : NULL;
}
