#include <swift/gui.h>
#include <swift/io.h>
#include <swift/window.h>

static void swift_update_list_offset(swift_listbox_t *list)
{
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&list->list,
			swift_array_get(&list->list, list->voffset), list->voffset);
	swift_widget_t *item;
	swift_text_t *str;
	bool isonlist = false;

	while (iter.index < list->voffset + list->max && iter.data) {
		item = *(swift_widget_t **)iter.data;
		str = item->control;

		if (list->control && list->control == item)
			isonlist = true;

		if (iter.index == list->voffset)
			item->pos = list->offset;
		else {
			item->pos.xy.x = list->offset.xy.x;
			item->pos.xy.y = list->offset.xy.y + ((iter.index - list->voffset) *
				str->font_size);
		}

		item->sprite.pos.xyz.x = item->pos.xy.x +
			item->parent->sprite.pos.xyz.x;
		item->sprite.pos.xyz.y = item->pos.xy.y +
			item->parent->sprite.pos.xyz.y;

		swift_lbl_pos_update(str, item);
		swift_array_next(&iter);
	}

	list->selected = isonlist ? true : false;

	if (list->selected) {
		list->select.pos.xyz.x = list->control->sprite.pos.xyz.x;
		list->select.pos.xyz.y = list->control->sprite.pos.xyz.y;
	}
}

static void swift_draw_listbox(swift_widgets_t *wgts, swift_widget_t *control)
{
	swift_listbox_t *list = control->control;
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&list->list,
			swift_array_get(&list->list, list->voffset), list->voffset);
	swift_widget_t *item;

	swift_render(wgts->window, &control->sprite);

	if (list->selected)
		swift_render(wgts->window, &list->select);

	if (list->isover)
		swift_render(wgts->window, &list->over);

	while (iter.index < list->voffset + list->max && iter.data) {
		item = *(swift_widget_t **)iter.data;
		item->control_draw(wgts, item);
		swift_array_next(&iter);
	}
}

static void swift_draw_label(swift_widgets_t *wgts, swift_widget_t *control)
{
	swift_text_t *str = control->control;

	if (str) {
		swift_array_iter_t iter;

		for (iter = swift_array_first(str->buf); iter.data;
			swift_array_next(&iter))
			swift_render(wgts->window, iter.data);
	}
}

static void swift_handle_slabel_move(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_text_t *str = control->control;
	swift_lbl_pos_update(str, control);
}

static void swift_handle_listbox_release(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	swift_listbox_t *list = control->control;
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&list->list,
			swift_array_get(&list->list, list->voffset), list->voffset);
	swift_widget_t *item;

	while (iter.index < list->voffset + list->max && iter.data) {
		item = *(swift_widget_t **)iter.data;

		if (swift_widget_rect_contains(wgts, item)) {
			if (item->action.clicked) {
				item->action.clicked = false;
				item->control_mouse_down(wgts, item, button, pressed);
			}
		}

		swift_array_next(&iter);
	}
}

static void swift_handle_listbox_click(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	swift_listbox_t *list = control->control;

	if (list->control && swift_widget_rect_contains(wgts, list->control)) {
		list->selected = false;
		list->control = NULL;
	}
	else {
		swift_array_iter_t iter = SWIFT_ARRAY_ITER(&list->list,
			swift_array_get(&list->list, list->voffset), list->voffset);
		swift_widget_t *item;

		while (iter.index < list->voffset + list->max && iter.data) {
			item = *(swift_widget_t **)iter.data;

			if (swift_widget_rect_contains(wgts, item)) {
				item->action.clicked = true;
				item->control_mouse_down(wgts, item, button, pressed);
			}

			swift_array_next(&iter);
		}
	}
}

static void swift_handle_listbox_mouse_over(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_listbox_t *list = control->control;
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&list->list,
			swift_array_get(&list->list, list->voffset), list->voffset);
	swift_widget_t *item;

	while (iter.index < list->voffset + list->max && iter.data) {
		item = *(swift_widget_t **)iter.data;

		if (swift_widget_rect_contains(wgts, item)) {
			item->action.mouse_over = true;
			item->control_mouse_over(wgts, item);
		}
		else {
			if (item->action.mouse_over) {
				item->action.mouse_over = false;
				item->control_mouse_exit(wgts, item);
			}
		}

		swift_array_next(&iter);
	}
}

static void swift_handle_listbox_mouse_exit(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_listbox_t *list = control->control;
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&list->list,
			swift_array_get(&list->list, list->voffset), list->voffset);
	swift_widget_t *item;

	while (iter.index < list->voffset + list->max && iter.data) {
		item = *(swift_widget_t **)iter.data;

		if (item->action.mouse_over) {
			item->action.mouse_over = false;
			item->control_mouse_exit(wgts, item);
		}

		swift_array_next(&iter);
	}
}

static void swift_handle_listbox_label_click(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	swift_listbox_t *list = (swift_listbox_t *)control->parent->control;

	if (swift_widget_usable(control)) {
		if (control->action.clicked) {
			list->control = control;
			list->selected = true;
			list->select.pos.xyz.x = control->sprite.pos.xyz.x;
			list->select.pos.xyz.y = control->sprite.pos.xyz.y;
		}
	}

	control->mouse_down(wgts, control, button, pressed);
}

static void swift_handle_listbox_label_mouse_over(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_listbox_t *list = control->parent->control;

	if (control->action.mouse_over) {
		list->isover = true;
		list->over.pos.xyz.x = control->sprite.pos.xyz.x;
		list->over.pos.xyz.y = control->sprite.pos.xyz.y;
	}

	control->mouse_over(wgts, control);
}

static void swift_handle_listbox_label_mouse_exit(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_listbox_t *list = control->parent->control;
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&list->list,
			swift_array_get(&list->list, list->voffset), list->voffset);
	swift_widget_t *item;
	bool isover = false;

	while (iter.index < list->voffset + list->max && iter.data) {
		item = *(swift_widget_t **)iter.data;

		if (item->action.mouse_over)
			isover = true;

		swift_array_next(&iter);
	}

	if (!isover)
		list->isover = false;

	control->mouse_exit(wgts, control);
}

static void handle_listbox_scroll(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	swift_listbox_t *list = control->parent->control;

	if (list->list.count >= list->max)
		list->voffset = control->value;

	swift_update_list_offset(list);
}

static void swift_handle_listbox_move(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_listbox_t *list = control->control;
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&list->list,
			swift_array_get(&list->list, list->voffset), list->voffset);
	swift_widget_t *item;

	swift_handle_control_move(wgts, control);

	while (iter.index < list->voffset + list->max && iter.data) {
		item = *(swift_widget_t **)iter.data;

		item->control_update_pos(wgts, item);
		swift_array_next(&iter);
	}
}

static void swift_set_list_indexs(swift_widget_t *control, uint32_t index)
{
	swift_listbox_t * list = control->control;
	swift_array_iter_t iter;
	swift_widget_t *item;

	if (!list || !control)
		return;

	if (index <= 0)
		iter = swift_array_first(&list->list);
	else
		iter = SWIFT_ARRAY_ITER(&list->list,
			swift_array_get(&list->list, index), index);

	while (iter.data) {
		item = *(swift_widget_t **)iter.data;
		item->value = iter.index;
		swift_array_next(&iter);
	}
}

void swift_create_listbox(swift_widget_t *control, swift_widget_t *parent,
	swift_vec3f_t pos, swift_vec2f_t hw, float barwidth, swift_vec2f_t btn_hw,
		swift_vec2f_t offset, swift_font_t *font, swift_vec4f_t color,
			swift_str_t *imglistbg, swift_str_t *imgbg, swift_str_t *imgup,
				swift_str_t *imgdown, swift_str_t *bar,
					swift_vec4f_t select_color, swift_vec4f_t over_color,
						size_t font_size, bool visible)
{
	swift_listbox_t *list = calloc(1, sizeof(swift_listbox_t));

	if (!list) {
		fprintf(stderr, "POINTER_NULL, list in swift_create_listbox.\n");
		return;
	}

	swift_init_array(&list->list, NULL, sizeof(swift_widget_t *));

	swift_init_widget(control);
	control->sprite.tex = swift_add_texture(imglistbg->data);
	control->sprite.shader = swift_get_shader("swift_basic_shader");
	control->pos.xy.x = pos.xyz.x;
	control->pos.xy.y = pos.xyz.y;
	control->sprite.size = hw;
	control->sprite.tex_pos.xywh.h = control->sprite.tex->size.wh.h;
	control->sprite.tex_pos.xywh.w = control->sprite.tex->size.wh.w;
	control->control_mouse_down = &swift_handle_listbox_click;
	control->control_mouse_up = &swift_handle_listbox_release;
	control->control_mouse_over = &swift_handle_listbox_mouse_over;
	control->control_mouse_exit = &swift_handle_listbox_mouse_exit;
	control->control_update_pos = &swift_handle_listbox_move;
	control->control_unload = &swift_unload_listbox;
	control->type = SWIFT_CONTROL_LISTBOX;
	control->control_draw = &swift_draw_listbox;

	list->select = swift_init_sprite_notex(select_color);
	list->select.size.wh.h = font_size;
	list->select.size.wh.w = control->sprite.size.wh.w  - barwidth;

	list->over = swift_init_sprite_notex(over_color);
	list->over.size.wh.h = font_size;
	list->over.size.wh.w = control->sprite.size.wh.w  - barwidth;

	list->offset = offset;
	list->voffset = 0;

	if (visible)
		swift_widget_add(parent, control);
	else
		swift_widget_add_hidden(parent, control);

	control->sprite.pos.xyz.x = control->pos.xy.x + parent->sprite.pos.xyz.x;
	control->sprite.pos.xyz.y = control->pos.xy.y + parent->sprite.pos.xyz.y;
	control->sprite.pos.xyz.z = pos.xyz.z;

	list->over.pos.xyz.x = offset.xy.x + control->sprite.pos.xyz.x;
	list->over.pos.xyz.y = offset.xy.y + control->sprite.pos.xyz.y;
	list->over.pos.xyz.z = pos.xyz.z;

	list->select.pos.xyz.x = offset.xy.x + control->sprite.pos.xyz.x;
	list->select.pos.xyz.y = offset.xy.y + control->sprite.pos.xyz.y;
	list->select.pos.xyz.z = pos.xyz.z;

	list->max = (hw.wh.h / (font_size + offset.xy.y));
	list->font = font;
	list->color = color;
	list->font_size = font_size;

	control->control = list;
	swift_create_vscrollbar(&list->vbar, control, SWIFT_VEC3F(hw.wh.w -
		barwidth, 0, pos.xyz.z), SWIFT_VEC2F(barwidth, hw.wh.h), btn_hw, 0, 0,
			imgbg, imgup, imgdown, bar, true);

	swift_set_scrollbar_buttons_mouse_down(&list->vbar, &handle_listbox_scroll);

	swift_del_str(imglistbg);
}

void swift_list_add(swift_widget_t *control, swift_str_t *name, void *data)
{
	swift_widget_t *item = calloc(1, sizeof(swift_widget_t));
	swift_text_t *str = calloc(1, sizeof(swift_text_t));
	swift_listbox_t *list = control->control;

	if (!item || !str || !list) {
		fprintf(stderr, "POINTER_NULL init_text or list \
			in swift_create_listbox.\n");
		return;
	}

	swift_init_widget(item);
	item->type = SWIFT_CONTROL_LABEL;
	item->pos.xy.x = list->offset.xy.x;
	item->pos.xy.y = list->offset.xy.y;
	item->control_draw = &swift_draw_label;
	item->control_mouse_down = &swift_handle_listbox_label_click;
	item->control_mouse_over = &swift_handle_listbox_label_mouse_over;
	item->control_mouse_exit = &swift_handle_listbox_label_mouse_exit;
	item->control_update_pos = &swift_handle_slabel_move;
	item->parent = control;

	swift_text_set(str, SWIFT_VEC2F(0, 0),
		SWIFT_VEC2F(100, 0), list->font, list->color, name, list->font_size);

	item->sprite.size.wh.h = list->font_size;
	item->sprite.size.wh.w = list->over.size.wh.w;
	item->control = str;
	item->data = data;

	item->sprite.pos.xyz.x = item->pos.xy.x + control->sprite.pos.xyz.x;
	item->sprite.pos.xyz.y = item->pos.xy.y + control->sprite.pos.xyz.y;
	item->sprite.pos.xyz.z = control->sprite.pos.xyz.z;

	swift_create_text_vertex(str, item);
	item->value = list->list.count;
	swift_array_push(&list->list, &item);
	swift_update_list_offset(list);

	if (list->list.count >= list->max)
		swift_vscrollbar_value_reset(&list->vbar, list->vbar.value,
			item->value - list->max);
}

void swift_list_insert(swift_widget_t *control, swift_str_t *name, void *data,
	size_t index)
{
	swift_widget_t *item = calloc(1, sizeof(swift_widget_t));
	swift_text_t *str = calloc(1, sizeof(swift_text_t));
	swift_listbox_t *list = control->control;

	if (!item || !str || !list) {
		fprintf(stderr, "POINTER_NULL init_text or list \
			in swift_create_listbox.\n");
		return;
	}

	swift_init_widget(item);
	item->type = SWIFT_CONTROL_LABEL;
	item->pos.xy.x = list->offset.xy.x;
	item->pos.xy.y = list->offset.xy.y;
	item->control_draw = &swift_draw_label;
	item->control_mouse_down = &swift_handle_listbox_label_click;
	item->control_mouse_over = &swift_handle_listbox_label_mouse_over;
	item->control_mouse_exit = &swift_handle_listbox_label_mouse_exit;
	item->control_update_pos = &swift_handle_slabel_move;
	item->parent = control;

	swift_text_set(str, SWIFT_VEC2F(0, 0),
		SWIFT_VEC2F(100, 0), list->font, list->color, name, list->font_size);

	item->sprite.size.wh.h = list->font_size;
	item->sprite.size.wh.w = list->over.size.wh.w;
	item->control = str;
	item->data = data;

	item->sprite.pos.xyz.x = item->pos.xy.x + control->sprite.pos.xyz.x;
	item->sprite.pos.xyz.y = item->pos.xy.y + control->sprite.pos.xyz.y;
	item->sprite.pos.xyz.z = control->sprite.pos.xyz.z;

	swift_create_text_vertex(str, item);
	swift_array_insert(&list->list, index, &item);
	swift_set_list_indexs(control, index);
	swift_update_list_offset(list);

	swift_vscrollbar_value_reset(&list->vbar, list->vbar.value,
		(list->list.count - 1) - list->max);
}

void swift_unload_listbox(swift_widgets_t *wgts, swift_widget_t *control)
{
	swift_listbox_t *list = control->control;
	swift_array_iter_t iter;
	swift_widget_t *item;

	list->control = NULL;

	for (iter = swift_array_first(&list->list); iter.data;
		swift_array_next(&iter)) {
		item = *(swift_widget_t **)iter.data;
		swift_unload_label(wgts, item);
		SWIFT_FREE(item);
	}

	swift_term_array(&list->list);
	swift_unload_base(wgts, control);
}

void swift_set_list_selected(swift_widget_t *control, swift_str_t *name,
	void *data)
{
	swift_listbox_t *list = control->control;
	swift_widget_t *item;
	swift_text_t *str;

	if (!list)
		return;

	item = list->control;
	str = item->control;

	if (!item || !str)
		return;

	if (name) {
		swift_del_str(str->str);
		str->str = name;
		swift_create_text_vertex(str, item);
	}

	if (data)
		item->data = data;
}

void swift_set_list_item(swift_widget_t *control, swift_str_t *name, void *data,
	size_t index)
{
	swift_listbox_t *list = control->control;
	swift_widget_t *item;
	swift_text_t *str;

	if (!list)
		return;

	item = *(swift_widget_t **)swift_array_get(&list->list, index);
	str = item->control;

	if (!item || !str)
		return;

	if (name) {
		swift_del_str(str->str);
		str->str = name;
		swift_create_text_vertex(str, item);
	}

	if (data)
		item->data = data;
}

void swift_remove_list_item(swift_widget_t *control, size_t index)
{
	swift_listbox_t *list = control->control;
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&list->list,
			swift_array_get(&list->list, index), index);
	swift_widget_t *item = *(swift_widget_t **)iter.data;

	if (!item)
		return;

	swift_unload_label(NULL, item);

	swift_array_remove_range(&list->list, index, 1);
	swift_set_list_indexs(control, index);
	swift_update_list_offset(list);
	swift_vscrollbar_value_reset(&list->vbar, list->vbar.value,
		(list->list.count - 1) - list->max);
}

void swift_remove_list_item_range(swift_widget_t *control, size_t index,
	size_t count)
{
	swift_listbox_t *list = control->control;
	swift_array_iter_t iter = SWIFT_ARRAY_ITER(&list->list,
			swift_array_get(&list->list, index), index);
	swift_widget_t *item;

	while (iter.index < index + count && iter.data) {
		item = *(swift_widget_t **)iter.data;

		swift_unload_label(NULL, item);
		swift_array_next(&iter);
	}

	swift_array_remove_range(&list->list, index, count);
	swift_set_list_indexs(control, index);
	swift_update_list_offset(list);
	swift_vscrollbar_value_reset(&list->vbar, list->vbar.value,
		(list->list.count - 1) - list->max);
}

void *swift_get_list_selected(swift_widget_t *control)
{
	swift_listbox_t *list = control->control;
	return list->control->data;
}

void *swift_get_list_item(swift_widget_t *control, size_t index)
{
	swift_listbox_t *list = control->control;
	swift_widget_t *item = *(swift_widget_t **)swift_array_get(&list->list,
		index);

	return item->data;
}

size_t swift_get_selected_index(swift_widget_t *control)
{
	swift_listbox_t *list = control->control;

	if (!list || !control)
		return 0;

	return list->control->value;
}
