#include <swift/gui.h>
#include <swift/io.h>
#include <swift/window.h>

static void swift_handle_picturebox_click(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	control->mouse_down(wgts, control, button, pressed);
}


void swift_create_picturebox(swift_widget_t *control, swift_widget_t *parent,
	swift_vec3f_t pos, swift_vec2f_t hw, swift_vec4f_t tex_pos,
		swift_str_t *img_path, bool visible)
{
	swift_init_widget(control);
	control->pos.xy.x = pos.xyz.x;
	control->pos.xy.y = pos.xyz.y;
	control->sprite.size = hw;
	control->sprite.tex_pos = tex_pos;
	control->type = SWIFT_CONTROL_PICTUREBOX;
	control->control_draw = &swift_draw_base;
	control->control_mouse_down = &swift_handle_picturebox_click;
	control->control_update_pos = &swift_handle_control_move;
	control->control_unload = &swift_unload_base;
	control->sprite.tex = swift_add_texture(img_path->data);
	control->sprite.shader = swift_get_shader("swift_basic_shader");

	if (visible)
		swift_widget_add(parent, control);
	else
		swift_widget_add_hidden(parent, control);

	control->sprite.pos.xyz.x = control->pos.xy.x + parent->sprite.pos.xyz.x;
	control->sprite.pos.xyz.y = control->pos.xy.y + parent->sprite.pos.xyz.y;
	control->sprite.pos.xyz.z = pos.xyz.z;

	swift_del_str(img_path);
}

void swift_update_picturebox(swift_widgets_t *wgts, swift_widget_t *control,
	swift_vec2f_t pos, swift_vec2f_t hw, swift_vec4f_t tex_pos,
		swift_str_t *img_path)
{

	if (img_path) {
		swift_remove_texture(control->sprite.tex);
		control->sprite.tex = swift_add_texture(img_path->data);
		swift_del_str(img_path);
	}

	control->pos = pos;
	control->sprite.size = hw;
	control->sprite.tex_pos = tex_pos;

	control->sprite.pos.xyz.x = control->pos.xy.x +
		control->parent->sprite.pos.xyz.x;
	control->sprite.pos.xyz.y = control->pos.xy.y +
		control->parent->sprite.pos.xyz.y;
}
