#include <swift/gui.h>
#include <swift/io.h>
#include <swift/window.h>

static void swift_handle_progressbar_click(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	control->mouse_down(wgts, control, button, pressed);
}

static void swift_handle_progress_move(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_widget_t *bar = control->control;

	control->sprite.pos.xyz.x = control->pos.xy.x +
		control->parent->sprite.pos.xyz.x;
	control->sprite.pos.xyz.y = control->pos.xy.y +
		control->parent->sprite.pos.xyz.y;

	if (control->type == SWIFT_CONTROL_HPROGRESSBAR) {
		control->sprite.size.wh.w = ((control->parent->sprite.size.wh.w /
			100.f) * control->parent->value);
		control->sprite.tex_pos.xywh.w = ((bar->sprite.tex->size.wh.w / 100.f) *
			control->parent->value);
	}
	else {
		control->sprite.size.wh.h = ((control->parent->sprite.size.wh.h /
			100.f) * control->parent->value);
		control->sprite.tex_pos.xywh.h = ((bar->sprite.tex->size.wh.h / 100.f) *
			control->parent->value);
	}
}

void swift_update_progressbar_value(swift_widget_t *control, size_t value)
{
	swift_widget_t *bar = control->control;
	control->value = value;

	if (control->type == SWIFT_CONTROL_HPROGRESSBAR) {
		bar->sprite.size.wh.w = (control->sprite.size.wh.w / 100.f) * value;
		bar->sprite.tex_pos.xywh.w = (bar->sprite.tex->size.wh.w / 100.f) *
			value;
	}
	else {
		bar->sprite.size.wh.h = (control->sprite.size.wh.h / -100.f) * value;
		bar->sprite.tex_pos.xywh.h = (bar->sprite.tex->size.wh.h / 100.f) *
			value;
	}
}

void swift_create_hprogressbar(swift_widget_t *control, swift_widget_t *parent,
	swift_vec3f_t pos, swift_vec2f_t hw, size_t value,
		swift_str_t *background, swift_str_t *hbar, bool visible)
{
	swift_widget_t *bar = calloc(1, sizeof(swift_widget_t));

	if (!bar) {
		fprintf(stderr, "POINTER_NULL, bar in swift_create_hprogressbar.\n");
		return;
	}

	swift_init_widget(control);
	swift_init_widget(bar);

	control->pos.xy.x = pos.xyz.x;
	control->pos.xy.y = pos.xyz.y;
	control->sprite.size = hw;
	control->sprite.tex_pos.xywh.h = hw.wh.h;
	control->sprite.tex_pos.xywh.w = hw.wh.w;
	control->type = SWIFT_CONTROL_HPROGRESSBAR;
	control->control_draw = &swift_draw_base;
	control->control_mouse_down = &swift_handle_progressbar_click;
	control->control_update_pos = &swift_handle_control_move;
	control->control_unload = &swift_unload_base;
	control->sprite.tex = swift_add_texture(background->data);
	control->sprite.shader = swift_get_shader("swift_basic_shader");

	control->value = value;

	bar->sprite.tex = swift_add_texture(hbar->data);
	bar->sprite.shader = swift_get_shader("swift_basic_shader");
	bar->pos.xy.x = .1f;
	bar->pos.xy.y = .1f;
	bar->sprite.size.wh.w = ((control->sprite.size.wh.w / 100.f) * value);
	bar->sprite.size.wh.h = bar->sprite.tex->size.wh.h;
	bar->sprite.tex_pos.xywh.h = bar->sprite.tex->size.wh.h;
	bar->sprite.tex_pos.xywh.w = ((bar->sprite.tex->size.wh.w / 100.f) * value);
	bar->type = SWIFT_CONTROL_HPROGRESSBAR;
	bar->control_draw = &swift_draw_base;
	bar->control_mouse_down = &swift_handle_progressbar_click;
	bar->control_update_pos = &swift_handle_progress_move;
	bar->control_unload = &swift_unload_base;

	control->control = bar;

	if (visible)
		swift_widget_add(parent, control);
	else
		swift_widget_add_hidden(parent, control);

	swift_widget_add(control, bar);

	control->sprite.pos.xyz.x = control->pos.xy.x + parent->sprite.pos.xyz.x;
	control->sprite.pos.xyz.y = control->pos.xy.y + parent->sprite.pos.xyz.y;
	control->sprite.pos.xyz.z = pos.xyz.z;

	bar->sprite.pos.xyz.x = bar->pos.xy.x + control->sprite.pos.xyz.x;
	bar->sprite.pos.xyz.y = bar->pos.xy.y + control->sprite.pos.xyz.y;
	bar->sprite.pos.xyz.z = pos.xyz.z;

	swift_del_str(background);
	swift_del_str(hbar);
}

void swift_create_vprogressbar(swift_widget_t *control, swift_widget_t *parent,
	swift_vec3f_t pos, swift_vec2f_t hw, size_t value,
		swift_str_t *background, swift_str_t *vbar, bool visible)
{
	swift_widget_t *bar = calloc(1, sizeof(swift_widget_t));

	if (!bar) {
		fprintf(stderr, "POINTER_NULL, bar in swift_create_vprogressbar.\n");
		return;
	}

	swift_init_widget(control);
	control->pos.xy.x = pos.xyz.x;
	control->pos.xy.y = pos.xyz.y;
	control->sprite.size = hw;
	control->sprite.tex_pos.xywh.h = hw.wh.h;
	control->sprite.tex_pos.xywh.w = hw.wh.w;
	control->type = SWIFT_CONTROL_VPROGRESSBAR;
	control->control_draw = &swift_draw_base;
	control->control_mouse_down = &swift_handle_progressbar_click;
	control->control_update_pos = &swift_handle_control_move;
	control->control_unload = &swift_unload_base;
	control->sprite.tex = swift_add_texture(background->data);
	control->sprite.shader = swift_get_shader("swift_basic_shader");

	control->value = value;

	swift_init_widget(bar);

	bar->sprite.tex = swift_add_texture(vbar->data);
	bar->sprite.shader = swift_get_shader("swift_basic_shader");
	bar->pos.xy.x = .1f;
	bar->pos.xy.y = .1f;
	bar->sprite.size.wh.h = (control->sprite.size.wh.h / 100.f) * value;
	bar->sprite.size.wh.w = bar->sprite.tex->size.wh.w;
	bar->sprite.tex_pos.xywh.h = (bar->sprite.tex->size.wh.h / 100.f) * value;
	bar->sprite.tex_pos.xywh.w = bar->sprite.tex->size.wh.w;
	bar->type = SWIFT_CONTROL_VPROGRESSBAR;
	bar->control_draw = &swift_draw_base;
	bar->control_mouse_down = &swift_handle_progressbar_click;
	bar->control_update_pos = &swift_handle_progress_move;
	bar->control_unload = &swift_unload_base;

	control->control = bar;

	if (visible)
		swift_widget_add(parent, control);
	else
		swift_widget_add_hidden(parent, control);

	swift_widget_add(control, bar);

	control->sprite.pos.xyz.x = control->pos.xy.x + parent->sprite.pos.xyz.x;
	control->sprite.pos.xyz.y = control->pos.xy.y + parent->sprite.pos.xyz.y;
	control->sprite.pos.xyz.z = pos.xyz.z;

	bar->sprite.pos.xyz.x = bar->pos.xy.x + control->sprite.pos.xyz.x;
	bar->sprite.pos.xyz.y = bar->pos.xy.y + control->sprite.pos.xyz.y;
	bar->sprite.pos.xyz.z = pos.xyz.z;

	swift_del_str(background);
	swift_del_str(vbar);
}
