#include <swift/gui.h>
#include <swift/io.h>
#include <swift/window.h>

static void swift_link_radio(swift_widget_t *main, swift_widget_t *control)
{
	if (main && control) {
		swift_radio_t *data = main->control;
		swift_radio_t *control_data = control->control;

		if (data && control_data) {
			swift_array_push(&data->arr, &control);

			control_data->main = main;
		}
	}
}

static void swift_reset_radio(swift_widget_t *control)
{
	swift_radio_t *data = control->control;
	swift_array_iter_t iter;
	swift_widget_t *wgt;

	if (data->main)
		data = data->main->control;

	for (iter = swift_array_first(&data->arr); iter.data;
		swift_array_next(&iter)) {
		wgt = *(swift_widget_t **)iter.data;
		wgt->action.checked = false;
		wgt->sprite.tex_pos.xywh.x = 0;
	}
}

static void swift_handle_radio_click(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	swift_radio_t *data = control->control;

	if (button == 0 && pressed == true) {
		if (data->main || data->arr.data)
			swift_reset_radio(control);

		control->action.checked = true;
	}

	if (control->action.checked)
		control->sprite.tex_pos.xywh.x = control->sprite.size.wh.w;

	control->mouse_down(wgts, control, button, pressed);
}

static void swift_unload_radio(swift_widgets_t *wgts, swift_widget_t *control)
{
	swift_radio_t *init_radio = control->control;

	if (init_radio) {
		init_radio->main = NULL;
		swift_term_array(&init_radio->arr);

		if (init_radio->lbl)
			SWIFT_FREE(init_radio->lbl);
	}

	swift_unload_base(wgts, control);
}

void swift_create_radio(swift_widget_t *control, swift_widget_t *parent,
	swift_vec3f_t pos, swift_vec2f_t hw, swift_str_t *img_path,
		swift_widget_t *linkparent, bool checked, bool visible)
{
	swift_radio_t *init_radio = calloc(1, sizeof(swift_radio_t));

	if (!init_radio) {
		fprintf(stderr, "POINTER_NULL, init_radio in swift_create_radio.\n");
		return;
	}

	swift_init_widget(control);
	control->pos.xy.x = pos.xyz.x;
	control->pos.xy.y = pos.xyz.y;
	control->sprite.size = hw;
	control->sprite.tex_pos.xywh.w = hw.wh.w;
	control->sprite.tex_pos.xywh.h = hw.wh.h;
	control->type = SWIFT_CONTROL_RADIO;
	control->control_draw = &swift_draw_base;
	control->control_mouse_down = &swift_handle_radio_click;
	control->control_update_pos = &swift_handle_control_move;
	control->control_unload = &swift_unload_radio;
	control->sprite.tex = swift_add_texture(img_path->data);
	control->sprite.shader = swift_get_shader("swift_basic_shader");

	if (checked) {
		control->action.checked = true;
		control->sprite.tex_pos.xywh.x = control->sprite.size.wh.w;
	}

	init_radio->main = NULL;
	control->control = init_radio;

	if (visible)
		swift_widget_add(parent, control);
	else
		swift_widget_add_hidden(parent, control);

	control->sprite.pos.xyz.x = control->pos.xy.x + parent->sprite.pos.xyz.x;
	control->sprite.pos.xyz.y = control->pos.xy.y + parent->sprite.pos.xyz.y;
	control->sprite.pos.xyz.z = pos.xyz.z;

	if (linkparent)
		swift_link_radio(linkparent, control);
	else {
		swift_init_array(&init_radio->arr, NULL, sizeof(swift_widget_t *));
		swift_array_push(&init_radio->arr, &control);
	}

	swift_del_str(img_path);
}

void swift_create_radio_lbl(swift_widget_t *control, swift_widget_t *parent,
	swift_vec3f_t pos, swift_vec2f_t hw, swift_str_t *img_path,
		swift_widget_t *linkparent, swift_str_t *text, swift_font_t *font,
			swift_vec4f_t color, bool checked, size_t font_size, bool visible)
{

	swift_create_radio(control, parent, pos, hw, img_path, linkparent, checked,
		visible);

	if (text) {
		swift_radio_t *init_radio = control->control;

		if (!(init_radio->lbl = calloc(1, sizeof(swift_widget_t)))) {
			eprintf("lbl in swift_create_button could not be made.\n");
			return;
		}

		swift_create_label(init_radio->lbl, control,
			SWIFT_VEC3F(hw.wh.w + 2, 0, pos.xyz.z), hw, color, font, 64, text,
				font_size, true);

		init_radio->lbl->action.can_click_behind = true;
	}
}
