#include <swift/gui.h>
#include <swift/io.h>
#include <swift/window.h>
#include <swift/data.h>
#include <swift/string.h>

static void swift_text_buf_add(swift_text_t *str, swift_sprite_t *sprite,
	size_t count)
{
	if (str->buf->count > 0 && count < str->buf->count)
		swift_array_set(str->buf, count, sprite);
	else
		swift_array_insert(str->buf, count, sprite);
}

size_t swift_get_string_width(swift_str_t *text, float size,
	swift_font_t *font)
{
	swift_str_iter_t iter;
	size_t ret = 0;
	swift_sym_t sym;

	if (!text || !text->data || text->size == 0 || !font)
		return 0;

	iter = swift_str_first(text);

	do {
		sym = swift_sym_resize(swift_get_glyph(font, iter.code), size);

		if (sym.code != ' ')
			ret += sym.width + sym.left;
		else
			ret += sym.ax + sym.left;

	} while(swift_str_next(&iter));

	return ret;
}

void swift_text_set(swift_text_t *data, swift_vec2f_t offset,
	swift_vec2f_t maxchars, swift_font_t *font, swift_vec4f_t color,
		swift_str_t *string, size_t size)
{
	data->color = color;
	swift_del_str(data->str);
	data->str = swift_new_strf("%s", string->data);
	data->maxchars = maxchars;
	data->offset = offset;
	data->font = font;
	data->font_size = size;

	if (!data->buf) {
		if (!(data->buf = swift_new_array_size(NULL, sizeof(swift_sprite_t),
			2)))
			eprintf("Data Buf in Text Set Equaled Nothing /n");
	}
}

void swift_create_text_vertex(swift_text_t *str, swift_widget_t *control)
{
	float texty = control->sprite.pos.xyz.y + str->font_size;
	float x = control->sprite.pos.xyz.x + str->offset.xy.x, y = texty;
	swift_vec3f_t pos = SWIFT_VEC3F(0, 0, control->sprite.pos.xyz.z);
	swift_str_iter_t iter;
	swift_sym_t sym, *_sym;
	swift_shader_t *shader = swift_get_shader("swift_text_shader");
	swift_sprite_t sprite = SWIFT_SPRITE(.rotation = 0, .col = str->color,
		.tex = &str->font->texture, .shader = shader);
	size_t total_line_width = 0, amount = 0, count = 0, lines = 1;

	str->text_hw.wh.h = 0;
	iter = swift_str_first(str->str);

	do {
		_sym = swift_get_glyph(str->font, iter.code);
		sym = swift_sym_resize(_sym, str->font_size);

		pos.xyz.x = (x + sym.left);
		pos.xyz.y = (y - sym.top);
		x += sym.ax;
		y = texty + sym.ay;

		if (control->action.is_multi_lined) {
			amount++;

			total_line_width += (sym.left >= 0) ? (sym.ax + sym.left) : sym.ax;

			if (!sym.width || !sym.height) {
				if (total_line_width <= sym.ax) {
					total_line_width = 0;
					x = control->sprite.pos.xyz.x + str->offset.xy.x;
				}
				else {
					if (total_line_width >= control->sprite.size.wh.w -
						str->offset.xy.x) {
						lines++;
						total_line_width = 0;
						amount = 0;
						texty = control->sprite.pos.xyz.y + (str->font_size  *
							lines);
						x = control->sprite.pos.xyz.x + str->offset.xy.x;
						y = texty + str->offset.xy.y;
						str->text_hw.wh.h = str->font_size * lines;
					}
				}

				continue;
			}
			else {
				if ((amount >= str->maxchars.xy.x && str->maxchars.xy.x > 0) ||
					total_line_width >= control->sprite.size.wh.w -
						str->offset.xy.x) {
					lines++;
					total_line_width = 0;
					amount = 0;
					texty = control->sprite.pos.xyz.y + (str->font_size  *
						lines);
					x = control->sprite.pos.xyz.x + str->offset.xy.x;
					y = texty + str->offset.xy.y;
					str->text_hw.wh.h = str->font_size * lines;
				}
			}

			if (str->text_hw.wh.h >= control->sprite.size.wh.w - str->offset.xy.y)
				break;
		}
		else {
			if (!sym.width || !sym.height)
				continue;
		}

		sprite.tex_pos.xywh.x = _sym->x;
		sprite.tex_pos.xywh.y = _sym->y;
		sprite.tex_pos.xywh.w = _sym->width;
		sprite.tex_pos.xywh.h = _sym->height;
		sprite.size.wh.w = sym.width;
		sprite.size.wh.h = sym.height;
		sprite.pos = pos;

		swift_text_buf_add(str, &sprite, count);

		count++;
	} while (swift_str_next(&iter));

	if (count < str->buf->count)
		swift_array_remove_range(str->buf, count, count - str->buf->count);
}

void swift_create_chatbox_text_vertex(swift_text_t *str,
	swift_widget_t *control, swift_vec4f_t color, swift_str_t *new_str)
{
	float texty = control->sprite.pos.xyz.y + str->font_size;
	float x = control->sprite.pos.xyz.x + str->offset.xy.x, y = texty;
	swift_vec3f_t pos = SWIFT_VEC3F(0, 0, control->sprite.pos.xyz.z);
	swift_str_iter_t iter;
	swift_sym_t sym, *_sym;
	swift_shader_t *shader = swift_get_shader("swift_text_shader");
	swift_sprite_t sprite = SWIFT_SPRITE(.rotation = 0, .col = color,
		.tex = &str->font->texture, .shader = shader);
	size_t count = str->buf->count;

	str->text_hw.wh.h = 0;
	iter = swift_str_first(new_str);

	if (count > 0) {
		swift_array_iter_t buf_iter = swift_array_last(str->buf);
		swift_sprite_t *sprite_last = buf_iter.data;
		pos = sprite_last->pos;
	}

	do {
		_sym = swift_get_glyph(str->font, iter.code);
		sym = swift_sym_resize(_sym, str->font_size);

		pos.xyz.x = (x + sym.left);
		pos.xyz.y = (y - sym.top);
		x += sym.ax;
		y = texty + sym.ay;

		if (!sym.width || !sym.height)
			continue;

		sprite.tex_pos.xywh.x = _sym->x;
		sprite.tex_pos.xywh.y = _sym->y;
		sprite.tex_pos.xywh.w = _sym->width;
		sprite.tex_pos.xywh.h = _sym->height;
		sprite.size.wh.w = sym.width;
		sprite.size.wh.h = sym.height;
		sprite.pos = pos;

		swift_text_buf_add(str, &sprite, count);

		count++;
	} while (swift_str_next(&iter));

	if (count < str->buf->count)
		swift_array_remove_range(str->buf, count, count - str->buf->count);
}

void swift_lbl_col_update(swift_text_t *str, swift_widget_t *control)
{
	swift_array_iter_t iter;
	swift_sprite_t *sprite;

	iter = swift_array_first(str->buf);

	do {
		sprite = iter.data;

		sprite->col = str->color;

	} while (swift_array_next(&iter));
}

void swift_lbl_pos_update(swift_text_t *str, swift_widget_t *control)
{
	float texty = control->sprite.pos.xyz.y + str->font_size;
	float x = control->sprite.pos.xyz.x + str->offset.xy.x;
	float y = texty + str->offset.xy.y;
	swift_vec3f_t pos = SWIFT_VEC3F(0, 0, control->sprite.pos.xyz.z);
	swift_array_iter_t iter;
	swift_str_iter_t iter2;
	swift_sym_t sym;
	swift_sprite_t *sprite;
	size_t amount, total_line_width, lines = 1;

	iter = swift_array_first(str->buf);
	iter2 = swift_str_first(str->str);

	do {
		sprite = iter.data;

		sym = swift_sym_resize(swift_get_glyph(str->font, iter2.code),
			str->font_size);

		pos.xyz.x = (x + sym.left);
		pos.xyz.y = (y - sym.top);
		x += sym.ax;
		y = texty + sym.ay;

		if (control->action.is_multi_lined) {
			amount++;

			total_line_width += (sym.left >= 0) ? (sym.ax + sym.left) :
				sym.ax;

			if (!sym.width || !sym.height) {
				if (total_line_width <= sym.ax) {
					total_line_width = 0;
					x = control->sprite.pos.xyz.x + str->offset.xy.x;
				}
				else {
					if (total_line_width >= control->sprite.size.wh.w -
						str->offset.xy.x) {
						lines++;
						total_line_width = 0;
						amount = 0;
						texty = control->sprite.pos.xyz.y + (str->font_size *
							lines);
						x = control->sprite.pos.xyz.x + str->offset.xy.x;
						y = texty + str->offset.xy.y;
						str->text_hw.wh.h = str->font_size * lines;
					}
				}

				continue;
			}
			else {
				if ((amount >= str->maxchars.xy.x && str->maxchars.xy.x > 0) ||
					total_line_width >= control->sprite.size.wh.w -
						str->offset.xy.x) {
					lines++;
					total_line_width = 0;
					amount = 0;
					texty = control->sprite.pos.xyz.y + (str->font_size *
						lines);
					x = control->sprite.pos.xyz.x + str->offset.xy.x;
					y = texty + str->offset.xy.y;
					str->text_hw.wh.h = str->font_size * lines;
				}
			}

			if (str->text_hw.wh.h >= control->sprite.size.wh.w -
				str->offset.xy.y)
				break;
		}
		else {
			if (!sym.width || !sym.height)
				continue;
		}

		sprite->pos = pos;

		swift_array_next(&iter);
	} while (swift_str_next(&iter2));
}

void swift_textbox_pos_update(swift_text_t *str, swift_widget_t *control)
{
	float x = control->sprite.pos.xyz.x + str->offset.xy.x;
	float y = control->sprite.pos.xyz.y + str->font_size + str->offset.xy.y;
	swift_vec3f_t pos = SWIFT_VEC3F(0, 0, control->sprite.pos.xyz.z);
	swift_array_iter_t iter;
	swift_str_iter_t iter2;
	swift_textbox_t *text = control->control;
	swift_sym_t sym;
	swift_sprite_t *sprite;

	if (control->action.is_password)
		sym = swift_sym_resize(swift_get_glyph(str->font, '*'), str->font_size);

	if (str->str->data) {
		iter = swift_array_first(str->buf);
		iter2 = text->start;

		do {
			sprite = iter.data;

			if (!control->action.is_password)
				sym = swift_sym_resize(swift_get_glyph(str->font, iter2.code),
				str->font_size);

			pos.xyz.x = (x + sym.left);
			pos.xyz.y = (y - sym.top);
			x += sym.ax;
			y = control->sprite.pos.xyz.y + str->font_size +
					str->offset.xy.y + sym.ay;

			if (!sym.width || !sym.height)
				continue;

			sprite->pos = pos;

			swift_array_next(&iter);
		} while (swift_str_next(&iter2));
	}

	if (str->str->count > 0)
		text->cursor.pos.xyz.x = sprite->pos.xyz.x + sprite->size.wh.w + 2;
	else
		text->cursor.pos.xyz.x = control->sprite.pos.xyz.x + str->offset.xy.x;
}

void swift_textbox_text_update(swift_text_t *str, swift_widget_t *control)
{
	float x = control->sprite.pos.xyz.x + str->offset.xy.x;
	float y = control->sprite.pos.xyz.y + str->font_size + str->offset.xy.y;
	swift_vec3f_t pos = SWIFT_VEC3F(0, 0, control->sprite.pos.xyz.z);
	swift_str_iter_t iter;
	swift_sym_t sym, *_sym;
	swift_shader_t *shader = swift_get_shader("swift_text_shader");
	swift_sprite_t sprite = SWIFT_SPRITE(.rotation = 0, .col = str->color,
		.tex = &str->font->texture, .shader = shader);
	swift_textbox_t *text = control->control;
	size_t count = 0;

	if (control->action.is_password) {
		_sym = swift_get_glyph(str->font, '*');
		sym = swift_sym_resize(_sym, str->font_size);
	}

	if (str->str->data) {
		text->start.code = swift_str_pos_get(text->start.str, text->start.pos);
		iter = text->start;

		if (text->start.str->count > 0) {
			do {
				if (!control->action.is_password) {
					_sym = swift_get_glyph(str->font, iter.code);
					sym = swift_sym_resize(_sym, str->font_size);
				}

				pos.xyz.x = (x + sym.left);
				pos.xyz.y = (y - sym.top);

				x += sym.ax;
				y = control->sprite.pos.xyz.y + str->font_size +
					str->offset.xy.y + sym.ay;

				if (!sym.width || !sym.height)
					continue;

				sprite.tex_pos.xywh.x = _sym->x;
				sprite.tex_pos.xywh.y = _sym->y;
				sprite.tex_pos.xywh.w = _sym->width;
				sprite.tex_pos.xywh.h = _sym->height;
				sprite.size.wh.w = sym.width;
				sprite.size.wh.h = sym.height;
				sprite.pos = pos;

				swift_text_buf_add(str, &sprite, count);

				count++;
			} while (swift_str_next(&iter));
		}

		if (count < str->buf->count)
			swift_array_remove_range(str->buf, count, count - str->buf->count);
	}

	if (count > 0)
		text->cursor.pos.xyz.x = sprite.pos.xyz.x + sprite.size.wh.w + 2;
	else
		text->cursor.pos.xyz.x = control->sprite.pos.xyz.x + str->offset.xy.x;
}
