#include <swift/gui.h>
#include <swift/io.h>
#include <swift/window.h>
#include <swift/time.h>
#include <swift/data.h>
#include <swift/string.h>

static void swift_draw_stextbox_text(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_textbox_t *data = control->control;
	swift_text_t *str = &data->str;
	swift_array_iter_t iter;

	swift_render(wgts->window, &control->sprite);

	swift_set_clip_box(wgts->window, SWIFT_VEC4F(control->sprite.pos.xyz.x,
		wgts->window->screen_box.xywh.h - control->sprite.pos.xyz.y -
			control->sprite.size.wh.h, control->sprite.pos.xyz.x +
				control->sprite.size.wh.w, wgts->window->screen_box.xywh.h -
					control->sprite.pos.xyz.y));

	for (iter = swift_array_first(str->buf); iter.data;
		swift_array_next(&iter)) {
		swift_render(wgts->window, iter.data);
	}

	if (data->visible) {
		double tick = swift_tick_count();

		if (control != wgts->focused)
			data->visible = false;

		if (data->time > tick) {
			swift_render(wgts->window, &data->cursor);
			data->blink = tick + .2;
		}
		else {
			if (data->blink < tick) {
				data->time = tick + .8;
				data->blink = tick + .2;
			}
		}
	}

	swift_set_clip_box(wgts->window, wgts->window->screen_box);
}

static void swift_handle_stextbox_click(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	swift_textbox_t *data = control->control;

	if (data->visible == false)
		data->visible = true;

	control->mouse_down(wgts, control, button, pressed);
}

static void swift_handle_stextbox_move(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_textbox_t *data = control->control;

	swift_handle_control_move(wgts, control);

	swift_textbox_pos_update(&data->str, control);
}

static void swift_handle_stextbox_input(swift_widgets_t *wgts,
	swift_widget_t *control, int key, int pressed)
{
	swift_textbox_t *text = control->control;
	swift_text_t *data = &text->str;
	swift_str_iter_t iter, first;
	swift_sym_t ch;

	if (text->visible) {
		if (control->action.is_password)
			ch = swift_sym_resize(swift_get_glyph(data->font, '*'),
				data->font_size);
		else
			ch = swift_sym_resize(swift_get_glyph(data->font, key),
				data->font_size);

		switch (key) {
			case SWIFT_KEY_ENTER: break;
			case SWIFT_KEY_BACKSPACE: {
				if (data->str && data->str->count > 0) {
					iter = swift_str_last(data->str);
					first = swift_str_first(data->str);

					if (text->start.pos != first.pos)
						swift_str_prev(&text->start);

					data->text_hw.wh.w -= ch.code != ' ' ?
						(ch.width + ch.left) : (ch.ax + ch.left);

					swift_remove_str(data->str, iter.pos, iter.size);

					if (text->count == 1)
						data->text_hw.wh.w = 0;

					text->count--;
					swift_textbox_text_update(data, control);
				}
			} break;

			default: {
				if (data->str && text->count < data->maxchars.xy.x) {
					data->text_hw.wh.w += ch.code != ' ' ?
						(ch.width + ch.left) : (ch.ax + ch.left);

					if (data->text_hw.wh.w >= control->sprite.size.wh.w -
						data->offset.xy.x - 4)
						swift_str_next(&text->start);

					swift_append_uchar(data->str, key);

					if (text->count == 0)
						text->start = swift_str_first(data->str);

					text->count++;
					swift_textbox_text_update(data, control);
				}
			}
		}
	}

	control->key_down(wgts, control, key, pressed);
}

static void swift_unload_textbox(swift_widgets_t *wgts, swift_widget_t *control)
{
	swift_textbox_t *text = control->control;
	swift_del_array(text->str.buf);
	swift_del_str(text->str.str);

	swift_unload_base(wgts, control);
}

void swift_create_stextbox(swift_widget_t *control, swift_widget_t *parent,
	swift_vec3f_t pos, swift_vec2f_t hw, swift_vec2f_t offset,
		swift_vec4f_t color, size_t maxchars, swift_font_t *font,
			size_t font_size, swift_str_t *img_path, bool is_pass,
				bool visible)
{
	swift_textbox_t *text = calloc(1, sizeof(swift_textbox_t));

	if (!text) {
		eprintf("POINTER_NULL, swift_create_stextbox.\n");
		return;
	}

	swift_init_widget(control);
	control->sprite.tex = swift_add_texture(img_path->data);
	control->pos.xy.x = pos.xyz.x;
	control->pos.xy.y = pos.xyz.y;
	control->sprite.size = hw;
	control->sprite.tex_pos.xywh.h = control->sprite.tex->size.wh.h;
	control->sprite.tex_pos.xywh.w = control->sprite.tex->size.wh.w;
	control->type = SWIFT_CONTROL_TEXTBOX;
	control->control_draw = &swift_draw_stextbox_text;
	control->control_mouse_down = &swift_handle_stextbox_click;
	control->control_update_pos = &swift_handle_stextbox_move;
	control->control_key_down = &swift_handle_stextbox_input;
	control->control_unload = &swift_unload_textbox;
	control->action.can_focus = true;
	control->sprite.shader = swift_get_shader("swift_basic_shader");

	if (is_pass)
		control->action.is_password = true;

	swift_text_set(&text->str, offset, SWIFT_VEC2F(maxchars, 0),
		font, color, swift_new_strf(""), font_size);

	text->time = 0;
	text->visible = false;
	text->cursor = SWIFT_SPRITE(0);
	text->cursor.size.wh.h = font_size + 2;
	text->cursor.size.wh.w = 2;
	text->cursor.col = SWIFT_VEC4F(0, 0, 0, 1);
	text->cursor.shader = swift_get_shader("swift_notex_shader");
	text->cursor.tex = NULL;
	text->start.code = 0;
	text->count = 0;

	control->control = text;

	if (visible)
		swift_widget_add(parent, control);
	else
		swift_widget_add_hidden(parent, control);

	control->sprite.pos.xyz.x = control->pos.xy.x + parent->sprite.pos.xyz.x;
	control->sprite.pos.xyz.y = control->pos.xy.y + parent->sprite.pos.xyz.y;
	control->sprite.pos.xyz.z = pos.xyz.z;

	text->cursor.pos.xyz.x = control->sprite.pos.xyz.x + offset.xy.x;
	text->cursor.pos.xyz.y = control->sprite.pos.xyz.y + offset.xy.y;
	text->cursor.pos.xyz.z = pos.xyz.z;

	swift_del_str(img_path);
}

swift_str_t *swift_get_textbox_string(swift_widget_t *control)
{
	swift_textbox_t *text = control->control;
	swift_text_t *data = &text->str;

	if (data)
		return data->str;
	else
		return NULL;
}
