#include <swift/gui.h>
#include <swift/io.h>
#include <swift/window.h>
#include "input.h"

static void swift_widget_clear_hidden(swift_widgets_t *wgts,
	swift_widget_t *parent);
static void swift_widget_clear_shown(swift_widgets_t *wgts,
	swift_widget_t *parent);

static void swift_widget_clear_shown(swift_widgets_t *wgts,
	swift_widget_t *parent)
{
	swift_list_iter_t iter;
	swift_widget_t *child;

	for (iter = swift_list_first(&parent->shown); iter.node;
		swift_list_next(&iter)) {
		child = SWIFT_CONTAINER_OF(iter.node, swift_widget_t, snode);

		if (!swift_list_is_empty(&child->shown))
			swift_widget_clear_shown(wgts, child);

		if (!swift_list_is_empty(&child->hidden))
			swift_widget_clear_hidden(wgts, child);

		if (child->type != SWIFT_CONTROL_ROOT) {
			if (child == wgts->focused)
				wgts->focused = NULL;

			if (child == wgts->over_widget)
				wgts->over_widget = NULL;

			child->parent = NULL;
			swift_list_remove(&parent->shown, iter.node);
			child->control_unload(wgts, child);
		}
	}
}

static void swift_widget_clear_hidden(swift_widgets_t *wgts,
	swift_widget_t *parent)
{
	swift_list_iter_t iter;
	swift_widget_t *child;

	for (iter = swift_list_first(&parent->hidden); iter.node;
		swift_list_next(&iter)) {
		child = SWIFT_CONTAINER_OF(iter.node, swift_widget_t, hnode);

		if (!swift_list_is_empty(&child->shown))
			swift_widget_clear_shown(wgts, child);

		if (!swift_list_is_empty(&child->hidden))
			swift_widget_clear_hidden(wgts, child);

		if (child->type != SWIFT_CONTROL_ROOT) {
			child->parent = NULL;
			swift_list_remove(&parent->hidden, iter.node);
			child->control_unload(wgts, child);
		}
	}
}

static void swift_widget_has_exited(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	if (!wgts->over_widget) {
		wgts->over_widget = control;
		return;
	}

	if (control != wgts->over_widget) {
		if (wgts->over_widget->action.mouse_over) {
			wgts->over_widget->action.mouse_over = false;
			wgts->over_widget->control_mouse_exit(wgts, wgts->over_widget);
		}

		wgts->over_widget = control;
	}
}

static void swift_widget_mouse_over(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_list_iter_t iter;
	swift_widget_t *child;

	for (iter = swift_list_last(&control->shown); iter.node;
		swift_list_prev(&iter)) {
		child = SWIFT_CONTAINER_OF(iter.node, swift_widget_t, snode);

		if (!child)
			continue;

		if (swift_widget_rect_contains(wgts, child)) {
			if (!swift_list_is_empty(&child->shown)) {
				swift_widget_mouse_over(wgts, child);
				return;
			}
			else {
				if (swift_widget_usable(child)) {
					if (child->action.can_click_behind) {
						swift_widget_has_exited(wgts, child->parent);
						child->parent->action.mouse_over = true;
						child->parent->control_mouse_over(wgts,
							child->parent);
					}
					else {
						swift_widget_has_exited(wgts, child);
						child->action.mouse_over = true;
						child->control_mouse_over(wgts, child);
					}
					return;
				}
			}
		}
		else
			swift_widget_has_exited(wgts, child);
	}

	if (swift_widget_usable(control)) {
		if (control->action.can_click_behind) {
			swift_widget_has_exited(wgts, control->parent);
			control->parent->action.mouse_over = true;
			control->parent->control_mouse_over(wgts, control->parent);
		}
		else {
			swift_widget_has_exited(wgts, control);
			control->action.mouse_over = true;
			control->control_mouse_over(wgts, control);
		}
	}
}

static void swift_widget_position_update(swift_widgets_t *wgts,
	swift_widget_t *parent)
{
	swift_list_iter_t iter;
	swift_widget_t *child;

	parent->control_update_pos(wgts, parent);

	for (iter = swift_list_first(&parent->shown); iter.node;
		swift_list_next(&iter)) {
		child = SWIFT_CONTAINER_OF(iter.node, swift_widget_t, snode);

		if (!swift_list_is_empty(&child->shown))
			swift_widget_position_update(wgts, child);
		else
			child->control_update_pos(wgts, child);
	}
}

static void swift_widget_clear_parent(swift_widgets_t *wgts,
	swift_widget_t *parent)
{
	if (!wgts || !parent)
		return;

	if (!swift_list_is_empty(&parent->hidden))
		swift_widget_clear_hidden(wgts, parent);

	if (!swift_list_is_empty(&parent->shown))
		swift_widget_clear_shown(wgts, parent);

	if (parent->control)
		SWIFT_FREE(parent->control);

	swift_term_list(&parent->hidden);
	swift_term_list(&parent->shown);
}

static bool swift_widget_is_parent_focused(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	swift_widget_t *parent = control->parent;

	if (control->action.always_useable)
		return true;

	while (parent && parent->type != SWIFT_CONTROL_ROOT) {
			if (parent->action.can_focus) {
				if (parent->action.is_focused)
					return true;
				else {
					swift_widget_manual_focused(wgts, parent);

					if (parent->action.focus_click) {
						parent->action.clicked = true;
						parent->control_mouse_down(wgts, parent, wgts->button,
							wgts->clicked);
					}

					return true;
				}
			}
			else
				parent = parent->parent;
	}
	return false;
}

static void swift_widget_set_release(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	if (control->action.window_moveable)
		wgts->moving = false;

	control->action.clicked = false;
	control->control_mouse_up(wgts, control, wgts->button, wgts->clicked);
}

static void swift_widget_set_clicked(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	if (control->action.window_moveable)
		wgts->moving = true;

	if (control->action.can_click_behind) {
		control->parent->action.clicked = true;
		control->parent->control_mouse_down(wgts, control->parent, wgts->button,
			wgts->clicked);
		return;
	}

	control->action.clicked = true;
	control->control_mouse_down(wgts, control, wgts->button,
		wgts->clicked);
}

static void swift_widget_set_focused(swift_widgets_t *wgts,
	swift_widget_t *control)
{

	swift_list_remove(&control->parent->shown, &control->snode);
	swift_list_insert_after(&control->parent->shown, &control->snode);

	if (wgts->focused)
		wgts->focused->action.is_focused = false;

	control->action.is_focused = true;
	wgts->focused = control;
	control->action.clicked = true;

	if (control->action.moveable) {
		if (swift_widget_rect_contains(wgts, control))
			control->action.moving = true;
	}

	control->control_mouse_down(wgts, control, wgts->button,
		wgts->clicked);
}

static void swift_set_widget_mouse_press_event(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	if (control->action.can_focus)
		swift_widget_set_focused(wgts, control);
	else {
		if (swift_widget_is_parent_focused(wgts, control) || control->type ==
			SWIFT_CONTROL_ROOT)
			swift_widget_set_clicked(wgts, control);
	}
}

static void swift_widget_init_mouse_press(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	if (control->action.can_click_behind) {
		if (!control->parent && control->parent->type != SWIFT_CONTROL_ROOT) {
			control->parent->control_mouse_down(wgts, control->parent, button,
				pressed);
			control->parent->mouse_down(wgts, control->parent, button,
				pressed);
		}
	}
}

static void swift_widget_init_mouse_release(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	if (control->action.can_click_behind) {
		if (!control->parent && control->parent->type != SWIFT_CONTROL_ROOT) {
			control->parent->control_mouse_down(wgts, control->parent, button,
				pressed);
			control->parent->mouse_down(wgts, control->parent, button,
				pressed);
		}
	}
}

static void swift_widget_init_mouse_wheel(swift_widgets_t *wgts,
	swift_widget_t *control, int moved) {}

static void swift_widget_init_key_pressed(swift_widgets_t *wgts,
	swift_widget_t *control, int key, int pressed) {}

static void swift_widget_init_draw(swift_widgets_t *wgts,
	swift_widget_t *control) {}

static void swift_widget_init_mouse_over(swift_widgets_t *wgts,
	swift_widget_t *control) {}

static void swift_widget_init_mouse_exit(swift_widgets_t *wgts,
	swift_widget_t *control) {}

static void swift_widget_init_update_pos(swift_widgets_t *wgts,
	swift_widget_t *control) {}

static void swift_widget_init_unload(swift_widgets_t *wgts,
	swift_widget_t *control) {}

static void swift_widget_init_control_draw(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	control->draw(wgts, control);
}

static void swift_widget_init_control_mouse_press(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	control->mouse_down(wgts, control, button, pressed);
}

static void swift_widget_init_control_mouse_release(swift_widgets_t *wgts,
	swift_widget_t *control, int button, int pressed)
{
	control->mouse_up(wgts, control, button, pressed);
}

static void swift_widget_init_control_mouse_wheel(swift_widgets_t *wgts,
	swift_widget_t *control, int moved)
{
	control->mouse_wheel(wgts, control, moved);
}

static void swift_widget_init_control_key_pressed(swift_widgets_t *wgts,
	swift_widget_t *control, int key, int pressed)
{
	control->key_down(wgts, control, key, pressed);
}

static void swift_widget_init_control_update_pos(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	control->update_pos(wgts, control);
}

static void swift_widget_init_control_mouse_over(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	control->mouse_over(wgts, control);
}

static void swift_widget_init_control_mouse_exit(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	control->mouse_exit(wgts, control);
}

static void swift_widget_init_control_unload(swift_widgets_t *wgts,
	swift_widget_t *control)
{
	control->unload(wgts, control);
}

void swift_widget_move(swift_widgets_t *wgts, float x, float y)
{
	wgts->new_mouse_pos.xy.x = x;
	wgts->new_mouse_pos.xy.y = y;

	if (wgts->moving) {
		swift_vec2f_t pos = swift_get_window_pos(wgts->window);

		swift_set_window_pos(wgts->window, SWIFT_VEC2F(x + pos.xy.x -
				wgts->mouse_clicked.xy.x, y + pos.xy.y -
					wgts->mouse_clicked.xy.y));
	}
	else {
		if (wgts->focused) {
			if (wgts->focused->action.moving) {
				swift_vec2f_t pos = SWIFT_VEC2F(x - wgts->mouse_pos.xy.x,
					y - wgts->mouse_pos.xy.y);

				if (wgts->focused->pos.xy.x + pos.xy.x <= 0 ||
					wgts->focused->pos.xy.y + pos.xy.y <= 0)
					return;

				if (wgts->focused->pos.xy.x + wgts->focused->sprite.size.wh.w +
					pos.xy.x >= wgts->window->screen_box.xywh.w + 400 ||
						wgts->focused->pos.xy.y +
							wgts->focused->sprite.size.wh.h + pos.xy.y >=
								wgts->window->screen_box.xywh.h + 400)
					return;

				wgts->focused->pos.xy.x = wgts->focused->pos.xy.x + pos.xy.x;
				wgts->focused->pos.xy.y = wgts->focused->pos.xy.y + pos.xy.y;

				swift_widget_position_update(wgts, wgts->focused);
			}

			if (swift_widget_rect_contains(wgts, wgts->focused)) {
				swift_widget_mouse_over(wgts, wgts->focused);
				return;
			}
		}

		swift_widget_mouse_over(wgts, &wgts->root);
	}
}

bool swift_widget_check_focus(swift_widgets_t *wgts)
{
	if (wgts->focused) {
		if (wgts->focused->action.moving)
			wgts->focused->action.moving = false;

		if (swift_widget_rect_contains(wgts, wgts->focused))
			return true;
	}

	return false;
}

bool swift_init_widgets(swift_window_t *window, swift_widgets_t *wgts,
	void *user_data)
{
	if (!window || !wgts || !swift_create_gui_callbacks(wgts))
		return false;

	wgts->mouse_clicked = SWIFT_VEC2F(0, 0);
	wgts->mouse_pos = SWIFT_VEC2F(-1, -1);
	wgts->new_mouse_pos = SWIFT_VEC2F(0, 0);
	wgts->focused = wgts->over_widget = NULL;
	wgts->clicked = (wgts->button = 0);
	wgts->window = window;
	wgts->user_data = user_data;
	swift_init_widget(&wgts->root);
	wgts->root.type = SWIFT_CONTROL_ROOT;
	wgts->moving = wgts->clicked = false;

	window->user_data = wgts;
	swift_set_char_callback(window, &swift_gui_char_callback);
	swift_set_mods_callback(window, &swift_gui_mods_callback);
	swift_set_key_callback(window, &swift_gui_key_callback);
	swift_set_cursor_pos_callback(window, &swift_gui_mouse_pos_callback);
	swift_set_mouse_press_callback(window, &swift_gui_mouse_press_callback);
	swift_set_scroll_callback(window, &swift_gui_mouse_wheel_callback);
	swift_set_window_close_callback(window, &swift_gui_window_close);
	swift_set_window_pos_callback(window, &swift_gui_window_pos_callback);
	swift_set_window_resize_callback(window, &swift_gui_window_resize_callback);
	swift_set_window_refresh_callback(window,
		&swift_gui_window_refresh_callback);
	swift_set_window_focus_callback(window, &swift_gui_window_focus_callback);
	swift_set_iconify_callback(window, &swift_gui_iconify_callback);
	swift_set_buffer_resize_callback(window, &swift_gui_buffer_resize_callback);
	swift_set_cursor_entered_callback(window,
		&swift_gui_cursor_entered_callback);
	swift_set_drops_callback(window, &swift_gui_drops_callback);

	return true;
}

void swift_widget_hide(swift_widget_t *control)
{
	if (!control|| !control->parent|| control->type == SWIFT_CONTROL_ROOT)
		return;

	if (!swift_list_is_empty(&control->snode))
		swift_list_remove(&control->parent->shown, &control->snode);

	if (swift_list_is_empty(&control->hnode))
		swift_list_push(&control->parent->hidden, &control->hnode);
}

void swift_widget_show(swift_widget_t *control)
{
	if (!control || !control->parent || control->type == SWIFT_CONTROL_ROOT)
		return;

	if (!swift_list_is_empty(&control->hnode))
		swift_list_remove(&control->parent->hidden, &control->hnode);

	if (swift_list_is_empty(&control->snode))
		swift_list_push(&control->parent->shown, &control->snode);
}

void swift_widget_add(swift_widget_t *parent, swift_widget_t *child)
{
	if (!child || !parent)
		return;

	child->parent = parent;

	if (swift_list_is_empty(&child->snode))
		swift_list_push(&parent->shown, &child->snode);
}

void swift_widget_add_hidden(swift_widget_t *parent, swift_widget_t *child)
{
	if (!child || !parent)
		return;

	child->parent = parent;

	if (swift_list_is_empty(&child->hnode))
		swift_list_push(&parent->hidden, &child->hnode);
}

void swift_init_widget(swift_widget_t *control)
{
	if (!control)
		return;

	swift_init_list(&control->shown);
	swift_init_list(&control->snode);
	swift_init_list(&control->hidden);
	swift_init_list(&control->hnode);
	control->sprite = SWIFT_SPRITE(0);
	control->sprite.pos = SWIFT_VEC3F(0, 0, 1);
	control->sprite.col = SWIFT_VEC4F(1, 1, 1, 1);
	control->sprite.tex_pos = SWIFT_VEC4F(0, 0, 1, 1);
	control->sprite.size = SWIFT_VEC2F(1, 1);
	control->sprite.rotation = 0;
	control->sprite.tex = NULL;
	control->parent = NULL;
	control->control = NULL;
	control->data = NULL;
	control->type = SWIFT_CONTROL_EMPTY;
	control->action = SWIFT_WIDGET_FLAGS(false);
	control->draw = &swift_widget_init_draw;
	control->mouse_down = &swift_widget_init_mouse_press;
	control->mouse_up = &swift_widget_init_mouse_release;
	control->mouse_wheel = &swift_widget_init_mouse_wheel;
	control->key_down = &swift_widget_init_key_pressed;
	control->mouse_over = &swift_widget_init_mouse_over;
	control->mouse_exit = &swift_widget_init_mouse_exit;
	control->update_pos = &swift_widget_init_update_pos;
	control->unload = &swift_widget_init_unload;
	control->control_draw = &swift_widget_init_control_draw;
	control->control_mouse_down = &swift_widget_init_control_mouse_press;
	control->control_mouse_up = &swift_widget_init_control_mouse_release;
	control->control_mouse_wheel = &swift_widget_init_control_mouse_wheel;
	control->control_key_down = &swift_widget_init_control_key_pressed;
	control->control_update_pos = &swift_widget_init_control_update_pos;
	control->control_mouse_over = &swift_widget_init_control_mouse_over;
	control->control_mouse_exit = &swift_widget_init_control_mouse_exit;
	control->control_unload = &swift_widget_init_control_unload;
}

void swift_unload_widgets(swift_widgets_t *wgts)
{
	swift_widget_unload(wgts, &wgts->root);
	swift_destroy_gui_callbacks(wgts);
}

void swift_unload_widgets_from(swift_widgets_t *wgts, swift_widget_t *parent)
{
	swift_widget_clear_parent(wgts, parent);
}

void swift_widget_unload(swift_widgets_t *wgts, swift_widget_t *parent)
{
	swift_widget_clear_parent(wgts, parent);

	if (parent->type != SWIFT_CONTROL_ROOT) {
		if (!swift_list_is_empty(&parent->snode))
			swift_list_remove(&parent->parent->shown, &parent->snode);

		if (!swift_list_is_empty(&parent->hnode))
			swift_list_remove(&parent->parent->hidden, &parent->hnode);

		parent->control_unload(wgts, parent);
	}

	if (parent == wgts->focused)
		wgts->focused = NULL;

	if (parent == wgts->over_widget)
		wgts->over_widget = NULL;

	parent->parent = NULL;
}

void swift_widgets_render(swift_widgets_t *wgts, swift_widget_t *parent)
{
	swift_list_iter_t iter;
	swift_widget_t *child;

	if (!parent || !wgts)
		return;

	for (iter = swift_list_first(&parent->shown); iter.node;
		swift_list_next(&iter)) {
		child = SWIFT_CONTAINER_OF(iter.node, swift_widget_t, snode);

		child->control_draw(wgts, child);

		if (!swift_list_is_empty(&child->shown))
			swift_widgets_render(wgts, child);
	}
}

bool swift_widget_rect_contains(swift_widgets_t *wgts, swift_widget_t *control)
{
	if (!control || wgts->mouse_pos.xy.x < control->sprite.pos.xyz.x ||
		wgts->mouse_pos.xy.x > control->sprite.pos.xyz.x +
			control->sprite.size.wh.w || wgts->mouse_pos.xy.y <
				control->sprite.pos.xyz.y || wgts->mouse_pos.xy.y >
					control->sprite.pos.xyz.y + control->sprite.size.wh.h)
		return false;

	return true;
}

bool swift_widget_has_mouse_over(swift_widgets_t *wgts, swift_widget_t *control)
{
	swift_widget_t *parent = control->parent;

	if (!swift_widget_rect_contains(wgts, control))
		return false;

	if (control->action.always_useable)
		return true;

	if (!control->action.is_focused) {
		while (parent && parent->type != SWIFT_CONTROL_ROOT) {
			if ((parent->action.can_focus && parent->action.is_focused) ||
				parent->action.always_useable) {
				return true;
			}
			else
				parent = parent->parent;
		}

		return false;
	}

	return true;
}

bool swift_widget_usable(swift_widget_t *control)
{
	swift_widget_t *parent;

	if (control->action.always_useable)
		return true;

	if (!control->action.is_focused) {

		if (control->type == SWIFT_CONTROL_ROOT)
			return false;

		for (parent = control->parent; parent && parent->type !=
			SWIFT_CONTROL_ROOT; parent = parent->parent) {
			if ((parent->action.can_focus && parent->action.is_focused) ||
				parent->action.always_useable) {
				return true;
			}
		}

		return false;
	}

	return true;
}

void swift_widget_manual_focused(swift_widgets_t *wgts, swift_widget_t *control)
{
	if (!control || control->type == SWIFT_CONTROL_ROOT)
		return;

	if (control->action.can_focus) {
		swift_list_remove(&control->parent->shown, &control->snode);
		swift_list_unshift(&control->parent->shown, &control->snode);

		if (wgts->focused)
			wgts->focused->action.is_focused = false;

		control->action.is_focused = true;
		wgts->focused = control;
	}
}

void swift_widget_mouse_release(swift_widgets_t *wgts, swift_widget_t *control)
{
	swift_list_iter_t iter;
	swift_widget_t *child;

	if (!swift_list_is_empty(&control->shown)) {
		for (iter = swift_list_last(&control->shown); iter.node;
			swift_list_prev(&iter)) {
			child = SWIFT_CONTAINER_OF(iter.node, swift_widget_t, snode);

			if (swift_widget_rect_contains(wgts, child)) {
				if (!swift_list_is_empty(&child->shown))
					swift_widget_mouse_release(wgts, child);
				else
					swift_widget_set_release(wgts, child);

				return;
			}
		}
	}

	swift_widget_set_release(wgts, control);
}

void swift_widget_mouse_press(swift_widgets_t *wgts, swift_widget_t *control)
{
	swift_list_iter_t iter;
	swift_widget_t *child;

	if (!swift_list_is_empty(&control->shown)) {
		for (iter = swift_list_last(&control->shown); iter.node;
			swift_list_prev(&iter)) {
			child = SWIFT_CONTAINER_OF(iter.node, swift_widget_t, snode);

			if (swift_widget_rect_contains(wgts, child)) {
				if (!swift_list_is_empty(&child->shown))
					swift_widget_mouse_press(wgts, child);
				else {
					if (child->action.moveable)
						control->action.moving = false;

					swift_set_widget_mouse_press_event(wgts, child);
				}

				return;
			}
		}
	}

	if (control->action.moveable) {
		if (swift_widget_rect_contains(wgts, control))
			control->action.moving = false;
	}

	swift_set_widget_mouse_press_event(wgts, control);
}

void swift_set_gui_user_data(swift_widgets_t *wgts, void *data)
{
	if (!data)
		return;

	wgts->user_data = data;
}

void *swift_get_gui_user_data(swift_widgets_t *wgts)
{
	return wgts->user_data;
}
