#include <swift/atomic.h>
#include <swift/gui.h>
#include <swift/io.h>
#include <swift/window.h>
#include <swift/data.h>

static swift_ref_t ref = SWIFT_REF(0);
static swift_list_t wgts_list;

static void swift_do_term_widget_manager(swift_ref_t *ref)
{
	swift_wgt_man_t *widget_manager;
	swift_list_t *node;

	while ((node = swift_list_pop(&wgts_list))) {
		widget_manager = SWIFT_CONTAINER_OF(node, swift_wgt_man_t, node);

		if (widget_manager->font)
			swift_unload_font(widget_manager->font);

		swift_destroy_window(widget_manager->window);
		swift_unload_widgets(&widget_manager->wgts);
	}

	swift_term_list(&wgts_list);
	swift_term_fonts();
	swift_term_windows();
}

swift_list_t *swift_get_widget_managers(void)
{
	return &wgts_list;
}

bool swift_init_widget_manager(swift_wgt_man_t *widget_manager,
	swift_vec2f_t hw, const char *title, bool decor, void *user_data)
{
	if (swift_acquire_ref_unless_zero(&ref))
		return true;

	if (!swift_init_list(&wgts_list)) {
		eprintf("wgts list was unable to init");
		return false;
	}

	if (!swift_init_list(&widget_manager->node)) {
		eprintf("wgt node was unable to init");
		goto ERR_LIST;
	}

	if (!swift_init_windows(true, "./")) {
		eprintf("Window sys was unable to init");
		goto ERR_NODE;
	}

	widget_manager->window = swift_create_window(NULL, hw,
		SWIFT_VEC2F(200, 200), title, decor, NULL);

	if (!widget_manager->window) {
		eprintf("Window was unable to init");
		goto ERR_WINDOW;
	}

	if (!swift_init_rendering()) {
		eprintf("rendering sys was unable to init");
		goto ERR_RENDER;
	}

	if (!swift_init_fonts("./fonts/")) {
		eprintf("font was unable to init");
		goto ERR_RENDER;
	}

	widget_manager->font = calloc(1, sizeof(swift_font_t));

	if (!widget_manager->font) {
		eprintf("font was unable to create memory");
		goto ERR_FONT_INIT;
	}

	if (!swift_load_font(widget_manager->font)) {
		eprintf("font was unable to load");
		goto ERR_FONT_LOAD;
	}

	swift_set_current_window(widget_manager->window);

	if (!swift_init_widgets(widget_manager->window, &widget_manager->wgts,
		user_data)) {
		eprintf("widget sys was unable to init");
		goto ERR_FONT_LOAD;
	}

	swift_list_unshift(&wgts_list, &widget_manager->node);

	swift_acquire_ref(&ref);

	return true;

ERR_FONT_LOAD:
	SWIFT_FREE(widget_manager->font);
ERR_FONT_INIT:
	swift_term_fonts();
ERR_RENDER:
	swift_destroy_window(widget_manager->window);
ERR_WINDOW:
	swift_term_windows();
ERR_NODE:
	swift_term_list(&widget_manager->node);
ERR_LIST:
	swift_term_list(&wgts_list);

	return false;
}

bool swift_widget_manager_create_window(swift_wgt_man_t *widget_manager,
	swift_wgt_man_t *new_widget_manager, swift_vec2f_t hw, swift_vec2f_t pos,
		const char *title, bool decor, void *user_data)
{
	new_widget_manager->window = swift_create_window(widget_manager->window, hw,
		pos, title, decor, NULL);
	new_widget_manager->font = widget_manager->font;

	swift_set_current_window(new_widget_manager->window);

	if (!swift_init_widgets(new_widget_manager->window,
		&new_widget_manager->wgts, user_data)) {
		eprintf("New widget sys was unable to init");
		swift_destroy_window(new_widget_manager->window);
		return false;
	}

	swift_list_unshift(&wgts_list, &new_widget_manager->node);

	return true;
}

void swift_term_widget_manager(void)
{
	swift_release_ref(&ref, swift_do_term_widget_manager);
}

void swift_widget_manager_term_window(swift_wgt_man_t *widget_manager)
{
	swift_unload_widgets(&widget_manager->wgts);
	swift_destroy_window(widget_manager->window);
	swift_list_remove(&wgts_list, &widget_manager->node);
}
