#include <swift/atomic.h>
#include <swift/image.h>
#include <swift/maths.h>

static swift_ref_t ref = SWIFT_REF(0);

static void swift_do_term_images(swift_ref_t *ref)
{
	swift_term_image_codecs();
	swift_term_pixfmts();
	swift_term_io();
}

bool swift_init_images(const char *path)
{
	if (swift_acquire_ref_unless_zero(&ref))
		return true;

	if (!swift_init_io())
		return false;

	if (!swift_init_pixfmts())
		goto err_term_io;

	if (!swift_init_image_codecs(path))
		goto err_term_pixfmts;

	swift_acquire_ref(&ref);

	return true;

err_term_pixfmts:
	swift_term_image_codecs();
err_term_io:
	swift_term_io();
	return false;
}

void swift_term_images(void)
{
	swift_release_ref(&ref, swift_do_term_images);
}

swift_image_t *swift_new_image(swift_pixfmt_t *fmt, unsigned int width,
	unsigned int height)
{
	swift_image_t *image;

	if (!fmt || !width || !height)
		return NULL;

	image = calloc(1, sizeof(swift_image_t));

	if (!image)
		return NULL;

	image->fmt = fmt;
	image->pitch = width * fmt->bytes_per_pixel;
	image->width = width;
	image->height = height;
	image->data = calloc(image->pitch * image->height, sizeof(char));

	if (!image->data)
		goto err_free_image;

	memset(image->data, 0xFF, image->pitch * image->height);

	return image;

err_free_image:
	SWIFT_FREE(image);
	return NULL;
}

void swift_del_image(swift_image_t *image)
{
	if (!image)
		return;

	SWIFT_FREE(image->data);
	SWIFT_FREE(image);
}

void swift_get_image_pixel(swift_image_t *image, size_t x, size_t y,
	swift_vec4u_t *colour)
{
	image->fmt->ops->read_pixel(image->fmt, image->data +
		y * image->pitch + x * image->fmt->bytes_per_pixel, colour);
}

void swift_set_image_pixel(swift_image_t *image, size_t x, size_t y,
	swift_vec4u_t colour)
{
	image->fmt->ops->write_pixel(image->fmt, image->data +
		y * image->pitch + x * image->fmt->bytes_per_pixel, colour);
}
