#include <swift/io.h>

static void swift_close_file_stream(swift_stream_t *stream);
static int64_t swift_seek_file_stream(swift_stream_t *stream,
	unsigned int whence, int64_t pos);
static size_t swift_read_file_stream(swift_stream_t *stream, void *buffer,
	size_t size);
static size_t swift_write_file_stream(swift_stream_t *stream,
	const void *buffer, size_t size);
static bool swift_flush_file_stream(swift_stream_t *stream);

static swift_stream_ops_t file_stream_ops = {
	.close = swift_close_file_stream,
	.seek = swift_seek_file_stream,
	.read = swift_read_file_stream,
	.write = swift_write_file_stream,
	.flush = swift_flush_file_stream,
};

swift_stream_t *swift_open_file_stream(const char *path, unsigned int flags)
{
	swift_file_stream_t *file_stream;
	swift_stream_t *stream;

	file_stream = malloc(sizeof(swift_file_stream_t));

	if (!file_stream)
		return NULL;

	file_stream->file = swift_open_file(path, flags);

	if (!file_stream->file)
		goto free_file_stream;

	stream = &file_stream->stream;
	swift_init_ref(&stream->ref);
	stream->stream_ops = &file_stream_ops;

	return stream;

free_file_stream:
	SWIFT_FREE(file_stream);
	return NULL;
}

static void swift_close_file_stream(swift_stream_t *stream)
{
	swift_file_stream_t *file_stream = SWIFT_FILE_STREAM(stream);

	swift_close_file(file_stream->file);
	SWIFT_FREE(file_stream);
}

static int64_t swift_seek_file_stream(swift_stream_t *stream,
	unsigned int whence, int64_t pos)
{
	swift_file_stream_t *file_stream = SWIFT_FILE_STREAM(stream);

	return swift_seek_file(file_stream->file, whence, pos);
}

static size_t swift_read_file_stream(swift_stream_t *stream,
	void *buffer, size_t size)
{
	swift_file_stream_t *file_stream = SWIFT_FILE_STREAM(stream);

	return swift_read_file(file_stream->file, buffer, size);

}

static size_t swift_write_file_stream(swift_stream_t *stream,
	const void *buffer, size_t size)
{
	swift_file_stream_t *file_stream = SWIFT_FILE_STREAM(stream);

	return swift_write_file(file_stream->file, buffer, size);
}

static bool swift_flush_file_stream(swift_stream_t *stream)
{
	swift_file_stream_t *file_stream = SWIFT_FILE_STREAM(stream);

	return swift_flush_file(file_stream->file);
}
