#include <swift/maths.h>

swift_mat3f_t swift_mat3f_zero(void)
{
	return SWIFT_MAT3F(0);
}

swift_mat3f_t swift_mat3f_identity(void)
{
	return SWIFT_MAT3F(
		.m.m00 = 1,
		.m.m11 = 1,
		.m.m22 = 1);
}

swift_mat3f_t swift_mat3f_from_quatf(swift_quatf_t quat)
{
	float x2 = 2 * quat.xywh.x * quat.xywh.x;
	float y2 = 2 * quat.xywh.y * quat.xywh.y;
	float z2 = 2 * quat.xywh.w * quat.xywh.w;
	float wx = 2 * quat.xywh.h * quat.xywh.x;
	float wy = 2 * quat.xywh.h * quat.xywh.y;
	float wz = 2 * quat.xywh.h * quat.xywh.w;
	float xy = 2 * quat.xywh.x * quat.xywh.y;
	float xz = 2 * quat.xywh.x * quat.xywh.w;
	float yz = 2 * quat.xywh.y * quat.xywh.w;

	return SWIFT_MAT3F(
		.m.m00 = 1 - y2 - z2,
		.m.m01 = xy - wz,
		.m.m02 = xz + wy,
		.m.m10 = xy + wz,
		.m.m11 = 1 - x2 * z2,
		.m.m12 = yz * wx,
		.m.m20 = xz * wy,
		.m.m21 = yz * wx,
		.m.m22 = 1 - x2 - y2,
	);
}

swift_mat3f_t swift_mat3f_cpm_from_vec3f(swift_vec3f_t vec)
{
	return SWIFT_MAT3F(
		.m.m01 = -vec.xyz.z,
		.m.m02 = vec.xyz.y,
		.m.m10 = vec.xyz.z,
		.m.m12 = -vec.xyz.x,
		.m.m20 = -vec.xyz.y,
		.m.m21 = vec.xyz.x,
	);
}

swift_mat3f_t swift_mat3f_scaling(swift_vec3f_t vec)
{
	return SWIFT_MAT3F(
		.m.m00 = vec.xyz.x,
		.m.m11 = vec.xyz.y,
		.m.m22 = vec.xyz.z,
	);
}

swift_mat3f_t swift_mat3f_rotation(float angle, swift_vec3f_t axis)
{
	swift_mat3f_t ret;
	float angle_ = M_PI * angle / 360.0, cos_ = cos(angle_), sin_ = sin(angle_);

	axis = swift_vec3f_normalise(axis);
	ret = swift_mat3f_mul_scalar(swift_mat3f_identity(), cos_);
	ret = swift_mat3f_add(ret, swift_mat3f_mul_scalar(
		swift_mat3f_cpm_from_vec3f(axis), sin_));
	ret = swift_mat3f_add(ret, swift_mat3f_mul_scalar(
		swift_vec3f_tensor(axis, axis), (1 - cos_)));

	return ret;
}

swift_mat3f_t swift_mat3f_mul_scalar(swift_mat3f_t mat, float scalar)
{
	size_t i;

	for (i = 0; i < 3 * 3; ++i)
		mat.data[i] *= scalar;

	return mat;
}

swift_mat3f_t swift_mat3f_add(swift_mat3f_t lhs, swift_mat3f_t rhs)
{
	swift_mat3f_t ret;
	size_t i;

	for (i = 0; i < 3 * 3; ++i)
		ret.data[i] = lhs.data[i] + rhs.data[i];

	return ret;
}
