#include <swift/maths.h>

swift_quatf_t swift_quatf_from_axis(swift_vec3f_t axis, float angle)
{
	float sin_;

	angle *= 0.5;
	sin_ = sin(angle);
	axis = swift_vec3f_normalise(axis);

	return SWIFT_QUATF(
		axis.xyz.x * sin_,
		axis.xyz.y * sin_,
		axis.xyz.z * sin_,
		cos(angle));
}

swift_quatf_t swift_quatf_add_scalar(swift_quatf_t lhs, float rhs)
{
	return SWIFT_QUATF(
		lhs.xywh.x + rhs,
		lhs.xywh.y + rhs,
		lhs.xywh.w + rhs,
		lhs.xywh.h + rhs);
}

swift_quatf_t swift_quatf_sub_scalar(swift_quatf_t lhs, float rhs)
{
	return SWIFT_QUATF(
		lhs.xywh.x - rhs,
		lhs.xywh.y - rhs,
		lhs.xywh.w - rhs,
		lhs.xywh.h - rhs);
}

swift_quatf_t swift_quatf_mul_scalar(swift_quatf_t lhs, float rhs)
{
	return SWIFT_QUATF(
		lhs.xywh.x * rhs,
		lhs.xywh.y * rhs,
		lhs.xywh.w * rhs,
		lhs.xywh.h * rhs);
}

swift_quatf_t swift_quatf_div_scalar(swift_quatf_t lhs, float rhs)
{
	return SWIFT_QUATF(
		lhs.xywh.x / rhs,
		lhs.xywh.y / rhs,
		lhs.xywh.w / rhs,
		lhs.xywh.h / rhs);
}

swift_quatf_t swift_quatf_conj(swift_quatf_t quat)
{
	return SWIFT_QUATF(
		-quat.xywh.x,
		-quat.xywh.y,
		-quat.xywh.w,
		quat.xywh.h);
}

float swift_quatf_norm(swift_quatf_t quat)
{
	return sqrt(swift_vec4f_dot(quat.vec, quat.vec));
}

swift_quatf_t swift_quatf_normalise(swift_quatf_t quat)
{
	return swift_quatf_div_scalar(quat, swift_quatf_norm(quat));
}
