#include <swift/maths.h>

swift_vec2f_t swift_vec2f_neg(swift_vec2f_t vec)
{
	return SWIFT_VEC2F(
		-vec.xy.x,
		-vec.xy.y,
	);
}

swift_vec2f_t swift_vec2f_add_scalar(swift_vec2f_t lhs, float rhs)
{
	return SWIFT_VEC2F(
		lhs.xy.x + rhs,
		lhs.xy.y + rhs,
	);
}

swift_vec2f_t swift_vec2f_sub_scalar(swift_vec2f_t lhs, float rhs)
{
	return SWIFT_VEC2F(
		lhs.xy.x - rhs,
		lhs.xy.y - rhs,
	);
}

swift_vec2f_t swift_vec2f_mul_scalar(swift_vec2f_t lhs, float rhs)
{
	return SWIFT_VEC2F(
		lhs.xy.x * rhs,
		lhs.xy.y * rhs,
	);
}

swift_vec2f_t swift_vec2f_div_scalar(swift_vec2f_t lhs, float rhs)
{
	return SWIFT_VEC2F(
		lhs.xy.x / rhs,
		lhs.xy.y / rhs,
	);
}

swift_vec2f_t swift_vec2f_add(swift_vec2f_t lhs, swift_vec2f_t rhs)
{
	return SWIFT_VEC2F(
		lhs.xy.x + rhs.xy.x,
		lhs.xy.y + rhs.xy.y,
	);
}

swift_vec2f_t swift_vec2f_sub(swift_vec2f_t lhs, swift_vec2f_t rhs)
{
	return SWIFT_VEC2F(
		lhs.xy.x - rhs.xy.x,
		lhs.xy.y - rhs.xy.y,
	);
}

float swift_vec2f_dot(swift_vec2f_t lhs, swift_vec2f_t rhs)
{
	float ret = 0;
	size_t i;

	for (i = 0; i < 2; ++i)
		ret += lhs.data[i] * rhs.data[i];

	return ret;
}

swift_mat2f_t swift_vec2f_tensor(swift_vec2f_t lhs, swift_vec2f_t rhs)
{
	swift_mat2f_t ret;
	size_t i, j;

	for (j = 0; j < 2; ++j)
		for (i = 0; i < 2; ++i)
			ret.data[j * 2 + i] = lhs.data[i] * rhs.data[j];

	return ret;
}

float swift_vec2f_norm(swift_vec2f_t vec)
{
	return sqrt(swift_vec2f_dot(vec, vec));
}

swift_vec2f_t swift_vec2f_normalise(swift_vec2f_t vec)
{
	return swift_vec2f_div_scalar(vec, swift_vec2f_norm(vec));
}

float swift_vec2f_length_sq(swift_vec2f_t vec)
{
	return (vec.xy.x * vec.xy.x) + (vec.xy.y * vec.xy.y);
}

float swift_vec2f_length(swift_vec2f_t vec)
{
	return sqrtf(vec.xy.x * vec.xy.x + vec.xy.y * vec.xy.y);
}
