#include <swift/maths.h>

swift_vec3f_t swift_vec3f_neg(swift_vec3f_t vec)
{
	return SWIFT_VEC3F(
		-vec.xyz.x,
		-vec.xyz.y,
		-vec.xyz.z,
	);
}

swift_vec3f_t swift_vec3f_add_scalar(swift_vec3f_t lhs, float rhs)
{
	return SWIFT_VEC3F(
		lhs.xyz.x + rhs,
		lhs.xyz.y + rhs,
		lhs.xyz.z + rhs,
	);
}

swift_vec3f_t swift_vec3f_sub_scalar(swift_vec3f_t lhs, float rhs)
{
	return SWIFT_VEC3F(
		lhs.xyz.x - rhs,
		lhs.xyz.y - rhs,
		lhs.xyz.z - rhs,
	);
}

swift_vec3f_t swift_vec3f_mul_scalar(swift_vec3f_t lhs, float rhs)
{
	return SWIFT_VEC3F(
		lhs.xyz.x * rhs,
		lhs.xyz.y * rhs,
		lhs.xyz.z * rhs,
	);
}

swift_vec3f_t swift_vec3f_div_scalar(swift_vec3f_t lhs, float rhs)
{
	return SWIFT_VEC3F(
		lhs.xyz.x / rhs,
		lhs.xyz.y / rhs,
		lhs.xyz.z / rhs,
	);
}

swift_vec3f_t swift_vec3f_add(swift_vec3f_t lhs, swift_vec3f_t rhs)
{
	return SWIFT_VEC3F(
		lhs.xyz.x + rhs.xyz.x,
		lhs.xyz.y + rhs.xyz.y,
		lhs.xyz.z + rhs.xyz.z,
	);
}

swift_vec3f_t swift_vec3f_sub(swift_vec3f_t lhs, swift_vec3f_t rhs)
{
	return SWIFT_VEC3F(
		lhs.xyz.x - rhs.xyz.x,
		lhs.xyz.y - rhs.xyz.y,
		lhs.xyz.z - rhs.xyz.z,
	);
}

float swift_vec3f_dot(swift_vec3f_t lhs, swift_vec3f_t rhs)
{
	float ret = 0;
	size_t i;

	for (i = 0; i < 3; ++i)
		ret += lhs.data[i] * rhs.data[i];

	return ret;
}

swift_vec3f_t swift_vec3f_cross(swift_vec3f_t lhs, swift_vec3f_t rhs)
{
	return SWIFT_VEC3F(
		lhs.xyz.y * rhs.xyz.z - lhs.xyz.z * rhs.xyz.y,
		lhs.xyz.z * rhs.xyz.x - lhs.xyz.x * rhs.xyz.z,
		lhs.xyz.x * rhs.xyz.y - lhs.xyz.y * rhs.xyz.x,
	);
}

swift_mat3f_t swift_vec3f_tensor(swift_vec3f_t lhs, swift_vec3f_t rhs)
{
	swift_mat3f_t ret;
	size_t i, j;

	for (j = 0; j < 3; ++j)
		for (i = 0; i < 3; ++i)
			ret.data[j * 3 + i] = lhs.data[i] * rhs.data[j];

	return ret;
}

float swift_vec3f_norm(swift_vec3f_t vec)
{
	return sqrt(swift_vec3f_dot(vec, vec));
}

swift_vec3f_t swift_vec3f_normalise(swift_vec3f_t vec)
{
	return swift_vec3f_div_scalar(vec, swift_vec3f_norm(vec));
}
