#include <swift/maths.h>

swift_vec4f_t swift_vec4f_zero(void)
{
	return SWIFT_VEC4F(0);
}

swift_vec4f_t swift_vec4f_neg(swift_vec4f_t vec)
{
	return SWIFT_VEC4F(
		-vec.xywh.x,
		-vec.xywh.y,
		-vec.xywh.w,
		-vec.xywh.h,
	);
}

swift_vec4f_t swift_vec4f_add_scalar(swift_vec4f_t lhs, float rhs)
{
	return SWIFT_VEC4F(
		lhs.xywh.x + rhs,
		lhs.xywh.y + rhs,
		lhs.xywh.w + rhs,
		lhs.xywh.h + rhs,
	);
}

swift_vec4f_t swift_vec4f_sub_scalar(swift_vec4f_t lhs, float rhs)
{
	return SWIFT_VEC4F(
		lhs.xywh.x - rhs,
		lhs.xywh.y - rhs,
		lhs.xywh.w - rhs,
		lhs.xywh.h - rhs,
	);
}

swift_vec4f_t swift_vec4f_mul_scalar(swift_vec4f_t lhs, float rhs)
{
	return SWIFT_VEC4F(
		lhs.xywh.x * rhs,
		lhs.xywh.y * rhs,
		lhs.xywh.w * rhs,
		lhs.xywh.h * rhs,
	);
}

swift_vec4f_t swift_vec4f_div_scalar(swift_vec4f_t lhs, float rhs)
{
	return SWIFT_VEC4F(
		lhs.xywh.x / rhs,
		lhs.xywh.y / rhs,
		lhs.xywh.w / rhs,
		lhs.xywh.h / rhs,
	);
}

swift_vec4f_t swift_vec4f_add(swift_vec4f_t lhs, swift_vec4f_t rhs)
{
	return SWIFT_VEC4F(
		lhs.xywh.x + rhs.xywh.x,
		lhs.xywh.y + rhs.xywh.y,
		lhs.xywh.w + rhs.xywh.w,
		lhs.xywh.h + rhs.xywh.h,
	);
}

swift_vec4f_t swift_vec4f_sub(swift_vec4f_t lhs, swift_vec4f_t rhs)
{
	return SWIFT_VEC4F(
		lhs.xywh.x - rhs.xywh.x,
		lhs.xywh.y - rhs.xywh.y,
		lhs.xywh.w - rhs.xywh.w,
		lhs.xywh.h - rhs.xywh.h,
	);
}

float swift_vec4f_dot(swift_vec4f_t lhs, swift_vec4f_t rhs)
{
	float ret = 0;
	size_t i;

	for (i = 0; i < 4; ++i)
		ret += lhs.data[i] * rhs.data[i];

	return ret;
}

swift_mat4f_t swift_vec4f_tensor(swift_vec4f_t lhs, swift_vec4f_t rhs)
{
	swift_mat4f_t ret;
	size_t i, j;

	for (j = 0; j < 4; ++j)
		for (i = 0; i < 4; ++i)
			ret.data[j * 4 + i] = lhs.data[i] * rhs.data[j];

	return ret;
}

float swift_vec4f_norm(swift_vec4f_t vec)
{
	return sqrt(swift_vec4f_dot(vec, vec));
}

swift_vec4f_t swift_vec4f_normalise(swift_vec4f_t vec)
{
	return swift_vec4f_div_scalar(vec, swift_vec4f_norm(vec));
}

swift_vec4f_t swift_rgba(uint8_t r, uint8_t g, uint8_t b, uint8_t a)
{
	return SWIFT_VEC4F(r / 255, g / 255, b / 255, a / 255);
}
