#include <swift/net.h>

#include "addr.h"

static void swift_do_term_addr(swift_ref_t *ref)
{
	swift_addr_t *addr = SWIFT_CONTAINER_OF(ref, swift_addr_t,
		ref);

	sys_term_addr(addr);
}

static void swift_do_del_addr(swift_ref_t *ref)
{
	swift_addr_t *addr = SWIFT_CONTAINER_OF(ref, swift_addr_t,
		ref);

	sys_term_addr(addr);
	SWIFT_FREE(addr);
}

bool swift_init_addr(swift_addr_t *addr, unsigned int addr_type)
{
	if (!addr)
		return false;

	if (!sys_init_addr(addr, addr_type))
		return false;

	swift_init_ref(&addr->ref);

	return true;
}

bool swift_init_addr_from(swift_addr_t *addr, const void *data)
{
	if (!addr)
		return false;

	if (!sys_init_addr_from(addr, data))
		return false;

	swift_init_ref(&addr->ref);

	return true;
}

void swift_term_addr(swift_addr_t *addr)
{
	if (!addr)
		return;

	swift_release_ref(&addr->ref, swift_do_term_addr);
}

swift_addr_t *swift_new_addr(unsigned int addr_type)
{
	swift_addr_t *addr;

	addr = malloc(sizeof(*addr));

	if (!addr)
		return NULL;

	if (!swift_init_addr(addr, addr_type))
		goto err_free_addr;

	return addr;

err_free_addr:
	SWIFT_FREE(addr);
	return NULL;
}

swift_addr_t *swift_new_addr_from(const void *data)
{
	swift_addr_t *addr;

	addr = malloc(sizeof(*addr));

	if (!addr)
		return NULL;

	if (!swift_init_addr_from(addr, data))
		goto err_free_addr;

	return addr;

err_free_addr:
	SWIFT_FREE(addr);
	return NULL;
}

void swift_del_addr(swift_addr_t *addr)
{
	if (!addr)
		return;

	swift_release_ref(&addr->ref, swift_do_del_addr);
}

swift_addr_t *swift_ref_addr(swift_addr_t *addr)
{
	if (!addr)
		return NULL;

	swift_acquire_ref(&addr->ref);

	return addr;
}

void swift_del_addrs(swift_array_t *addrs)
{
	swift_array_iter_t arr_iter;
	swift_addr_t *addr;

	if (!addrs)
		return;

	for (arr_iter = swift_array_first(addrs); arr_iter.data;
		swift_array_next(&arr_iter)) {
		addr = *(swift_addr_t **)arr_iter.data;

		swift_del_addr(addr);
	}

	swift_del_array(addrs);
}

swift_array_t *swift_resolve_addr(unsigned int addr_type,
	const char *name, uint16_t port, unsigned int flags)
{
	swift_array_t *addrs;

	addrs = swift_new_array(NULL, sizeof(swift_addr_t *));

	if (!addrs)
		return NULL;

	if (!sys_resolve_addr(addrs, addr_type, name, port, flags))
		goto err_del_addrs;

	return addrs;

err_del_addrs:
	swift_del_addrs(addrs);
	return NULL;
}

swift_str_t *swift_resolve_name(swift_addr_t *addr,
	unsigned int flags)
{
	if (!addr)
		return NULL;

	return sys_resolve_name(addr, flags);
}
