#include <swift/net.h>
#include <swift/data.h>
#include <swift/event.h>

#include "monitor.h"
#include "sock.h"

static void swift_do_term_monitor(swift_ref_t *ref)
{
	swift_monitor_t *monitor = SWIFT_CONTAINER_OF(ref,
		swift_monitor_t, ref);

	sys_term_monitor(monitor);
}

static void swift_do_del_monitor(swift_ref_t *ref)
{
	swift_monitor_t *monitor = SWIFT_CONTAINER_OF(ref,
		swift_monitor_t, ref);

	sys_term_monitor(monitor);
	SWIFT_FREE(monitor);
}

bool swift_init_monitor(swift_monitor_t *monitor,
	unsigned int thread_count)
{
	if (!monitor)
		return false;

	if (!sys_init_monitor(monitor, thread_count))
		return false;

	if (!swift_add_ev_src(swift_poll_monitor, monitor,
		SWIFT_MAIN_THREAD | SWIFT_WORKER_THREAD))
		goto err_term_monitor;

	swift_init_ref(&monitor->ref);

	return true;

err_term_monitor:
	sys_term_monitor(monitor);
	return false;
}

void swift_term_monitor(swift_monitor_t *monitor)
{
	if (!monitor)
		return;

	swift_release_ref(&monitor->ref, swift_do_term_monitor);
}

swift_monitor_t *swift_new_monitor(unsigned int thread_count)
{
	swift_monitor_t *monitor;

	monitor = malloc(sizeof(*monitor));

	if (!monitor)
		return NULL;

	if (!swift_init_monitor(monitor, thread_count))
		goto err_free_monitor;

	return monitor;

err_free_monitor:
	SWIFT_FREE(monitor);
	return NULL;
}

void swift_del_monitor(swift_monitor_t *monitor)
{
	if (!monitor)
		return;

	swift_release_ref(&monitor->ref, swift_do_del_monitor);
}

swift_monitor_t *swift_ref_monitor(swift_monitor_t *monitor)
{
	if (!monitor)
		return NULL;

	swift_acquire_ref(&monitor->ref);

	return monitor;
}

bool swift_monitor_sock(swift_monitor_t *monitor,
	swift_sock_t *sock)
{
	if (!monitor || !sock)
		return false;

	if (sock->monitor)
		return (sock->monitor == monitor);

	if (!sys_monitor_sock(monitor, sock))
		return false;

	sock->monitor = monitor;

	return true;
}

void swift_poll_monitor(void *user_data)
{
	swift_monitor_t *monitor = user_data;

	if (!monitor)
		return;

	sys_poll_monitor(monitor);
}
