#include <swift/data.h>
#include <swift/net.h>

#define WIN32_MEAN_AND_LEAN
#include <winsock2.h>
#include <windows.h>
#include <ws2tcpip.h>
#include <mswsock.h>

#include "msw_addr.h"
#include "msw_ipv4_addr.h"
typedef struct sys_ipv4_addr_t sys_ipv4_addr_t;

struct sys_ipv4_addr_t {
	sys_addr_t addr;
	struct sockaddr_in sockaddr;
};

static void sys_term_ipv4_addr(swift_addr_t *addr);
static struct sockaddr *sys_ipv4_addr_sockaddr(swift_addr_t *addr);
static size_t sys_ipv4_addr_sockaddr_size(swift_addr_t *addr);

static sys_addr_ops_t ipv4_addr_ops = {
	.term_addr = sys_term_ipv4_addr,
	.addr_sockaddr = sys_ipv4_addr_sockaddr,
	.addr_sockaddr_size = sys_ipv4_addr_sockaddr_size,
};

bool sys_init_ipv4_addr(swift_addr_t *addr)
{
	sys_ipv4_addr_t *ipv4_addr;
	sys_addr_t *sys_addr;

	ipv4_addr = calloc(1, sizeof(*ipv4_addr));

	if (!ipv4_addr)
		return false;

	sys_addr = &ipv4_addr->addr;
	sys_addr->ops = &ipv4_addr_ops;
	addr->data = sys_addr;
	addr->addr_type = SWIFT_ADDR_IPV4;

	return true;
}

bool sys_init_ipv4_addr_from(swift_addr_t *addr, const void *data)
{
	struct sockaddr_in *sockaddr;
	sys_ipv4_addr_t *ipv4_addr;
	sys_addr_t *sys_addr;

	ipv4_addr = calloc(1, sizeof(*ipv4_addr));

	if (!ipv4_addr)
		return false;

	sockaddr = (struct sockaddr_in *)data;
	memcpy(&ipv4_addr->sockaddr, sockaddr, sizeof(ipv4_addr->sockaddr));
	sys_addr = &ipv4_addr->addr;
	sys_addr->ops = &ipv4_addr_ops;
	addr->data = sys_addr;
	addr->addr_type = SWIFT_ADDR_IPV4;

	return true;
}

static void sys_term_ipv4_addr(swift_addr_t *addr)
{
	sys_addr_t *sys_addr = addr->data;
	sys_ipv4_addr_t *ipv4_addr = SWIFT_CONTAINER_OF(sys_addr,
		sys_ipv4_addr_t, addr);

	SWIFT_FREE(ipv4_addr);
}

static struct sockaddr *sys_ipv4_addr_sockaddr(swift_addr_t *addr)
{
	sys_addr_t *sys_addr = addr->data;
	sys_ipv4_addr_t *ipv4_addr = SWIFT_CONTAINER_OF(sys_addr,
		sys_ipv4_addr_t, addr);

	return (struct sockaddr *)&ipv4_addr->sockaddr;
}

static size_t sys_ipv4_addr_sockaddr_size(swift_addr_t *addr)
{
	sys_addr_t *sys_addr = addr->data;
	sys_ipv4_addr_t *ipv4_addr = SWIFT_CONTAINER_OF(sys_addr,
		sys_ipv4_addr_t, addr);

	return sizeof(ipv4_addr->sockaddr);
}
