#include <swift/data.h>
#include <swift/net.h>

#define WIN32_MEAN_AND_LEAN
#include <winsock2.h>
#include <windows.h>
#include <ws2tcpip.h>
#include <mswsock.h>

#include "msw_addr.h"
#include "msw_ipv6_addr.h"

typedef struct sys_ipv6_addr_t sys_ipv6_addr_t;

struct sys_ipv6_addr_t {
	sys_addr_t addr;
	struct sockaddr_in6 sockaddr;
};

static void sys_term_ipv6_addr(swift_addr_t *addr);
static struct sockaddr *sys_ipv6_addr_sockaddr(swift_addr_t *addr);
static size_t sys_ipv6_addr_sockaddr_size(swift_addr_t *addr);

static sys_addr_ops_t ipv6_addr_ops = {
	.term_addr = sys_term_ipv6_addr,
	.addr_sockaddr = sys_ipv6_addr_sockaddr,
	.addr_sockaddr_size = sys_ipv6_addr_sockaddr_size,
};

bool sys_init_ipv6_addr(swift_addr_t *addr)
{
	sys_ipv6_addr_t *ipv6_addr;
	sys_addr_t *sys_addr;

	ipv6_addr = calloc(1, sizeof(*ipv6_addr));

	if (!ipv6_addr)
		return false;

	sys_addr = &ipv6_addr->addr;
	sys_addr->ops = &ipv6_addr_ops;
	addr->data = sys_addr;
	addr->addr_type = SWIFT_ADDR_IPV6;

	return true;
}

bool sys_init_ipv6_addr_from(swift_addr_t *addr, const void *data)
{
	struct sockaddr_in6 *sockaddr;
	sys_ipv6_addr_t *ipv6_addr;
	sys_addr_t *sys_addr;

	ipv6_addr = calloc(1, sizeof(*ipv6_addr));

	if (!ipv6_addr)
		return false;

	sockaddr = (struct sockaddr_in6 *)data;
	memcpy(&ipv6_addr->sockaddr, sockaddr, sizeof(ipv6_addr->sockaddr));
	sys_addr = &ipv6_addr->addr;
	sys_addr->ops = &ipv6_addr_ops;
	addr->data = sys_addr;
	addr->addr_type = SWIFT_ADDR_IPV6;

	return true;
}

static void sys_term_ipv6_addr(swift_addr_t *addr)
{
	sys_addr_t *sys_addr = addr->data;
	sys_ipv6_addr_t *ipv6_addr = SWIFT_CONTAINER_OF(sys_addr,
		sys_ipv6_addr_t, addr);

	SWIFT_FREE(ipv6_addr);
}

static struct sockaddr *sys_ipv6_addr_sockaddr(swift_addr_t *addr)
{
	sys_addr_t *sys_addr = addr->data;
	sys_ipv6_addr_t *ipv6_addr = SWIFT_CONTAINER_OF(sys_addr,
		sys_ipv6_addr_t, addr);

	return (struct sockaddr *)&ipv6_addr->sockaddr;
}

static size_t sys_ipv6_addr_sockaddr_size(swift_addr_t *addr)
{
	sys_addr_t *sys_addr = addr->data;
	sys_ipv6_addr_t *ipv6_addr = SWIFT_CONTAINER_OF(sys_addr,
		sys_ipv6_addr_t, addr);

	return sizeof(ipv6_addr->sockaddr);
}
