#include <swift/data.h>
#include <swift/net.h>

#define WIN32_MEAN_AND_LEAN
#include <winsock2.h>
#include <windows.h>
#include <ws2tcpip.h>
#include <mswsock.h>

#include "msw/msw_monitor.h"
#include "msw/msw_sock.h"

bool sys_init_monitor(swift_monitor_t *monitor, unsigned long thread_count)
{
	sys_monitor_t *sys_monitor;

	sys_monitor = malloc(sizeof(*sys_monitor));

	if (!sys_monitor)
		return false;

	sys_monitor->handle = CreateIoCompletionPort(INVALID_HANDLE_VALUE, NULL,
		OPERATION_KEY, thread_count);

	if (!sys_monitor->handle)
		goto err_free_monitor;

	monitor->data = sys_monitor;

	return true;

err_free_monitor:
	SWIFT_FREE(sys_monitor);
	return false;
}

void sys_term_monitor(swift_monitor_t *monitor)
{
	sys_monitor_t *sys_monitor;

	sys_monitor = monitor->data;

	CloseHandle(sys_monitor->handle);
	SWIFT_FREE(sys_monitor);
}

bool sys_monitor_sock(swift_monitor_t *monitor, swift_sock_t *sock)
{
	sys_monitor_t *sys_monitor = monitor->data;
	sys_sock_t *sys_sock = sock->data;
	HANDLE handle = CreateIoCompletionPort((HANDLE)sys_sock->handle,
		sys_monitor->handle, OPERATION_KEY, 0);

	if (!handle)
		return false;

	return true;
}

void sys_deregister_sock(swift_monitor_t *monitor, swift_sock_t *sock)
{
}

void sys_poll_monitor(swift_monitor_t *monitor)
{
	sys_monitor_t *sys_monitor = monitor->data;
	sys_event_t *monitor_event;
	OVERLAPPED *overlapped;
	ULONG_PTR key;
	DWORD error, transferred, ret;

	while ((ret = GetQueuedCompletionStatus(sys_monitor->handle,
		&transferred, &key, &overlapped, 0)) || overlapped) {
		monitor_event = SWIFT_CONTAINER_OF(overlapped,
			sys_event_t, overlapped);
		error = (ret) ? ERROR_SUCCESS : GetLastError();

		monitor_event->on_complete(error, transferred, monitor_event);
	} while (ret);
}
