#include <swift/net.h>

static void swift_close_ring_buffer_stream( swift_stream_t *stream);
static size_t swift_read_ring_buffer_stream( swift_stream_t *stream,
	void *data, size_t size);
static size_t swift_write_ring_buffer_stream( swift_stream_t *stream,
	const void *data, size_t size);

static swift_stream_ops_t ring_buffer_stream_ops = {
	.close = swift_close_ring_buffer_stream,
	.seek = NULL,
	.read = swift_read_ring_buffer_stream,
	.write = swift_write_ring_buffer_stream,
	.flush = NULL,
};

 swift_stream_t *swift_create_ring_buffer_stream(size_t size)
{
	 swift_ring_buffer_stream_t *buffer_stream;
	 swift_stream_t *stream;

	buffer_stream = calloc(1, sizeof( swift_ring_buffer_stream_t));

	if (!buffer_stream)
		return NULL;

	buffer_stream->buffer = swift_create_ring_buffer(size);

	if (!buffer_stream->buffer)
		goto err_free_buffer_stream;

	stream = &buffer_stream->stream;
	swift_init_ref(&stream->ref);
	stream->stream_ops = &ring_buffer_stream_ops;

	return stream;

err_free_buffer_stream:
	SWIFT_FREE(buffer_stream);
	return NULL;
}

 swift_stream_t *swift_create_ring_buffer_stream_from(
	swift_ring_buffer_t *buffer)
{
	 swift_ring_buffer_stream_t *buffer_stream;
	 swift_stream_t *stream;

	buffer_stream = calloc(1, sizeof( swift_ring_buffer_stream_t));

	if (!buffer_stream)
		return NULL;

	buffer_stream->buffer = buffer;
	stream = &buffer_stream->stream;
	stream->stream_ops = &ring_buffer_stream_ops;

	return stream;
}

static void swift_close_ring_buffer_stream( swift_stream_t *stream)
{
	 swift_ring_buffer_stream_t *buffer_stream =
		SWIFT_RING_BUFFER_STREAM(stream);

	swift_destroy_ring_buffer(buffer_stream->buffer);
	SWIFT_FREE(buffer_stream);
}

static size_t swift_read_ring_buffer_stream( swift_stream_t *stream,
	void *data, size_t size) {
	 swift_ring_buffer_stream_t *buffer_stream =
		SWIFT_RING_BUFFER_STREAM(stream);

	return swift_read_ring_buffer(buffer_stream->buffer, data, size);
}

static size_t swift_write_ring_buffer_stream( swift_stream_t *stream,
	const void *data, size_t size) {
	 swift_ring_buffer_stream_t *buffer_stream =
		SWIFT_RING_BUFFER_STREAM(stream);

	return swift_write_ring_buffer(buffer_stream->buffer, data, size);
}
