#include <swift/atomic.h>
#include <swift/data.h>
#include <swift/thread.h>

#include "barrier.h"

static void swift_do_del_barrier(swift_ref_t *ref)
{
	swift_barrier_t *barrier;

	if (!ref)
		return;

	barrier = SWIFT_CONTAINER_OF(ref, swift_barrier_t, ref);

	native_term_barrier(barrier);

	if (barrier->on_heap)
		SWIFT_FREE(barrier);
}

bool swift_init_barrier(swift_barrier_t *barrier, unsigned int size)
{
	if (!barrier)
		return false;

	if (!native_init_barrier(barrier, size))
		return false;

	swift_init_ref(&barrier->ref);

	return true;
}

void swift_term_barrier(swift_barrier_t *barrier)
{
	if (!barrier)
		return;

	swift_release_ref(&barrier->ref, swift_do_del_barrier);
}

swift_barrier_t *swift_new_barrier(unsigned int size)
{
	swift_barrier_t *barrier;

	barrier = calloc(1, sizeof(swift_barrier_t));

	if (!barrier)
		return NULL;

	barrier->on_heap = true;

	if (!swift_init_barrier(barrier, size))
		goto err_free_barrier;

	return barrier;

err_free_barrier:
	SWIFT_FREE(barrier);
	return NULL;
}

void swift_del_barrier(swift_barrier_t *barrier)
{
	if (!barrier)
		return;

	swift_release_ref(&barrier->ref, swift_do_del_barrier);
}

swift_barrier_t *swift_copy_barrier(swift_barrier_t *barrier)
{
	if (!barrier)
		return NULL;

	swift_acquire_ref(&barrier->ref);

	return barrier;
}

bool swift_barrier_wait(swift_barrier_t *barrier)
{
	if (!barrier)
		return false;

	return native_barrier_wait(barrier);
}
