#include <swift/atomic.h>
#include <swift/data.h>
#include <swift/thread.h>
#include <swift/time.h>

#include "cond.h"

static void swift_do_del_cond(swift_ref_t *ref)
{
	swift_cond_t *cond;

	if (!ref)
		return;

	cond = SWIFT_CONTAINER_OF(ref, swift_cond_t, ref);

	native_term_cond(cond);

	if (cond->on_heap)
		SWIFT_FREE(cond);
}

bool swift_init_cond(swift_cond_t *cond)
{
	if (!cond)
		return false;

	if (!native_init_cond(cond))
		return false;

	swift_init_ref(&cond->ref);

	return true;
}

void swift_term_cond(swift_cond_t *cond)
{
	if (!cond)
		return;

	swift_release_ref(&cond->ref, swift_do_del_cond);
}

swift_cond_t *swift_new_cond(void)
{
	swift_cond_t *cond;

	cond = calloc(1, sizeof(swift_cond_t));

	if (!cond)
		return NULL;

	cond->on_heap = true;

	if (!swift_init_cond(cond))
		goto err_free_cond;

	return cond;

err_free_cond:
	SWIFT_FREE(cond);
	return NULL;
}

void swift_del_cond(swift_cond_t *cond)
{
	if (!cond)
		return;

	swift_release_ref(&cond->ref, swift_do_del_cond);
}

swift_cond_t *swift_copy_cond(swift_cond_t *cond)
{
	if (!cond)
		return NULL;

	swift_acquire_ref(&cond->ref);

	return cond;
}

bool swift_cond_wait(swift_cond_t *cond, swift_mutex_t *mutex)
{
	if (!cond)
		return false;

	return native_cond_wait(cond, mutex);
}

bool swift_cond_timed_wait(swift_cond_t *cond,
	swift_mutex_t *mutex, swift_time_t time)
{
	if (!cond || time.s < 0)
		return false;

	return native_cond_timed_wait(cond, mutex, time);
}

bool swift_cond_signal(swift_cond_t *cond)
{
	if (!cond)
		return false;

	return native_cond_signal(cond);
}

bool swift_cond_broadcast(swift_cond_t *cond)
{
	if (!cond)
		return false;

	return native_cond_broadcast(cond);
}
