#include <swift/atomic.h>
#include <swift/thread.h>
#include <swift/time.h>

#include "generic/generic_barrier.h"

bool native_init_barrier(swift_barrier_t *barrier, unsigned int size)
{
	native_barrier_t *barrier_data;

	if (!barrier)
		return false;

	barrier_data = malloc(sizeof(native_barrier_t));

	if (!barrier_data)
		return false;

	barrier_data->count = 0;
	barrier_data->size = size;

	barrier_data->lock = swift_new_mutex(SWIFT_MUTEX_RECURSIVE);

	if (!barrier_data->lock)
		goto err_free_barrier_data;

	barrier_data->wait = swift_new_cond();

	if (!barrier_data->wait)
		goto err_del_lock;

	barrier->data = barrier_data;

	return true;

err_del_lock:
	swift_del_mutex(barrier_data->lock);
err_free_barrier_data:
	SWIFT_FREE(barrier_data);
	return false;
}

void native_term_barrier(swift_barrier_t *barrier)
{
	native_barrier_t *barrier_data;

	if (!barrier)
		return;

	barrier_data = barrier->data;
	swift_del_cond(barrier_data->wait);
	swift_del_mutex(barrier_data->lock);
	SWIFT_FREE(barrier_data);
}

bool native_barrier_wait(swift_barrier_t *barrier)
{
	native_barrier_t *barrier_data;

	if (!barrier)
		return false;

	barrier_data = barrier->data;

	swift_mutex_lock(barrier_data->lock);

	++barrier_data->count;

	if (barrier_data->count < barrier_data->size) {
		swift_cond_wait(barrier_data->wait, barrier_data->lock);
	} else {
		swift_cond_broadcast(barrier_data->wait);
	}

	--barrier_data->count;

	if (barrier_data->count) {
		swift_cond_wait(barrier_data->wait, barrier_data->lock);
	} else {
		swift_cond_broadcast(barrier_data->wait);
	}

	swift_mutex_unlock(barrier_data->lock);

	return true;
}
