#include <swift/atomic.h>
#include <swift/thread.h>
#include <swift/time.h>

#define WIN32_MEAN_AND_LEAN
#include <windows.h>

#include "msw/msw_mutex.h"

bool native_init_mutex(swift_mutex_t *mutex)
{
	native_mutex_t *mutex_data;

	if (!mutex)
		return false;

	mutex_data = malloc(sizeof(native_mutex_t));

	if (!mutex_data)
		return false;

	mutex_data->handle = CreateMutex(NULL, FALSE, NULL);

	if (!mutex_data->handle)
		goto err_free_mutex_data;

	mutex->data = mutex_data;

	return true;

err_free_mutex_data:
	SWIFT_FREE(mutex_data);
	return false;
}

void native_term_mutex(swift_mutex_t *mutex)
{
	native_mutex_t *mutex_data;

	if (!mutex)
		return;

	mutex_data = mutex->data;
	CloseHandle(mutex_data->handle);
	SWIFT_FREE(mutex_data);
}

bool native_mutex_lock(swift_mutex_t *mutex)
{
	native_mutex_t *mutex_data;

	if (!mutex)
		return false;

	mutex_data = mutex->data;

	return (WaitForSingleObject(mutex_data->handle, INFINITE) ==
		WAIT_OBJECT_0);
}

bool native_mutex_try_lock(swift_mutex_t *mutex)
{
	native_mutex_t *mutex_data;

	if (!mutex)
		return false;

	mutex_data = mutex->data;

	return (WaitForSingleObject(mutex_data->handle, 0) == WAIT_OBJECT_0);
}

bool native_mutex_timed_lock(swift_mutex_t *mutex, swift_time_t time_)
{
	native_mutex_t *mutex_data;
	DWORD time;

	if (!mutex)
		return false;

	mutex_data = mutex->data;
	time = time_.s * 1000 + time_.ns / 1000000;

	return (WaitForSingleObject(mutex_data->handle, time) == WAIT_OBJECT_0);
}

bool native_mutex_unlock(swift_mutex_t *mutex)
{
	native_mutex_t *mutex_data;

	if (!mutex)
		return false;

	mutex_data = mutex->data;

	return ReleaseMutex(mutex_data->handle);
}
