#include <swift/atomic.h>
#include <swift/thread.h>
#include <swift/time.h>

#define WIN32_MEAN_AND_LEAN
#include <windows.h>

#include "msw/msw_sem.h"

bool native_init_sem(swift_sem_t *sem, int count)
{
	native_sem_t *sem_data;

	if (!sem)
		return false;

	sem_data = malloc(sizeof(native_sem_t));

	if (!sem_data)
		return false;

	sem_data->handle = CreateSemaphore(NULL, count, INT_MAX, NULL);

	if (!sem_data->handle)
		goto err_free_sem_data;

	sem->data = sem_data;

	return true;

err_free_sem_data:
	SWIFT_FREE(sem_data);
	return false;
}

void native_term_sem(swift_sem_t *sem)
{
	native_sem_t *sem_data;

	if (!sem)
		return;

	sem_data = sem->data;
	CloseHandle(sem_data->handle);
	SWIFT_FREE(sem_data);
}

bool native_sem_wait(swift_sem_t *sem)
{
	native_sem_t *sem_data;

	if (!sem)
		return false;

	sem_data = sem->data;

	return (WaitForSingleObject(sem_data->handle, INFINITE) ==
		WAIT_OBJECT_0);
}

bool native_sem_try_wait(swift_sem_t *sem)
{
	native_sem_t *sem_data;

	if (!sem)
		return false;

	sem_data = sem->data;

	return (WaitForSingleObject(sem_data->handle, 0) == WAIT_OBJECT_0);
}

bool native_sem_timed_wait(swift_sem_t *sem, swift_time_t time_)
{
	native_sem_t *sem_data;
	DWORD time;

	if (!sem)
		return false;

	sem_data = sem->data;
	time = time_.s * 1000 + time_.ns / 1000000;

	return (WaitForSingleObject(sem_data->handle, time) == WAIT_OBJECT_0);
}

bool native_sem_signal(swift_sem_t *sem)
{
	native_sem_t *sem_data;

	if (!sem)
		return false;

	sem_data = sem->data;

	return ReleaseSemaphore(sem_data->handle, 1, NULL);
}
