#include <swift/atomic.h>
#include <swift/thread.h>

#define WIN32_MEAN_AND_LEAN
#include <process.h>
#include <windows.h>

#include "thread.h"

#include "msw/msw_thread.h"

static unsigned __stdcall native_thread_run(void *user_data)
{
	swift_thread_run(user_data);

	return 0;
}

bool native_init_threads(void)
{
	return true;
}

void native_term_threads(void)
{
}

swift_thread_id_t native_get_current_thread_id(void)
{
	return (swift_thread_id_t)GetCurrentThreadId();
}

void native_yield_thread(void)
{
	Sleep(0);
}

void native_exit_thread(int status)
{
	_endthreadex(status);
}

bool native_init_thread(swift_thread_t *thread, void *user_data,
	size_t stack_size, unsigned int flags)
{
	native_thread_t *thread_data;

	if (!thread)
		return false;

	thread_data = malloc(sizeof(native_thread_t));

	if (!thread_data)
		return false;

	thread_data->handle = (HANDLE)_beginthreadex(NULL,
		(unsigned int)stack_size, native_thread_run, user_data, 0,
		(unsigned int *)&thread->thread_id);

	if (!thread_data->handle)
		goto err_free_thread_data;

	thread->data = thread_data;

	return true;

err_free_thread_data:
	SWIFT_FREE(thread_data);
	return false;
}

bool native_init_current_thread(swift_thread_t *thread)
{
	native_thread_t *thread_data;

	if (!thread)
		return false;

	thread_data = malloc(sizeof(native_thread_t));

	if (!thread_data)
		return false;

	thread_data->handle = GetCurrentThread();

	return true;
}

void native_term_thread(swift_thread_t *thread)
{
	native_thread_t *thread_data;

	if (!thread)
		return;

	thread_data = thread->data;

	if (!(thread->flags & SWIFT_THREAD_FOREIGN))
		CloseHandle(thread_data->handle);

	SWIFT_FREE(thread_data);
}

bool native_thread_setup(swift_thread_t *thread)
{
	return true;
}

bool native_thread_wait(swift_thread_t *thread)
{
	native_thread_t *thread_data;

	if (!thread)
		return false;

	thread_data = thread->data;

	return (WaitForSingleObject(thread_data->handle, INFINITE) ==
		WAIT_OBJECT_0);
}

bool native_thread_cancel(swift_thread_t *thread)
{
	native_thread_t *thread_data;

	if (!thread)
		return false;

	thread_data = thread->data;

	return TerminateThread(thread_data->handle, 0);
}

bool native_thread_set_prio(swift_thread_t *thread, unsigned int prio_)
{
	native_thread_t *thread_data;
	DWORD prio;

	if (!thread)
		return false;

	thread_data = thread->data;

	if (prio_ <= 20) {
		prio = THREAD_PRIORITY_LOWEST;
	} else if (prio_ <= 40) {
		prio = THREAD_PRIORITY_BELOW_NORMAL;
	} else if (prio_ <= 60) {
		prio = THREAD_PRIORITY_NORMAL;
	} else if (prio_ <= 80) {
		prio = THREAD_PRIORITY_ABOVE_NORMAL;
	} else {
		prio = THREAD_PRIORITY_HIGHEST;
	}

	return SetThreadPriority(thread_data->handle, prio);
}
