#include <swift/atomic.h>
#include <swift/thread.h>
#include <swift/time.h>

#include <errno.h>
#include <pthread.h>

#include "posix/posix_barrier.h"

bool native_init_barrier(swift_barrier_t *barrier, unsigned int size)
{
	native_barrier_t *barrier_data;

	if (!barrier)
		return false;

	barrier_data = malloc(sizeof(native_barrier_t));

	if (!barrier_data)
		return false;

	if (pthread_barrier_init(&barrier_data->handle, NULL, size) < 0)
		goto err_free_barrier_data;

	barrier->data = barrier_data;

	return true;

err_free_barrier_data:
	SWIFT_FREE(barrier_data);
	return false;
}

void native_term_barrier(swift_barrier_t *barrier)
{
	native_barrier_t *barrier_data;

	if (!barrier)
		return;

	barrier_data = barrier->data;
	pthread_barrier_destroy(&barrier_data->handle);
	SWIFT_FREE(barrier_data);
}

bool native_barrier_wait(swift_barrier_t *barrier)
{
	native_barrier_t *barrier_data;
	int ret;

	if (!barrier)
		return false;

	barrier_data = barrier->data;

	do {
		ret = pthread_barrier_wait(&barrier_data->handle);
	} while (ret == EINTR);

	return (ret == 0 || ret == PTHREAD_BARRIER_SERIAL_THREAD);
}
