#include <swift/atomic.h>
#include <swift/thread.h>
#include <swift/time.h>

#include <errno.h>
#include <pthread.h>
#include <semaphore.h>

#include "posix/posix_sem.h"

bool native_init_sem(swift_sem_t *sem, int count)
{
	native_sem_t *sem_data;

	if (!sem)
		return false;

	sem_data = malloc(sizeof(native_sem_t));

	if (!sem_data)
		return false;

	if (sem_init(&sem_data->handle, 0, count) < 0)
		goto err_free_sem_data;

	sem->data = sem_data;

	return true;

err_free_sem_data:
	SWIFT_FREE(sem_data);
	return false;
}

void native_term_sem(swift_sem_t *sem)
{
	native_sem_t *sem_data;

	if (!sem)
		return;

	sem_data = sem->data;
	sem_destroy(&sem_data->handle);
	SWIFT_FREE(sem_data);
}

bool native_sem_wait(swift_sem_t *sem)
{
	native_sem_t *sem_data;
	int ret;

	if (!sem)
		return false;

	sem_data = sem->data;

	do {
		ret = sem_wait(&sem_data->handle);
	} while (ret == EINTR);

	return (ret == 0);
}

bool native_sem_try_wait(swift_sem_t *sem)
{
	native_sem_t *sem_data;
	int ret;

	if (!sem)
		return false;

	sem_data = sem->data;

	do {
		ret = sem_trywait(&sem_data->handle);
	} while (ret == EINTR);

	return (ret == 0);
}

bool native_sem_timed_wait(swift_sem_t *sem, swift_time_t time_)
{
	struct timespec time;
	native_sem_t *sem_data;
	int ret;

	if (!sem)
		return false;

	sem_data = sem->data;

	time_ = swift_add_time(swift_get_monotonic_time(), time_);
	time.tv_sec = time_.s;
	time.tv_nsec = time_.ns;

	do {
		ret = sem_timedwait(&sem_data->handle, &time);
	} while (ret == EINTR);

	return (ret == 0);
}

bool native_sem_signal(swift_sem_t *sem)
{
	native_sem_t *sem_data;

	if (!sem)
		return false;

	sem_data = sem->data;

	return (sem_post(&sem_data->handle) == 0);
}
