#include <swift/time.h>

#define WIN32_MEAN_AND_LEAN
#include <windows.h>

#define UNIX_EPOCH (UINT64_C(116444736000000000))

static uint64_t swift_raw_time(void)
{
	uint64_t time;

	if (!QueryPerformanceCounter((LARGE_INTEGER*) &time))
		return 0;

	return time;
}

double swift_tick_count(void)
{
	static uint64_t base = 0;
	static double res = 0;

	if (res == 0) {
		uint64_t freq;

		if (QueryPerformanceFrequency((LARGE_INTEGER*) &freq))
			res = 1.0 / freq;
		else
			res = 0.001; // winmm resolution is 1 ms

		base = swift_raw_time();
	}

	return (double) (swift_raw_time() - base) * res;
}

bool swift_has_monotonic_time(void)
{
	LARGE_INTEGER ticks;

	return (QueryPerformanceCounter(&ticks) &&
		QueryPerformanceFrequency(&ticks));
}

swift_time_t swift_get_monotonic_res(void)
{
	LARGE_INTEGER freq;

	if (!QueryPerformanceFrequency(&freq))
		return SWIFT_TIME();

	return SWIFT_TIME(
		1 / freq.QuadPart,
		(1000000000 / freq.QuadPart) % 1000000000,
	);
}

swift_time_t swift_get_monotonic_time(void)
{
	LARGE_INTEGER ticks, freq;

	if (!QueryPerformanceFrequency(&freq))
		return SWIFT_TIME();

	if (!QueryPerformanceCounter(&ticks))
		return SWIFT_TIME();

	return SWIFT_TIME(
		ticks.QuadPart / freq.QuadPart,
		(1000000000 * ticks.QuadPart / freq.QuadPart) % 1000000000
	);
}

swift_time_t swift_get_calendar_res(void)
{
	return SWIFT_TIME(0, 100);
}

swift_time_t swift_get_calendar_time(void)
{
	FILETIME system_time;
	ULARGE_INTEGER time;

	GetSystemTimeAsFileTime(&system_time);

	time.HighPart = system_time.dwHighDateTime;
	time.LowPart = system_time.dwLowDateTime;
	time.QuadPart = time.QuadPart - UNIX_EPOCH;

	return SWIFT_TIME(time.QuadPart / 10000000, 100 * time.QuadPart);
}

void swift_timed_wait(swift_time_t time)
{
	Sleep(time.s * 1000 + time.ns / 1000000);
}
