#include <swift/unicode.h>

size_t swift_enc_utf16(void *dst_, uint32_t code)
{
	size_t dst_size;
	uint16_t *dst = (uint16_t *)dst_;

	if (code > 0x10FFFF)
		code = SWIFT_UNICODE_UNKNOWN;

	dst_size = (code < 0x10000) ? 1 : 2;

	if (!dst)
		return dst_size * sizeof(uint16_t);

	if (code < 0x10000) {
		*dst++ = code;
	} else {
		*dst++ = 0xD800 | ((code >> 10) & 0x3FF);
		*dst++ = 0xDC00 | (code & 0x3FF);
	}

	return dst_size * sizeof(uint16_t);
}

swift_unicode_ret_t swift_dec_utf16(const void *src_, size_t src_size)
{
	size_t dst_size = 0;
	uint16_t *src = (uint16_t *)src_;
	uint16_t w1, w2;

	if (src_size < sizeof(uint16_t))
		goto err_illegal_seq;

	w1 = *src++;
	dst_size += sizeof(uint16_t);
	src_size -= sizeof(uint16_t);

	if (w1 < 0xD800 || w1 > 0xDFFF)
		return SWIFT_UNICODE_RET(dst_size, w1, true);

	if (w1 > 0xDBFF)
		goto err_illegal_seq;

	if (src_size < sizeof(uint16_t))
		goto err_illegal_seq;

	w2 = *src++;
	dst_size += sizeof(uint16_t);
	src_size -= sizeof(uint16_t);

	if (w2 < 0xDC00 || w2 > 0xDFFF)
		goto err_illegal_seq;

	return SWIFT_UNICODE_RET(dst_size, (((w1 & 0x3FF) << 10) |
		(w2 & 0x3FF)) + 0x10000, true);

err_illegal_seq:
	return SWIFT_UNICODE_RET(dst_size, SWIFT_UNICODE_UNKNOWN, false);
}
