#include <swift/unicode.h>

#define SWIFT_UTF8_SUCCESS 0
#define SWIFT_UTF8_FAILURE 12

/* The table that maps bytes to character classes. */
static const uint8_t utf8_type_table[] = {
	 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
	 9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,
	10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
	11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
	11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
	 1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,
	 2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,
	 7,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  4,  3,  3,
	 8,  6,  6,  6,  5,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
};

/* The table that maps the current state to the next one, based on the character
 * class.
 */
static const uint8_t utf8_state_table[] = {
	 0, 12, 24, 36, 48, 60, 72, 84, 96, 12, 12, 12,
	12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
	12, 12, 12, 12, 12, 12, 12, 12, 12,  0,  0,  0,
	12, 12, 12, 12, 12, 12, 12, 12, 12, 24, 24, 24,
	12, 12, 12, 12, 12, 12, 12, 12, 12, 24, 24, 12,
	12, 12, 12, 12, 12, 12, 12, 12, 12, 36, 12, 12,
	12, 12, 12, 12, 12, 12, 12, 12, 12, 36, 36, 36,
	12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 24,
	12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 36, 36,
};

size_t swift_enc_utf8(void *dst_, swift_uchar_t code)
{
	size_t dst_size, bits;
	uint8_t *dst = (uint8_t *)dst_;
	uint8_t mask;

	if (code > 0x10FFFF)
		code = SWIFT_UNICODE_UNKNOWN;

	if (code <= 0x7F) {
		dst_size = 1;
	} else if (code <= 0x7FF) {
		dst_size = 2;
	} else if (code <= 0xFFFF) {
		dst_size = 3;
	} else {
		dst_size = 4;
	}

	if (!dst)
		return dst_size * sizeof(uint8_t);

	bits = (dst_size - 1) * 6;
	mask = (dst_size == 1) ? 0x00 : (0xFF << (8 - dst_size));
	*dst++ = mask | ((code >> bits) & (~mask >> 1));

	while (bits) {
		bits -= 6;
		*dst++ = 0x80 | ((code >> bits) & 0x3F);
	}

	return dst_size * sizeof(uint8_t);
}

swift_unicode_ret_t swift_dec_utf8(const void *src_, size_t src_size)
{
	size_t dst_size = 0;
	uint8_t *src = (uint8_t *)src_;
	swift_uchar_t code;
	uint8_t byte, type, state = SWIFT_UTF8_SUCCESS;

	while (src_size >= sizeof(uint8_t)) {
		byte = *src++;
		dst_size += sizeof(uint8_t);
		src_size -= sizeof(uint8_t);

		type = utf8_type_table[byte];
		code = (state == SWIFT_UTF8_SUCCESS) ? (0xFF >> type) & byte :
			(byte & 0x3F) | (code << 6);
		state = utf8_state_table[state + type];

		if (state == SWIFT_UTF8_SUCCESS)
			return SWIFT_UNICODE_RET(dst_size, code, true);

		if (state == SWIFT_UTF8_FAILURE)
			break;
	} while (state != SWIFT_UTF8_SUCCESS && state != SWIFT_UTF8_FAILURE);

	return SWIFT_UNICODE_RET(dst_size, SWIFT_UNICODE_UNKNOWN, false);
}
