#include <swift/window.h>
#include <swift/io.h>
#include "callbacks.h"

bool swift_create_callbacks(swift_window_t *window)
{
	if (!window || !(window->callbacks = calloc(1, sizeof(swift_intern_cb_t))))
		return false;

	return true;
}

void swift_destroy_callbacks(swift_window_t *window)
{
	if (window && window->callbacks)
		SWIFT_FREE(window->callbacks);
}

void swift_callback_char(GLFWwindow *screen, unsigned int key)
{
	swift_window_t *window = glfwGetWindowUserPointer(screen);

	if (window && window->callbacks) {
		swift_intern_cb_t *cbs = window->callbacks;

		if (cbs->chars)
			cbs->chars(window, key);
	}
}

void swift_callback_mods(GLFWwindow *screen, unsigned int key, int mods)
{
	swift_window_t *window = glfwGetWindowUserPointer(screen);

	if (window && window->callbacks) {
		swift_intern_cb_t *cbs = window->callbacks;

		if (cbs->mods)
			cbs->mods(window, key, mods);
	}
}

void swift_callback_key(GLFWwindow *screen, int key, int scancode, int action,
	int mods)
{
	swift_window_t *window = glfwGetWindowUserPointer(screen);

	if (window && window->callbacks) {
		swift_intern_cb_t *cbs = window->callbacks;

		if (cbs->keys)
			cbs->keys(window, key, scancode, action, mods);
	}
}

void swift_callback_cursor_pos(GLFWwindow *screen, double x, double y)
{
	swift_window_t *window = glfwGetWindowUserPointer(screen);

	if (window && window->callbacks) {
		swift_intern_cb_t *cbs = window->callbacks;

		if (cbs->cursor_pos)
			cbs->cursor_pos(window, x, y);
	}
}

void swift_callback_mouse_press(GLFWwindow *screen, int button, int pressed,
	int mods)
{
	swift_window_t *window = glfwGetWindowUserPointer(screen);

	if (window && window->callbacks) {
		swift_intern_cb_t *cbs = window->callbacks;

		if (cbs->mouse_button)
			cbs->mouse_button(window, button, pressed, mods);
	}
}

void swift_callback_scroll(GLFWwindow *screen, double x, double y)
{
	swift_window_t *window = glfwGetWindowUserPointer(screen);

	if (window && window->callbacks) {
		swift_intern_cb_t *cbs = window->callbacks;

		if (cbs->scroll)
			cbs->scroll(window, x, y);
	}
}

void swift_callback_window_close(GLFWwindow *screen)
{
	swift_window_t *window = glfwGetWindowUserPointer(screen);

	if (window && window->callbacks) {
		swift_intern_cb_t *cbs = window->callbacks;

		if (cbs->win_close)
			cbs->win_close(window);
	}
}

void swift_callback_window_pos(GLFWwindow *screen, int x, int y)
{
	swift_window_t *window = glfwGetWindowUserPointer(screen);

	if (window && window->callbacks) {
		swift_intern_cb_t *cbs = window->callbacks;

		if (cbs->win_pos)
			cbs->win_pos(window, x, y);
	}
}

void swift_callback_window_resize(GLFWwindow *screen, int width, int height)
{
	swift_window_t *window = glfwGetWindowUserPointer(screen);

	if (window && window->callbacks) {
		swift_intern_cb_t *cbs = window->callbacks;

		if (cbs->win_resize)
			cbs->win_resize(window, width, height);
		else
			swift_handle_window_resize(window, width, height);
	}
}

void swift_callback_window_refresh(GLFWwindow *screen)
{
	swift_window_t *window = glfwGetWindowUserPointer(screen);

	if (window && window->callbacks) {
		swift_intern_cb_t *cbs = window->callbacks;

		if (cbs->win_refresh)
			cbs->win_refresh(window);
	}
}

void swift_callback_window_focus(GLFWwindow *screen, int focused)
{
	swift_window_t *window = glfwGetWindowUserPointer(screen);

	if (window && window->callbacks) {
		swift_intern_cb_t *cbs = window->callbacks;

		if (cbs->win_focus)
			cbs->win_focus(window, focused);
	}
}

void swift_callback_iconify(GLFWwindow *screen, int iconified)
{
	swift_window_t *window = glfwGetWindowUserPointer(screen);

	if (window && window->callbacks) {
		swift_intern_cb_t *cbs = window->callbacks;

		if (cbs->iconify)
			cbs->iconify(window, iconified);
	}
}

void swift_callback_buffer_resize(GLFWwindow *screen, int width, int height)
{
	swift_window_t *window = glfwGetWindowUserPointer(screen);

	if (window && window->callbacks) {
		swift_intern_cb_t *cbs = window->callbacks;

		if (cbs->buffer_resize)
			cbs->buffer_resize(window, width, height);
	}
}

void swift_callback_cursor_entered(GLFWwindow *screen, int entered)
{
	swift_window_t *window = glfwGetWindowUserPointer(screen);

	if (window && window->callbacks) {
		swift_intern_cb_t *cbs = window->callbacks;

		if (cbs->cursor_enter)
			cbs->cursor_enter(window, entered);
	}
}

void swift_callback_drops(GLFWwindow *screen, int count, const char **paths)
{
	swift_window_t *window = glfwGetWindowUserPointer(screen);

	if (window && window->callbacks) {
		swift_intern_cb_t *cbs = window->callbacks;

		if (cbs->drops)
			cbs->drops(window, count, paths);
	}
}

void swift_set_char_callback(swift_window_t *window, swift_char_fn_t callback)
{
	if (window && window->callbacks && callback) {
		swift_intern_cb_t *cbs = window->callbacks;

		cbs->chars = callback;
	}
}

void swift_set_mods_callback(swift_window_t *window, swift_mods_fn_t callback)
{
	if (window && window->callbacks && callback) {
		swift_intern_cb_t *cbs = window->callbacks;

		cbs->mods = callback;
	}
}

void swift_set_key_callback(swift_window_t *window, swift_key_fn_t callback)
{
	if (window && window->callbacks && callback) {
		swift_intern_cb_t *cbs = window->callbacks;

		cbs->keys = callback;
	}
}

void swift_set_cursor_pos_callback(swift_window_t *window,
	swift_cur_pos_fn_t callback)
{
	if (window && window->callbacks && callback) {
		swift_intern_cb_t *cbs = window->callbacks;

		cbs->cursor_pos = callback;
	}
}

void swift_set_mouse_press_callback(swift_window_t *window,
	swift_mouse_btn_fn_t callback)
{
	if (window && window->callbacks && callback) {
		swift_intern_cb_t *cbs = window->callbacks;

		cbs->mouse_button = callback;
	}
}

void swift_set_scroll_callback(swift_window_t *window,
	swift_scroll_fn_t callback)
{
	if (window && window->callbacks && callback) {
		swift_intern_cb_t *cbs = window->callbacks;

		cbs->scroll = callback;
	}
}

void swift_set_window_close_callback(swift_window_t *window,
	swift_win_close_fn_t callback)
{
	if (window && window->callbacks && callback) {
		swift_intern_cb_t *cbs = window->callbacks;

		cbs->win_close = callback;
	}
}

void swift_set_window_pos_callback(swift_window_t *window,
	swift_win_pos_fn_t callback)
{
	if (window && window->callbacks && callback) {
		swift_intern_cb_t *cbs = window->callbacks;

		cbs->win_pos = callback;
	}
}

void swift_set_window_resize_callback(swift_window_t *window,
	swift_win_resize_fn_t callback)
{
	if (window && window->callbacks && callback) {
		swift_intern_cb_t *cbs = window->callbacks;

		cbs->win_resize = callback;
	}
}

void swift_set_window_refresh_callback(swift_window_t *window,
	swift_win_refresh_fn_t callback)
{
	if (window && window->callbacks && callback) {
		swift_intern_cb_t *cbs = window->callbacks;

		cbs->win_refresh = callback;
	}
}

void swift_set_window_focus_callback(swift_window_t *window,
	swift_win_focus_fn_t callback)
{
	if (window && window->callbacks && callback) {
		swift_intern_cb_t *cbs = window->callbacks;

		cbs->win_focus = callback;
	}
}

void swift_set_iconify_callback(swift_window_t *window,
	swift_iconify_fn_t callback)
{
	if (window && window->callbacks && callback) {
		swift_intern_cb_t *cbs = window->callbacks;

		cbs->iconify = callback;
	}
}

void swift_set_buffer_resize_callback(swift_window_t *window,
	swift_buf_resize_fn_t callback)
{
	if (window && window->callbacks && callback) {
		swift_intern_cb_t *cbs = window->callbacks;

		cbs->buffer_resize = callback;
	}
}

void swift_set_cursor_entered_callback(swift_window_t *window,
	swift_cur_enter_fn_t callback)
{
	if (window && window->callbacks && callback) {
		swift_intern_cb_t *cbs = window->callbacks;

		cbs->cursor_enter = callback;
	}
}

void swift_set_drops_callback(swift_window_t *window,
	swift_drop_fn_t callback)
{
	if (window && window->callbacks && callback) {
		swift_intern_cb_t *cbs = window->callbacks;

		cbs->drops = callback;
	}
}
